package com.dayou.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.dayou.common.DropDownBoxData;
import com.dayou.dto.calculate.CalculateBaseInfoDTO;
import com.dayou.result.Result;
import com.dayou.result.Status;
import com.dayou.vo.AssetsCalculateVO;
import com.dayou.vo.CRReportReferencesCalculateVO;
import com.dayou.vo.EqptReportReferencesCalculateVO;
import com.dayou.vo.calculate.AssetsCalculateProgressVO;
import com.fasterxml.jackson.core.JsonProcessingException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.AssetsCalculateService;
import com.dayou.entity.AssetsCalculate;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;

/**
 * 资产项目测算信息表
 *
 * @author mybatis-plus-generator
 * @since 2024-11-13
 */
@RestController
@RequestMapping("assetsCalculate")
@Slf4j
@CrossOrigin
@SaCheckLogin
public class AssetsCalculateController {

    @Autowired
    private AssetsCalculateService assetsCalculateService;

    /**
    * 资产项目测算信息表列表
    */
    @GetMapping("/listAllByProjectId/{projectId}")
    public Result<List<AssetsCalculateVO>> listAllByProjectId(@PathVariable Long projectId){
        List<AssetsCalculateVO> pages = assetsCalculateService.listAllByProjectId(projectId);
        return Result.build(pages);
    }

    /**
     * 资产项目测算信息表详情
     */
    @GetMapping("/detail/{id}")
    public Result<AssetsCalculate> detail(@PathVariable Long id){
        AssetsCalculate xAssetsCalculate =assetsCalculateService.detail(id);
        return Result.build(xAssetsCalculate);
     }

    /**
     * 资产项目测算信息表新增
     */
    @PostMapping("/save")
    public Result<Boolean> save(@RequestBody AssetsCalculate assetsCalculate) {
        Boolean ret = assetsCalculateService.add(assetsCalculate);
        return Result.build(ret);
    }

    /**
     * 资产项目测算信息表更新
     */
    @PutMapping("/update")
    public Result<Boolean> update(@RequestBody AssetsCalculate assetsCalculate) {
        Boolean ret = assetsCalculateService.update(assetsCalculate);
        return Result.build(ret);
    }

    /**
     * 资产项目测算信息表删除
     */
    @DeleteMapping("/delete/{id}")
    public Result<Boolean> delete(@PathVariable Long id) {
        Boolean ret = assetsCalculateService.delete(id);
        return Result.build(ret);
    }

    /**
     * 根据项目id获取未完成测算表的进度信息
     * @param projectId 项目id
     * @return AssetsCalculateStepBaseVO
     */
    @GetMapping("/getUnFinishedCalculateProgress/{projectId}")
    public Result<AssetsCalculateProgressVO> getUnFinishedCalculateProgress(@PathVariable Long projectId){
        return Result.build(assetsCalculateService.getUnFinishedCalculateProgress(projectId));
    }

    /**
     * 导入保存机器设备基础信息
     * @param multipartFile Excel文件
     * @param calculateId 测算表id
     */
    @PostMapping("/importEqptExcelData")
    public Result importEqptExcelData(MultipartFile multipartFile, Long calculateId) throws IOException {
        assetsCalculateService.importEqptExcelData(multipartFile, calculateId);
        return Result.build(Status.SUCCESS);
    }

    /**
     * 更新测算表基础测算信息(存的json格式字符串)
     * @param calculateBaseInfoDTO dto
     * @return Boolean
     */
    @PutMapping("/updateCalculateBaseInfo")
    public Result<Boolean> updateCalculateBaseInfo(@RequestBody CalculateBaseInfoDTO calculateBaseInfoDTO) {
        return Result.build(assetsCalculateService.updateCalculateBaseInfo(calculateBaseInfoDTO.getCalculateId(), calculateBaseInfoDTO.getBaseInfo()));
    }

    /**
     * 生成测算表
     * @param calculateId 测算表id
     * @return Boolean
     * @throws IOException
     */
    @GetMapping("/generateEquipmentCalculate/{calculateId}")
    public Result<Boolean> generateEquipmentCalculate(@PathVariable Long calculateId) throws IOException {
        return Result.build(assetsCalculateService.generateEquipmentCalculate(calculateId));
    }

    /**
     * 根据测算表id获取基础测算信息
     * @param calculateId 测算表id
     * @return String
     */
    @GetMapping("/getCalculateBaseInfo/{calculateId}")
    public Result<String> getCalculateBaseInfo(@PathVariable Long calculateId) {
        return Result.build(assetsCalculateService.getCalculateBaseInfo(calculateId));
    }

    /**
     * 根据测算表id获取评估基准日
     * @param calculateId 测算表id
     * @return String
     */
    @GetMapping("/getValuationBasisDate/{calculateId}")
    public Result<LocalDate> getValuationBasisDate(@PathVariable Long calculateId) {
        return Result.build(assetsCalculateService.getValuationBasisDate(calculateId));
    }

    /**
     * 根据项目id获取测算信息下拉框数据
     * @param projectId 项目id
     * @return List<DropDownBoxData>
     */
    @GetMapping("/getDropDownBoxData/{projectId}")
    public Result<List<DropDownBoxData>> getDropDownBoxData(@PathVariable Long projectId) {
        return Result.build(assetsCalculateService.getDropDownBoxData(projectId));
    }

    /**
     * 根据测算表id获取生成机器设备融资项目报告引用的测算表信息
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    @GetMapping("/eqpt/getReferencesInfoById/{calculateId}")
    public Result<EqptReportReferencesCalculateVO> getEqptReferencesInfoById(@PathVariable Long calculateId) {
       return Result.build(assetsCalculateService.getEqptReferencesInfoById(calculateId));
    }

    /**
     * 根据测算表id获取生成债权分析报告引用的测算表信息
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    @GetMapping("/cr/getReferencesInfoById/{calculateId}")
    public Result<CRReportReferencesCalculateVO> getCRReferencesInfoById (@PathVariable Long calculateId) {
        return Result.build(assetsCalculateService.getCRReferencesInfoById(calculateId));
    }


}

