package com.dayou.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.dayou.dto.calculate.equipment.EqptQueryDTO;
import com.dayou.entity.AssetsCalculateEqptData;
import com.dayou.entity.AssetsCalculateEqptImpData;
import com.dayou.entity.AssetsCalculateEqptNonsData;
import com.dayou.result.Result;
import com.dayou.service.AssetsCalculateEqptDataService;
import com.dayou.vo.AssetsCalculateEqptImpDataVO;
import com.dayou.vo.AssetsCalculateEqptNonsDataVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 资产机器设备测算数据
 *
 * @author GouGengquan
 * @since 2024-11-18
 */
@RestController
@RequestMapping("assetsCalculateEqptData")
@Slf4j
@CrossOrigin
@SaCheckLogin
public class AssetsCalculateEqptDataController {

    @Autowired
    private AssetsCalculateEqptDataService dataService;

    /**
     * 新增机器设备信息
     * @param assetsCalculateEqptData 设备信息
     * @return Boolean
     */
    @PostMapping("/addEqptData")
    public Result<Boolean> addEqptData(@RequestBody AssetsCalculateEqptData assetsCalculateEqptData) {
        return Result.build(dataService.addEqptData(assetsCalculateEqptData));
    }

    /**
     * 获取是否有进口设备
     * @param calculateId 测算表id
     * @return Result<Boolean>
     */
    @GetMapping("/hasImportedEquipment/{calculateId}")
    public Result<Boolean> hasImportedEquipment(@PathVariable Long calculateId) {
        return Result.build(dataService.hasImportedEquipment(calculateId));
    }

    /**
     * 分页查询导入的机器设备测算数据
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptData>
     */
    @GetMapping("/getDataPage")
    public Result<List<AssetsCalculateEqptData>> getDataPage(EqptQueryDTO dto) {
        return Result.build(dataService.getDataPage(dto));
    }

    /**
     * 更新机器设备数据
     * @param assetsCalculateEqptData 机器设备信息
     * @return Boolean
     */
    @PutMapping("/update")
    public Result<Boolean> update(@RequestBody AssetsCalculateEqptData assetsCalculateEqptData){
        return Result.build(dataService.updateData(assetsCalculateEqptData));
    }


    /**
     * 批量更新
     * @param dataList 测算信息集合
     * @return Boolean
     */
    @PutMapping("/updateBatch")
    public Result<Boolean> updateBatch(@RequestBody List<AssetsCalculateEqptData> dataList) {
        return Result.build(dataService.updateBatch(dataList));
    }

    /**
     * 分页查询进口设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptImpData>
     */
    @GetMapping("/getImpDataPage")
    public Result<List<AssetsCalculateEqptImpDataVO>> getImpDataPage(EqptQueryDTO dto) {
        return Result.build(dataService.getImpDataPage(dto));
    }

    /**
     * 更新机器设备进口设备数据
     * @param assetsCalculateEqptImpData 进口设备信息
     * @return Boolean
     */
    @PutMapping("/updateImpData")
    public Result<Boolean> updateImpData(@RequestBody AssetsCalculateEqptImpData assetsCalculateEqptImpData) {
       return Result.build(dataService.updateImpData(assetsCalculateEqptImpData));
    }

    /**
     * 批量更新机器设备进口设备数据
     * @param dataList 进口设备信息
     * @return Boolean
     */
    @PutMapping("/updateBatchImpData")
    public Result<Boolean> updateBatchImpData(@RequestBody List<AssetsCalculateEqptImpData> dataList) {
        return Result.build(dataService.updateBatchImpData(dataList));
    }

    /**
     * 分页查询非标设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    @GetMapping("/getNonSDataPage")
    public Result<List<AssetsCalculateEqptNonsDataVO>> getNonSDataPage(EqptQueryDTO dto) {
        return Result.build(dataService.getNonSDataPage(dto));
    }

    /**
     * 更新机器设备非标设备数据
     * @param assetsCalculateEqptNonsData 非标设备信息
     * @return Boolean
     */
    @PutMapping("/updateNonSData")
    public Result<Boolean> updateNonSData(@RequestBody AssetsCalculateEqptNonsData assetsCalculateEqptNonsData) {
        return Result.build(dataService.updateNonSData(assetsCalculateEqptNonsData));
    }

    /**
     * 批量更新机器设备非标设备数据
     * @param dataList 非标设备信息
     * @return Boolean
     */
    @PutMapping("/updateBatchNonSData")
    public Result<Boolean> updateBatchNonSData(@RequestBody List<AssetsCalculateEqptNonsData> dataList) {
        return Result.build(dataService.updateBatchNonSData(dataList));
    }

    /**
     * 根据机器设备信息id删除
     * @param eqptId 设备信息id
     * @return Boolean
     */
    @DeleteMapping("/deleteEqptDataById/{eqptId}")
    public Result<Boolean> deleteEqptDataById(@PathVariable Long eqptId){
        return Result.build(dataService.deleteDqptDataById(eqptId));
    }

}
