package com.dayou.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import com.dayou.dto.report.ReportBaseInfoDTO;
import com.dayou.result.Result;
import com.dayou.vo.AssetsReportVO;
import com.dayou.vo.report.AssetsReportProgressVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.AssetsReportService;
import com.dayou.entity.AssetsReport;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;

/**
 * 资产项目报告信息表
 *
 * @author mybatis-plus-generator
 * @since 2024-12-06
 */
@RestController
@RequestMapping("assetsReport")
@Slf4j
@SaCheckLogin
@CrossOrigin
public class AssetsReportController {

    @Autowired
    private AssetsReportService assetsReportService;

    /**
     * 根据项目id查询所有报告信息
     * @param projectId 项目id
     * @return List<AssetsReport>
     */
    @GetMapping("/listAllByProjectId/{projectId}")
    public Result<List<AssetsReportVO>> listAllByProjectId(@PathVariable Long projectId) {
        List<AssetsReportVO> pages = assetsReportService.listAllByProjectId(projectId);
        return Result.build(pages);
    }

    /**
     * 资产项目报告信息表详情
     */
    @GetMapping("/detail/{id}")
    public Result<AssetsReport> detail(@PathVariable Long id) {
        AssetsReport xAssetsReport = assetsReportService.detail(id);
        return Result.build(xAssetsReport);
    }

    /**
     * 资产项目报告信息表新增
     */
    @PostMapping("/save")
    public Result<Boolean> save(@RequestBody AssetsReport assetsReport) {
        Boolean ret = assetsReportService.add(assetsReport);
        return Result.build(ret);
    }

    /**
     * 资产项目报告信息表更新
     */
    @PutMapping("/update")
    public Result<Boolean> update(@RequestBody AssetsReport assetsReport) {
        Boolean ret = assetsReportService.update(assetsReport);
        return Result.build(ret);
    }

    /**
     * 资产项目报告信息表删除
     */
    @DeleteMapping("/delete/{id}")
    public Result<Boolean> delete(@PathVariable Long id) {
        Boolean ret = assetsReportService.delete(id);
        return Result.build(ret);
    }

    /**
     * 根据项目id获取未完成报告的进度信息
     * @param projectId 项目id
     * @return AssetsReportProgressVO
     */
    @GetMapping("/getUnFinishedReportProgress/{projectId}")
    public Result<AssetsReportProgressVO> getUnFinishedReportProgress(@PathVariable Long projectId) {
        return Result.build(assetsReportService.getUnFinishedReportProgress(projectId));
    }

    /**
     * 更新报告基础信息
     * @param reportBaseInfoDTO dto
     * @return Boolean
     */
    @PutMapping("/updateReportBaseInfo")
    public Result<Boolean> updateReportBaseInfo(@RequestBody ReportBaseInfoDTO reportBaseInfoDTO) {
        return Result.build(assetsReportService.updateReportBaseInfo(reportBaseInfoDTO));
    }

    /**
     * 根据报告id获取报告基础信息
     * @param reportId 报告id
     * @return String
     */
    @GetMapping("/getReportBaseInfo/{reportId}")
    public Result<String> getReportBaseInfo(@PathVariable Long reportId) {
        return Result.build(assetsReportService.getReportBaseInfo(reportId));
    }

    /**
     * 生成机器设备报告word文件
     * @param reportId 报告id
     * @return Boolean
     */
    @GetMapping("/generateEquipmentReport/{reportId}")
    public Result<Boolean> generateEquipmentReport(@PathVariable Long reportId) throws Exception {
        return Result.build(assetsReportService.generateEquipmentReport(reportId));
    }

    /**
     * 生成债权分析报告word文件
     * @param reportId 报告id
     * @return Boolean
     */
    @GetMapping("/generateCRReport/{reportId}")
    public Result<Boolean> generateCRReport(@PathVariable Long reportId) throws Exception {
        return Result.build(assetsReportService.generateCRReport(reportId));
    }

}

