package com.dayou.controller;

import cn.dev33.satoken.annotation.SaCheckLogin;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjectUtil;
import com.dayou.result.Result;
import com.dayou.result.Status;
import com.dayou.service.AuthService;
import com.dayou.vo.UserVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/auth")
@Slf4j
@CrossOrigin
public class AuthController {

    @Autowired
    private AuthService authService;

    /**
     * 通过oa系统的信息登录到生产力平台，并同步用户信息
     * @param oaToken oa系统token
     * @return Result<UserVO>
     */
    @PostMapping("/loginByOAInfo")
    public Result<UserVO> loginByOAInfo(String oaToken){
        UserVO userVO = authService.loginByOAInfo(oaToken);
        if (ObjectUtil.isNotNull(userVO)){
            return Result.build(Status.SUCCESS, userVO);
        }else {
            return Result.build(Status.NOT_LOGIN);
        }
    }

    /**
     * 退出登录
     */
    @SaCheckLogin
    @PostMapping("/logout")
    public Result<String> logout(){
        StpUtil.logout();
        return Result.build(Status.SUCCESS);
    }

}
