package com.dayou.controller;

import cn.hutool.core.lang.UUID;
import com.dayou.config.FileNetConfig;
import com.dayou.entity.TmplAssetCalculate;
import com.dayou.entity.TmplAssetCalculateSection;
import com.dayou.exception.BusinessException;
import com.dayou.result.Result;
import com.dayou.result.Status;
import com.dayou.service.DocumentProductionService;
import com.dayou.service.HouseExcelDataService;
import com.dayou.service.TmplAssetCalculateSectionService;
import com.dayou.service.TmplAssetCalculateService;
import com.dayou.utils.FileNetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.Map;
import java.util.Objects;

import static com.dayou.constants.HouseGuarantyDoc.COLLECT1_XLSX;
import static com.dayou.enums.EqptCalculateTmplCode.MAIN_DETAIL;

@RestController
@RequestMapping("/file")
@Slf4j
@CrossOrigin
//@SaCheckLogin
public class FileController {

    @Autowired
    private TmplAssetCalculateService tmplAssetCalculateService;

    @Autowired
    private TmplAssetCalculateSectionService tmplAssetCalculateSectionService;

    @Autowired
    private FileNetConfig fileNetConfig;

    @Autowired
    private HouseExcelDataService houseExcelDataService;

    @Autowired
    private DocumentProductionService documentProductionService;

    /**
     * 上传资产测算表模板
     * @param multipartFile 文件
     * @return Result<Map<String, String>> 文件信息
     */
    @PostMapping("/assets/uploadCalculateTmpl")
    public Result<Map<String, String>> uploadAssetsCalculateTmpl(MultipartFile multipartFile){
        // 判断文件类型
        if (Objects.equals(multipartFile.getContentType(), "application/vnd.ms-excel") || Objects.equals(multipartFile.getContentType(), "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")) {
            return Result.build(FileNetUtil.uploadToServer(multipartFile, fileNetConfig.getBaseDir(), fileNetConfig.getAssetTemplateCalculatePath(), System.currentTimeMillis() + "_" +multipartFile.getOriginalFilename()));
        }else {
            return Result.build(Status.PARAM_ERROR, "请上传xls或xlsx类型的excel文档!");
        }
    }

    /**
     * 上传资产报告模板
     * @param multipartFile 文件
     * @return Result<Map<String, String>> 文件信息
     */
    @PostMapping("/assets/uploadReportTmpl")
    public Result<Map<String, String>> uploadAssetsReportTmpl(MultipartFile multipartFile){
        // 判断文件类型
        if (Objects.equals(multipartFile.getContentType(), "application/msword") || Objects.equals(multipartFile.getContentType(), "application/vnd.openxmlformats-officedocument.wordprocessingml.document")){
            return Result.build(FileNetUtil.uploadToServer(multipartFile, fileNetConfig.getBaseDir(), fileNetConfig.getAssetTemplateReportPath(), System.currentTimeMillis() + "_" + multipartFile.getOriginalFilename()));
        }else {
            return Result.build(Status.PARAM_ERROR, "请上传doc或docx类型的word文档!");
        }
    }

    /**
     * 房地产报告-表单1(估价结果一览表、基本信息)
     * @param file
     * @return
     */
    @PostMapping("/house/guaranty/target")
    public Result<Map<String, String>> updateCollect(MultipartFile file){
        // 判断文件类型
        if (Objects.equals(file.getContentType(), "application/vnd.ms-excel") || Objects.equals(file.getContentType(), "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")) {
            Map<String, String> upload = FileNetUtil.upload(file, fileNetConfig.getBaseDir(), fileNetConfig.getHouseGuarantyPath(), COLLECT1_XLSX);
            //解析excel原始文件
            Long excelId = houseExcelDataService.analysisOriginExcelData(file, upload.get("filePath"));
            Long docId = documentProductionService.saveDocumentProductionByExcelId(excelId);
            upload.put("excelId",String.valueOf(excelId));
            upload.put("docId",String.valueOf(docId));
            return Result.build(upload);
        }else {
            return Result.build(Status.PARAM_ERROR, "请上传xls或xlsx类型的excel文档!");
        }
    }

    /**
     * 下载房地产表单一
     * @param request
     * @param response
     */
    @GetMapping("/download/house/guaranty/collect1")
    public void downloadHouseGuarantyCollect1(HttpServletRequest request, HttpServletResponse response){
        String collect1Path = fileNetConfig.getBaseDir() + fileNetConfig.getHouseGuarantyTemplatePath() + COLLECT1_XLSX;
        File collect1 = new File(collect1Path);
        FileNetUtil.writeFile2Response(request,response,collect1,COLLECT1_XLSX);
    }

    /**
     * 上传图片
     * @param file 文件
     */
    @PostMapping("/upload/image")
    public Result<String> uploadImage(MultipartFile file) {
        String contentType = file.getContentType();
        boolean isImage = contentType != null && contentType.startsWith("image/");
        // 判断文件类型
        if (isImage){
            String fileName = System.currentTimeMillis() + "_" + file.getOriginalFilename();
            FileNetUtil.uploadToServer(file, fileNetConfig.getBaseDir(), fileNetConfig.getImage(), fileName);
            return Result.build(fileNetConfig.getDomain() + "/images/" + fileName);
        }else {
            return Result.build(Status.PARAM_ERROR, "请上传图片!");
        }
    }

    /**
     * 通用下载文件接口
     * @param request
     * @param response
     * @param fileDiskPath 文件在磁盘的路径
     * @param fileName 文件名
     */
    @GetMapping("/download")
    public void downloadFile(HttpServletRequest request, HttpServletResponse response, String fileDiskPath, String fileName){
        File file = new File(fileNetConfig.getBaseDir() + fileDiskPath);
        FileNetUtil.writeFile2Response(request,response,file,fileName);
    }

    /**
     * 下载资产业务测算表模板
     * @param request
     * @param response
     * @param tmplCode 模板code
     * @param tmplType 模板类型(MAIN:主模板,SECTION:段落模板)
     */
    @GetMapping("/download/assets/tmpl")
    public void downloadAnalysisMain(HttpServletRequest request, HttpServletResponse response, String tmplCode, String tmplType){
        // 获取模板
        if (tmplType.equals("MAIN")) {
            TmplAssetCalculate tmpl = tmplAssetCalculateService.getCalculateTmplByCode(tmplCode);
            String path = fileNetConfig.getBaseDir() + tmpl.getFileUrl() + tmpl.getFileName();
            File file = new File(path);
            FileNetUtil.writeFile2Response(request,response,file,tmpl.getFileName());
        }else if (tmplType.equals("SECTION")){
            TmplAssetCalculateSection tmpl = tmplAssetCalculateSectionService.getTmplByCode(tmplCode);
            String path = fileNetConfig.getBaseDir() + tmpl.getSectionFileUrl() + tmpl.getSectionFileName();
            File file = new File(path);
            FileNetUtil.writeFile2Response(request,response,file,tmpl.getSectionFileName());
        }else {
            throw new BusinessException("无符合条件的模板,请联系管理员");
        }
    }

}
