package com.dayou.controller;


import com.dayou.dto.HouseGuarantyTableDTO;
import com.dayou.entity.DocumentProduction;
import com.dayou.entity.HouseGuarantyProcess;
import com.dayou.result.Result;
import com.dayou.service.DocumentProductionService;
import com.dayou.service.HouseGuarantyProcessService;
import com.dayou.service.HouseGuarantyService;
import com.dayou.vo.ProcessVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 房地产抵押类报告
 */
@RestController
@RequestMapping("house/guaranty")
@Slf4j
public class HouseGuarantyDocController {

    @Autowired
    private HouseGuarantyService houseGuarantyService;

    @Autowired
    private HouseGuarantyProcessService houseGuarantyProcessService;

    @Autowired
    private DocumentProductionService documentProductionService;

    /**
     * 生成估价结果一览表的word文件
     * @return 过程表id
     */
    @PostMapping("/targetTable")
    public Result<Long> genTargetsTableWord(@RequestBody HouseGuarantyTableDTO houseGuarantyTableDTO){
        return Result.build(houseGuarantyService.createTableWord(houseGuarantyTableDTO));
    }


    /**
     * 生成致委托人函word
     * @param excelId
     * @param processId
     * @return
     */
    @GetMapping("/consignor")
    public Result<Boolean> genConsignorLetter(Long excelId,Long processId) throws Exception {
        return Result.build(houseGuarantyService.genConsignorLetter(excelId,processId));
    }

    /**
     * 获取过程信息
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    public Result<HouseGuarantyProcess> getHouseGuarantyProcess(@PathVariable("id") Long id){
        return Result.build(houseGuarantyProcessService.getHouseGuarantyProcess(id));
    }

    /**
     * 房地产抵押过程节点
     * id = parentId
     * @return
     */
    @GetMapping("/process")
    public Result<List<ProcessVO>> getHouseGuarantyProcesses(Long id){
        List<ProcessVO> list = houseGuarantyProcessService.getHouseGuarantyProcesses(id);
        return Result.build(list);
    }


    /**
     * 生成《估价的假设和限制条件及使用报告说明》
     * @param id
     * @return
     */
    @GetMapping("/condition/{id}")
    public Result<String> genConditionLetter(@PathVariable("id") Long id){
        return Result.build(houseGuarantyService.genConditionLetter(id));
    }

    /**
     * 生成《房地产抵押估价结果报告》
     * @param id
     * @return
     */
    @GetMapping("/result/{id}")
    public Result<Boolean> genResultLetter(@PathVariable("id") Long id){
        return Result.build(houseGuarantyService.genResultLetter(id));
    }

    @GetMapping("/cerf/html/{businessId}")
    public Result<HouseGuarantyProcess> getCertificatesHtml(@PathVariable("businessId") Long businessId){
        return Result.build(houseGuarantyService.getCertificatesHtml(businessId));
    }


    /**
     * 合并文档成完整报告
     * id = parentId
     * @return
     */
    @GetMapping("/merge")
    public Result<Boolean> houseGuarantyMerge(Long id) throws Exception {
        Boolean ret = houseGuarantyProcessService.houseGuarantyMerge(id);
        return Result.build(ret);
    }

    /**
     * 完整报告信息
     * @return
     */
    @GetMapping("/final/{id}")
    public Result<DocumentProduction> finalDoc(@PathVariable("id") Long id) throws Exception {
        DocumentProduction dp = documentProductionService.getById(id);
        return Result.build(dp);
    }

}
