package com.dayou.controller;

import com.dayou.common.DropDownBoxData;
import com.dayou.entity.TmplAssetReportSection;
import com.dayou.result.Result;
import com.dayou.vo.TmplAssetReportDetailVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.web.bind.annotation.RestController;
import com.dayou.service.TmplAssetReportService;
import com.dayou.entity.TmplAssetReport;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 资产报告模板信息表
 *
 * @author mybatis-plus-generator
 * @since 2024-12-05
 */
@RestController
@RequestMapping("tmplAssetReport")
@Slf4j
@CrossOrigin
public class TmplAssetReportController {
    @Autowired
    private TmplAssetReportService tmplAssetReportService;

    /**
    * 资产报告模板信息表列表
    */
    @GetMapping("/page")
    public Result<Page<TmplAssetReport>> page(TmplAssetReport tmplAssetReport, Page page){
        Page<TmplAssetReport> pages=tmplAssetReportService.selectPage(page,tmplAssetReport);
        return Result.build(pages);
    }

    /**
     * 通过模板id获取报告模板详细信息
     * @param id 模板id
     * @return TmplAssetReportDetailVO
     */
    @GetMapping("/detail/{id}")
    public Result<TmplAssetReportDetailVO> detail(@PathVariable Long id){
        return Result.build(tmplAssetReportService.detail(id));
     }

    /**
     * 资产报告模板信息表新增
     */
    @PostMapping("/save")
    public Result<Boolean> save(@RequestBody TmplAssetReport tmplAssetReport) {
        Boolean ret = tmplAssetReportService.add(tmplAssetReport);
        return Result.build(ret);
    }

    /**
     * 资产报告模板信息表更新
     */
    @PutMapping("/update")
    public Result<Boolean> update(@RequestBody TmplAssetReport tmplAssetReport) {
        Boolean ret = tmplAssetReportService.update(tmplAssetReport);
        return Result.build(ret);
    }

    /**
     * 资产报告模板信息表删除
     */
    @DeleteMapping("/delete/{id}")
    public Result<Boolean> delete(@PathVariable Long id) {
        Boolean ret = tmplAssetReportService.delete(id);
        return Result.build(ret);
    }

    @GetMapping("/getDropDownBoxData")
    public Result<List<DropDownBoxData>> getDropDownBoxData() {
        return Result.build(tmplAssetReportService.getDropDownBoxData());
    }

}

