package com.dayou.handler;

import cn.dev33.satoken.exception.DisableServiceException;
import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import cn.dev33.satoken.exception.NotRoleException;
import com.dayou.exception.BusinessException;
import com.dayou.result.Result;
import com.dayou.result.Status;
import lombok.extern.slf4j.Slf4j;
import org.mybatis.spring.MyBatisSystemException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 全局异常处理
 */
@ControllerAdvice // 可指定包前缀，比如：(basePackages = "com.xx.xxxx")
@Slf4j
public class GlobalException {

    // 在当前类每个方法进入之前触发的操作
    @ModelAttribute
    public void get(HttpServletRequest request) throws IOException {

    }

    @ResponseBody
    @ExceptionHandler
    public Result handlerException(Exception e, HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (e instanceof NotLoginException) {
            // 未登录
            log.error(e.getMessage());
            return Result.build(Status.NOT_LOGIN);
        } else if(e instanceof NotRoleException) {
            // 无角色
            log.error(e.getMessage());
            return Result.build(Status.NOT_ROLE);
        } else if(e instanceof NotPermissionException) {
            // 无权限
            log.error(e.getMessage());
            return Result.build(Status.NOT_PERMISSION);
        } else if(e instanceof HttpRequestMethodNotSupportedException) {
            // 请求异常
            log.error("Exception Message:",e);
            return Result.build(Status.REQUEST_ERROR);
        } else if(e instanceof MyBatisSystemException) {
            // MyBatis抛出异常
            log.error("Exception Message:",e);
            return Result.build(Status.MYBATIS_ERROR);
        } else if(e instanceof BusinessException){
            // 其他异常
            log.error("Exception Message:",e);
            return Result.build(Status.SYSTEM_ERROR,e.getMessage());
        }
        else {
            // 其他异常
            log.error("Exception Message:",e);
            return Result.build(Status.SYSTEM_ERROR);
        }
    }
}
