package com.dayou.utils;

import java.util.*;
import java.util.stream.Collectors;

public class CollectionUtil {

    private static final String EN_DOT = ",";

    private static final String ZH_DOT = "，";

    private static final String ZH_STOP = "、";

    private static final String ZH_SPLIT = "；";

    /**
     * 根据Map的value个数升序
     * @param map
     * @return
     */
    public static LinkedHashMap<String, ? extends List<?>> sortedByCounting(Map<String, ? extends List<?>> map ) {
        // 使用 Stream API 排序
        return map.entrySet().stream()
                .sorted(Map.Entry.comparingByValue(Comparator.comparing(List::size)))
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        Map.Entry::getValue,
                        (oldValue, newValue) -> oldValue, // 解决键冲突，这里不会发生因为键是唯一的
                        LinkedHashMap::new
                ));

    }

    /**
     * 改逗号为分号然后去掉前后中括号
     * @param collection
     * @return
     */
    public static String formatSplitAndRemoveMiddle(Collection<?> collection){
        return formatRemoveMiddle(formatZHSplit(collection));
    }

    /**
     * 改逗号为顿号然后去掉前后中括号
     * @param collection
     * @return
     */
    public static String formatDotAndRemoveMiddle(Collection<?> collection){
        return formatRemoveMiddle(formatZHStop(collection));
    }

    /**
     * 格式化集合类型数据的中括号
     * @param string
     * @return
     */
    public static String formatRemoveMiddle(String string) {
        return  string.replaceAll("^\\[|\\]$", "");
    }

    /**
     * 英文逗号改为中文分号
     * @param collection
     * @return
     */
    public static String formatZHSplit(Collection<?> collection){
       return  collection.toString().replaceAll(EN_DOT,ZH_SPLIT);
    }

    /**
     * 英文逗号改为中文顿号
     * @param collection
     * @return
     */
    public static String formatZHStop(Collection<?> collection){
        return  collection.toString().replaceAll(EN_DOT,ZH_STOP);
    }

    /**
     * 判断集合是否为空
     * @param collection
     * @return
     */
    public static Boolean isNotEmpty(Collection<?> collection){
        return cn.hutool.core.collection.CollectionUtil.isNotEmpty(collection);
    }

    /**
     * 字符串去掉中括号和双引号改英文逗号为中文顿号
     * @param string
     * @return
     */
    public static String formatQuoteAndRemoveMiddle(String string){
       return formatRemoveMiddle(string).replaceAll("\"","").replaceAll(EN_DOT,ZH_STOP).replaceAll(" ","");
    }

    public static void main(String[] args) {
        System.out.println(formatQuoteAndRemoveMiddle("[\"成本法\",\"收益法\"]"));
    }
}
