package com.dayou.utils;

import cn.hutool.core.util.ClassUtil;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class DataUtil {

    /**
     * 将Object对象类型的数据转为Map(支持嵌套对象的情况)
     * @param objData 对象数据
     * @return Map<String, Object>
     * @throws IntrospectionException
     * @throws InvocationTargetException
     * @throws IllegalAccessException
     */
    public static Map<String, Object> objToMap(Object objData) throws IntrospectionException, InvocationTargetException, IllegalAccessException {
        Class<?> aClass = objData.getClass();
        java.lang.reflect.Field[] fields = aClass.getDeclaredFields();
        Map<String, Object> data = new HashMap<>(fields.length);
        // 循环对象中的属性
        for (java.lang.reflect.Field field : fields) {
            PropertyDescriptor pd = new PropertyDescriptor(field.getName(), aClass);
            Method method = pd.getReadMethod();
            String key = field.getName();
            Object value = method.invoke(objData);
            if (value != null) {
                // 判断属性类型是否为基本类型(包括包装类和原始类)或集合(包括Collection和Map)以及其他类型等
                // TODO:类型只判断了一部分,后面看情况补充
                if (
                        ClassUtil.isBasicType(field.getType()) ||
                        String.class.isAssignableFrom(field.getType()) ||
                        Collection.class.isAssignableFrom(field.getType()) ||
                        Map.class.isAssignableFrom(field.getType()) ||
                        BigDecimal.class.isAssignableFrom(field.getType()) ||
                        Date.class.isAssignableFrom(field.getType())
                ) {
                    data.put(key, value);
                }else {
                    // 属性为对象继续转换
                    data.putAll(objToMap(value));
                }
            }
        }
        return data;
    }

}
