package com.dayou.utils;

import java.text.SimpleDateFormat;
import java.util.Date;

public class DateToChinese {

    /**
     * 将Date转换为汉字
     * @param date 时间
     * @return 汉字年月日
     */
    public static String dateConvertChinese(Date date) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-M-d");
        String format = sdf.format(date);
        String[] split = format.split("-");
        String year = split[0];
        String month = split[1];
        String day = split[2];
        String strYear = getStrYear(year);
        String strMonth = getStrMonth(month);
        String strDay = getStrDay(day);

        return strYear + "年" + strMonth + "月" + strDay + "号";
    }

    /**
     * 将字符串日期转换为汉字
     * @param dateStr 时间(如:2024年12月11日)
     * @return 汉字年月日(如:二〇二四年十二月十三号)
     */
    public static String dateStrConvertChinese(String dateStr) {
        String[] parts = dateStr.split("[年月日]");
        String year = parts[0];
        String month = parts[1];
        String day = parts[2];
        String strYear = getStrYear(year);
        String strMonth = getStrMonth(month);
        String strDay = getStrDay(day);

        return strYear + "年" + strMonth + "月" + strDay + "号";
    }

    /**
     * @param day 数字日 13
     * @return 汉字日 13==>十三
     */
    public static String getStrDay(String day) {
        String strDay = "";
        if (Integer.parseInt(day) < 10) {
            strDay = numToStr(day);
            return strDay;
        } else {
            if (day.startsWith("1")) {
                if (day.equals("10")) {
                    return "十";
                } else {
                    String substring = day.substring(1);
                    strDay = "十" + numToStr(substring);
                    return strDay;
                }
            } else if (day.startsWith("2")) {
                if (day.equals("20")) {
                    return "二十";
                } else {
                    String substring = day.substring(1);
                    strDay = "二十" + numToStr(substring);
                    return strDay;
                }
            } else if (day.startsWith("3")) {
                if (day.equals("30")) {
                    return "三十";
                } else {
                    String substring = day.substring(1);
                    strDay = "三十" + numToStr(substring);
                    return strDay;
                }
            }
        }
        return "";
    }


    /**
     * @param month 数字月 7
     * @return 中文月 7==>七
     */
    public static String getStrMonth(String month) {
        return numToStr(month);
    }

    /**
     * @param year 数字年 2024
     * @return 汉字年 二零二四
     */
    public static String getStrYear(String year) {
        int length = year.length();
        StringBuilder strYear = new StringBuilder();
        for (int i = 0; i < length; i++) {
            String substring = year.substring(i, i + 1);
            strYear.append(numToStr(substring));
        }
        return strYear.toString();
    }

    /**
     * @param nun 0~9 ==>〇~九
     * @return
     */
    public static String numToStr(String nun) {
        switch (nun) {
            case "0":
                return "〇";
            case "1":
            case "01":
                return "一";
            case "2":
            case "02":
                return "二";
            case "3":
            case "03":
                return "三";
            case "4":
            case "04":
                return "四";
            case "5":
            case "05":
                return "五";
            case "6":
            case "06":
                return "六";
            case "7":
            case "07":
                return "七";
            case "8":
            case "08":
                return "八";
            case "9":
            case "09":
                return "九";
            case "10":
                return "十";
            case "11":
                return "十一";
            case "12":
                return "十二";
        }
        return "";
    }

}
