package com.dayou.utils;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.dayou.exception.BusinessException;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;

import static com.dayou.result.Status.SYSTEM_ERROR;
import static org.apache.poi.ss.usermodel.CellType.NUMERIC;


public class EasyExcelUtil {

    private static final Map<String,String> nameMap = new HashMap<>();

    static {
        nameMap.put("报告编号","docNo");
        nameMap.put("估价项目名称","docName");
        nameMap.put("估价委托人","consignor");
        nameMap.put("不动产权利人","owner");
        nameMap.put("注册房地产估价师1","appraiser1");
        nameMap.put("注册房地产估价师2","appraiser2");
        nameMap.put("注册号1","appraNo1");
        nameMap.put("注册号2","appraNo2");
        nameMap.put("估价报告出具日期","docDate");
        nameMap.put("估价对象描述1","summary1");
        nameMap.put("估价对象描述2","summary2");
        nameMap.put("致函-估价目的","workPurpose");
        nameMap.put("价值时点","valueTiming");
        nameMap.put("价值类型","valueType");
        nameMap.put("估价方法","workMethod");
        nameMap.put("特别提示1","expeTip1");
        nameMap.put("特别提示2","expeTip2");
        nameMap.put("特别提示3","expeTip3");
        nameMap.put("特别提示4","expeTip4");
        nameMap.put("特别提示5","expeTip5");
        nameMap.put("附件1","affix1");
        nameMap.put("附件2","affix2");
        nameMap.put("附件3","affix3");
        nameMap.put("附件4","affix4");
        nameMap.put("附件5","affix5");
        nameMap.put("附件6","affix6");
        nameMap.put("附件7","affix7");
        nameMap.put("附件8","affix8");
        nameMap.put("声明1","statement1");
        nameMap.put("声明2","statement2");
        nameMap.put("声明3","statement3");
        nameMap.put("声明4","statement4");
        nameMap.put("声明5","statement5");
        nameMap.put("一般假设1","normalAssumption1");
        nameMap.put("一般假设2","normalAssumption2");
        nameMap.put("一般假设3","normalAssumption3");
        nameMap.put("一般假设4","normalAssumption4");
        nameMap.put("一般假设5","normalAssumption5");
        nameMap.put("一般假设6","normalAssumption6");
        nameMap.put("一般假设7","normalAssumption7");
        nameMap.put("一般假设8","normalAssumption8");
        nameMap.put("一般假设9","normalAssumption9");
        nameMap.put("一般假设10","normalAssumption10");
        nameMap.put("未定事项假设1","undefinedAssumption1");
        nameMap.put("未定事项假设2","undefinedAssumption2");
        nameMap.put("未定事项假设3","undefinedAssumption3");
        nameMap.put("未定事项假设4","undefinedAssumption4");
        nameMap.put("未定事项假设5","undefinedAssumption5");
        nameMap.put("未定事项假设6","undefinedAssumption6");
        nameMap.put("不相一致假设1","differentAssumption1");
        nameMap.put("不相一致假设2","differentAssumption2");
        nameMap.put("依据不足假设1","deficiencyAssumption1");
        nameMap.put("依据不足假设2","deficiencyAssumption2");
        nameMap.put("依据不足假设3","deficiencyAssumption3");
        nameMap.put("依据不足假设4","deficiencyAssumption4");
        nameMap.put("依据不足假设5","deficiencyAssumption5");
        nameMap.put("限制条件1","limitCondition1");
        nameMap.put("限制条件2","limitCondition2");
        nameMap.put("限制条件3","limitCondition3");
        nameMap.put("限制条件4","limitCondition4");
        nameMap.put("限制条件5","limitCondition5");
        nameMap.put("限制条件6","limitCondition6");
        nameMap.put("限制条件7","limitCondition7");
        nameMap.put("限制条件8","limitCondition8");
        nameMap.put("使用本报告的一般说明1","normalExplain1");
        nameMap.put("使用本报告的一般说明2","normalExplain2");
        nameMap.put("使用本报告的一般说明3","normalExplain3");
        nameMap.put("使用本报告的一般说明4","normalExplain4");
        nameMap.put("使用本报告的一般说明5","normalExplain5");
        nameMap.put("使用本报告的一般说明6","normalExplain6");
        nameMap.put("使用本报告的一般说明7","normalExplain7");
        nameMap.put("使用本报告的一般说明8","normalExplain8");
        nameMap.put("使用本报告的一般说明9","normalExplain9");
        nameMap.put("估价中的特殊处理事项1","expeHandle1");
        nameMap.put("估价中的特殊处理事项2","expeHandle2");
        nameMap.put("估价中的特殊处理事项3","expeHandle3");
        nameMap.put("估价中的特殊处理事项4","expeHandle4");
        nameMap.put("类型","consignorType");
        nameMap.put("住所","consignorAddress");
        nameMap.put("负责人","consignorPerson");
        nameMap.put("注册资本","consignorMoney");
        nameMap.put("统一社会信用代码","creditCode");
        nameMap.put("四、估价对象及其范围","targetScope");
        nameMap.put("四、建筑物实物状况","buildingInfo");
        nameMap.put("四、土地实物状况","landInfo");
        nameMap.put("四、担保物权状况","guarantyInfo");
        nameMap.put("四、用益物权状况","remInfo");
        nameMap.put("四、租赁或占用状况","rentInfo");
        nameMap.put("五、价值时点2","valueTimingInfo");
        nameMap.put("六、（二）价值内涵1","valueMeaning1");
        nameMap.put("六、（二）价值内涵2","valueMeaning2");
        nameMap.put("六、（二）价值内涵3","valueMeaning3");
        nameMap.put("六、（二）价值内涵4","valueMeaning4");
        nameMap.put("七、估价原则","principle");
        nameMap.put("八、估价依据（一）","gist1");
        nameMap.put("八、估价依据（二）","gist2");
        nameMap.put("八、估价依据（三）","gist3");
        nameMap.put("八、估价依据（四）","gist4");
        nameMap.put("十、估价结果（一）","result1");
        nameMap.put("已抵押担保的债权数额","pledgeValue");
        nameMap.put("拖欠的建设工程价款","debtMoney");
        nameMap.put("其他法定优先受偿款","firstMoney");
        nameMap.put("合计","totalMoney");
        nameMap.put("十、估价结果（三）","result3");
        nameMap.put("十、估价结果（四）","result4");
        nameMap.put("十二、实地查勘期","sceneDate");
        nameMap.put("十三、估价作业期","workDate");

    }

    /**
     * 填充动态数据到Excel
     *
     * @param inputFilePath  模板文件路径
     * @param outputFilePath 输出文件路径
     * @param dataMap        数据map
     * @param evaluate       是否计算公式(大文件慎用,可能会导致内存溢出)
     */
    public static void fillExcelDataByMap(String inputFilePath, String outputFilePath, Map<String, Object> dataMap, boolean evaluate) {
        // 使用EasyExcel填充数据
        ExcelWriter excelWriter = EasyExcel.write(outputFilePath)
                .withTemplate(inputFilePath)
                .inMemory(true) // 启用内存模式
                .build();
        WriteSheet writeSheet = EasyExcel.writerSheet().build();
        excelWriter.fill(dataMap, writeSheet); // 填充数据

        // 由于填充后不会自动更新公式值,所以需要手动更新
        if (evaluate) {
            Workbook workbook = excelWriter.writeContext().writeWorkbookHolder().getWorkbook();
            workbook.getCreationHelper().createFormulaEvaluator().evaluateAll(); // 强制计算公式
        }
        excelWriter.finish();
    }

    /**
     * 合并Excel文件
     *
     * @param sourceExcelPath  源文件路径(段落模板)
     * @param targetExcelPath  目标文件路径(主模板)
     * @param targetSheetIndex 目标sheet索引(要插入的sheet)
     * @param targetRowNum     目标行索引(要插入的行)
     * @param sourceSheetIndex 源文件被引用的sheet
     * @return Workbook 返回工作簿
     * @throws IOException 抛出IO异常
     */
    public static Workbook mergeExcel(String sourceExcelPath, String targetExcelPath, int targetSheetIndex, int targetRowNum, int sourceSheetIndex) throws IOException {
        // 获取源Excel
        InputStream inputStream = Files.newInputStream(Paths.get(sourceExcelPath));
        Workbook sourceWorkbook = new XSSFWorkbook(inputStream);
        Sheet sourceSheet = sourceWorkbook.getSheetAt(sourceSheetIndex);

        // 打开已存在的Excel工作簿
        Workbook targetWorkbook = WorkbookFactory.create(Files.newInputStream(Paths.get(targetExcelPath)));
        Sheet targetSheet = targetWorkbook.getSheetAt(targetSheetIndex); // 获取目标sheet

        // 复制样式映射
        Map<Short, Short> styleMap = copyCellStyle(sourceWorkbook, targetWorkbook, targetSheetIndex, sourceSheetIndex);

        // 复制行和单元格
        for (Row sourceRow : sourceSheet) {
            Row targetRow = targetSheet.createRow(targetRowNum + sourceRow.getRowNum());
            for (Cell sourceCell : sourceRow) {
                Cell targetCell = targetRow.createCell(sourceCell.getColumnIndex());
                copyCell(sourceCell, targetCell, targetWorkbook, styleMap);
            }
        }

        sourceWorkbook.close();
        inputStream.close();

        return targetWorkbook;
    }

    /**
     * 合并Excel文件-并且复制数据验证
     *
     * @param sourceExcelPath  源文件路径(段落模板)
     * @param targetExcelPath  目标文件路径(主模板)
     * @param targetSheetIndex 目标sheet索引(要插入的sheet)
     * @param targetRowNum     目标行索引(要插入的行)
     * @param sourceSheetIndex 源文件被引用的sheet
     * @return Workbook 返回工作簿
     * @throws IOException 抛出IO异常
     */
    public static Workbook mergeExcel(String sourceExcelPath, String targetExcelPath, int targetSheetIndex, int targetRowNum, int sourceSheetIndex, int skipRow) throws IOException {
        // 获取源Excel
        InputStream inputStream = Files.newInputStream(Paths.get(sourceExcelPath));
        Workbook sourceWorkbook = new XSSFWorkbook(inputStream);
        Sheet sourceSheet = sourceWorkbook.getSheetAt(sourceSheetIndex);

        // 打开已存在的Excel工作簿
        Workbook targetWorkbook = WorkbookFactory.create(Files.newInputStream(Paths.get(targetExcelPath)));
        Sheet targetSheet = targetWorkbook.getSheetAt(targetSheetIndex); // 获取目标sheet

        // 复制样式映射
        Map<Short, Short> styleMap = copyCellStyle(sourceWorkbook, targetWorkbook, targetSheetIndex, sourceSheetIndex);

        // 复制行和单元格
        for (Row sourceRow : sourceSheet) {
            Row targetRow = targetSheet.createRow(targetRowNum + sourceRow.getRowNum());
            for (Cell sourceCell : sourceRow) {
                Cell targetCell = targetRow.createCell(sourceCell.getColumnIndex());
                copyCell(sourceCell, targetCell, targetWorkbook, styleMap);
            }
        }

        // 复制数据验证
        copyDataValidations(sourceSheet, targetSheet, skipRow);

        sourceWorkbook.close();
        inputStream.close();

        return targetWorkbook;
    }

    /**
     * 合并Excel文件-目标文件为WorkBook方式且复制数据验证
     *
     * @param sourceExcelPath  源文件路径(段落模板)
     * @param targetWorkbook   目标工作表(主模板)
     * @param targetSheetIndex 目标sheet索引(要插入的sheet)
     * @param targetRowNum     目标行索引(要插入的行)
     * @param sourceSheetIndex 源文件被引用的sheet
     * @throws IOException 抛出IO异常
     */
    public static void mergeExcel(String sourceExcelPath, Workbook targetWorkbook, int targetSheetIndex, int targetRowNum, int sourceSheetIndex, int skipRow) throws IOException {
        // 获取源Excel
        InputStream inputStream = Files.newInputStream(Paths.get(sourceExcelPath));
        Workbook sourceWorkbook = new XSSFWorkbook(inputStream);
        Sheet sourceSheet = sourceWorkbook.getSheetAt(sourceSheetIndex);

        // 打开已存在的Excel工作簿
        Sheet targetSheet = targetWorkbook.getSheetAt(targetSheetIndex); // 获取目标sheet

        // 复制样式映射
        Map<Short, Short> styleMap = copyCellStyle(sourceWorkbook, targetWorkbook, targetSheetIndex, sourceSheetIndex);

        // 复制行和单元格
        for (Row sourceRow : sourceSheet) {
            Row targetRow = targetSheet.createRow(targetRowNum + sourceRow.getRowNum());
            for (Cell sourceCell : sourceRow) {
                Cell targetCell = targetRow.createCell(sourceCell.getColumnIndex());
                copyCell(sourceCell, targetCell, targetWorkbook, styleMap);
            }
        }

        // 复制数据验证
        copyDataValidations(sourceSheet, targetSheet, skipRow);

        sourceWorkbook.close();
        inputStream.close();

    }

    /**
     * 合并Excel文件-目标文件为WorkBook方式
     *
     * @param sourceExcelPath  源文件路径(段落模板)
     * @param targetWorkbook   目标工作表(主模板)
     * @param targetSheetIndex 目标sheet索引(要插入的sheet)
     * @param targetRowNum     目标行索引(要插入的行)
     * @param sourceSheetIndex 源文件被引用的sheet
     * @throws IOException 抛出IO异常
     */
    public static void mergeExcel(String sourceExcelPath, Workbook targetWorkbook, int targetSheetIndex, int targetRowNum, int sourceSheetIndex) throws IOException {
        // 获取源Excel
        InputStream inputStream = Files.newInputStream(Paths.get(sourceExcelPath));
        Workbook sourceWorkbook = new XSSFWorkbook(inputStream);
        Sheet sourceSheet = sourceWorkbook.getSheetAt(sourceSheetIndex);

        // 打开已存在的Excel工作簿
        Sheet targetSheet = targetWorkbook.getSheetAt(targetSheetIndex); // 获取目标sheet

        // 复制样式映射
        Map<Short, Short> styleMap = copyCellStyle(sourceWorkbook, targetWorkbook, targetSheetIndex, sourceSheetIndex);

        // 复制行和单元格
        for (Row sourceRow : sourceSheet) {
            Row targetRow = targetSheet.createRow(targetRowNum + sourceRow.getRowNum());
            for (Cell sourceCell : sourceRow) {
                Cell targetCell = targetRow.createCell(sourceCell.getColumnIndex());
                copyCell(sourceCell, targetCell, targetWorkbook, styleMap);
            }
        }

        sourceWorkbook.close();
        inputStream.close();

    }

    /**
     * 复制单元格样式
     *
     * @param srcBook          源工作簿
     * @param desBook          目标工作簿
     * @param targetSheetIndex 目标sheet索引
     * @param sourceSheetIndex 源文件被引用的sheet
     * @return Map<Short, Short> 样式
     */
    private static Map<Short, Short> copyCellStyle(Workbook srcBook, Workbook desBook, int targetSheetIndex, int sourceSheetIndex) {
        Map<Short, Short> styleMap = new HashMap<>();
        // 复制样式
        for (short i = 0; i < srcBook.getNumCellStyles(); i++) {
            CellStyle srcStyle = srcBook.getCellStyleAt(i);
            CellStyle desStyle = desBook.createCellStyle();
            desStyle.cloneStyleFrom(srcStyle);
            styleMap.put(srcStyle.getIndex(), desStyle.getIndex());
        }

        // 复制合并单元格并应用样式
        Sheet srcSheet = srcBook.getSheetAt(sourceSheetIndex);
        Sheet destSheet = desBook.getSheetAt(targetSheetIndex);

        // 合并单元格
        for (int mergeIndex = 0; mergeIndex < srcSheet.getNumMergedRegions(); mergeIndex++) {
            CellRangeAddress srcRange = srcSheet.getMergedRegion(mergeIndex);
            // 创建新的合并单元格区域
            CellRangeAddress newRange = new CellRangeAddress(
                    srcRange.getFirstRow(),
                    srcRange.getLastRow(),
                    srcRange.getFirstColumn(),
                    srcRange.getLastColumn()
            );
            destSheet.addMergedRegion(newRange);

            // 获取原合并单元格的样式索引
            short srcStyleIndex = srcSheet.getRow(srcRange.getFirstRow()).getCell(srcRange.getFirstColumn()).getCellStyle().getIndex();
            // 应用样式到新合并单元格的左上角单元格
            short destStyleIndex = styleMap.get(srcStyleIndex);
            Row destRow = destSheet.getRow(srcRange.getFirstRow());
            if (destRow == null) {
                destRow = destSheet.createRow(srcRange.getFirstRow());
            }
            Cell destCell = destRow.getCell(srcRange.getFirstColumn());
            if (destCell == null) {
                destCell = destRow.createCell(srcRange.getFirstColumn());
            }
            destCell.setCellStyle(desBook.getCellStyleAt(destStyleIndex));
        }
        return styleMap;
    }

    /**
     * 复制单元格
     *
     * @param srcCell        源单元格
     * @param desCell        目标单元格
     * @param targetWorkbook 目标工作簿
     * @param styleMap       样式
     */
    private static void copyCell(Cell srcCell, Cell desCell, Workbook targetWorkbook, Map<Short, Short> styleMap) {
        // 判断单元格值类型
        if (srcCell.getCellType() == NUMERIC) {
            desCell.setCellValue(srcCell.getNumericCellValue());
        } else if (srcCell.getCellType() == CellType.STRING) {
            desCell.setCellValue(srcCell.getStringCellValue());
        } else if (srcCell.getCellType() == CellType.BOOLEAN) {
            desCell.setCellValue(srcCell.getBooleanCellValue());
        } else if (srcCell.getCellType() == CellType.BLANK) {
            desCell.setBlank();
        } else if (srcCell.getCellType() == CellType.FORMULA) {
            desCell.setCellFormula(srcCell.getCellFormula());
        }

        // 复制样式
        desCell.setCellStyle(targetWorkbook.getCellStyleAt(styleMap.get(srcCell.getCellStyle().getIndex())));
    }

    /**
     * 复制数据验证(下拉框)
     *
     * @param sourceSheet 源Sheet
     * @param targetSheet 目标Sheet
     * @param skipRow     跳过的行数
     */
    private static void copyDataValidations(Sheet sourceSheet, Sheet targetSheet, int skipRow) {
        for (DataValidation validation : sourceSheet.getDataValidations()) {
            // 设置要设置数据验证的单元格坐标
            // 比如数据验证在第2列第3行那坐标就是(2-3),后续设置到目标excel也是沿用这个坐标
            // skipRow会在在此基础上加行,比如skipRow是2,那么坐标就是(4-3),skipRow一般用于目标excel和原excel数据验证位置不一样的情况
            CellRangeAddressList regions = new CellRangeAddressList(validation.getRegions().getCellRangeAddress(0).getFirstRow() + skipRow,
                    validation.getRegions().getCellRangeAddress(0).getLastRow() + skipRow,
                    validation.getRegions().getCellRangeAddress(0).getFirstColumn(),
                    validation.getRegions().getCellRangeAddress(0).getLastColumn());
            // 获取数据源的数据验证信息
            DataValidationHelper validationHelper = targetSheet.getDataValidationHelper();
            DataValidationConstraint constraint = validation.getValidationConstraint();
            // 新建数据验证
            DataValidation newValidation = validationHelper.createValidation(constraint, regions);
            // 在目标excel中设置数据验证
            targetSheet.addValidationData(newValidation);
        }
    }

    /**
     * 房地产基本信息sheet表单 固定解析函数
     *
     * @param wb
     * @param sheetName
     * @return
     */
    public static Map<String, String> getExcelCellValue(Workbook wb, String sheetName) {
        Map<String, String> valueMap = new HashMap<>();
        Sheet sheet = wb.getSheet(sheetName);// 获取指定名称Sheet的内容
        int lastRowNum = sheet.getLastRowNum();
        Row rowKey = null;
        Row rowValue = null;
        Cell cellKey = null;
        Cell cellValue = null;
        for (int rowNum = sheet.getFirstRowNum(); rowNum <= lastRowNum / 2; rowNum += 2) {
            rowKey = sheet.getRow(rowNum);
            rowValue = sheet.getRow(rowNum + 1);
            int lastColNum = rowKey.getLastCellNum();
            for (int colNum = 0; colNum <= lastColNum; colNum++) {
                cellKey = rowKey.getCell(colNum);
                cellValue = rowValue.getCell(colNum);
                if (cellKey != null && StrUtil.isNotBlank(cellKey.toString())
                        && cellValue != null) {
                    String attr = cellKey.getHyperlink().getAddress();
                    valueMap.put(attr, cellValue.toString());
                }
            }
        }
        return valueMap;
    }

    /**
     * 已知列下标,通过目标值获取行下标
     *
     * @param columnIndex 列下标
     * @param targetValue 目标值
     * @param filePath    excel文件
     * @param sheetName   工作表名
     * @return Integer 返回行下标
     */
    public static Integer hasColumnIndexGetRowIndexByValue(int columnIndex, String targetValue, String filePath, String sheetName) {
        try (FileInputStream fis = new FileInputStream(filePath);
            Workbook workbook = new XSSFWorkbook(fis)) {
            Sheet sheet = workbook.getSheet(sheetName); // 获取工作表
            int rowIndex = -1; // 初始化行号为-1，表示未找到

            // 遍历每一行
            for (Row row : sheet) {
                Cell cell = row.getCell(columnIndex); // 获取指定列的单元格
                if (cell != null) {
                    String cellValue = cell.getStringCellValue();
                    if (ObjectUtil.isNotEmpty(cellValue) && targetValue.equals(cellValue.replaceAll(" ",""))) { // 检查是否匹配目标值
                        rowIndex = row.getRowNum(); // 获取行号
                        break; // 找到后退出循环
                    }
                }
            }
            return rowIndex;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 通过单元格下标获取单元格的值
     *
     * @param rowIndex    行下标
     * @param columnIndex 列下标
     * @param filePath    excel文件
     * @param sheetName   工作表名
     * @return String 返回单元格内容
     */
    public static String getCellValueByIndex(int rowIndex, int columnIndex, String filePath, String sheetName) {
        String cellValue = null;
        try (FileInputStream fis = new FileInputStream(filePath);
             Workbook workbook = new XSSFWorkbook(fis)) {
            Sheet sheet = workbook.getSheet(sheetName);
            Row row = sheet.getRow(rowIndex); // 获取指定行
            if (row != null) {
                Cell cell = row.getCell(columnIndex); // 获取指定列的单元格
                if (cell != null) {
                    // 根据单元格类型获取值
                    switch (cell.getCellType()) {
                        case STRING:
                            cellValue = cell.getStringCellValue();
                            break;
                        case NUMERIC:
                            if (DateUtil.isCellDateFormatted(cell)) {
                                cellValue = String.valueOf(cell.getDateCellValue());
                            } else {
                                cellValue = String.valueOf(cell.getNumericCellValue());
                            }
                            break;
                        case BOOLEAN:
                            cellValue = String.valueOf(cell.getBooleanCellValue());
                            break;
                        case FORMULA:
                            cellValue = String.valueOf(cell.getNumericCellValue());
                            break;
                    }
                } else {
                    throw new BusinessException(sheetName + "中查找的单元格不存在", SYSTEM_ERROR.getCode().toString());
                }
            } else {
                throw new BusinessException(sheetName + "中查找的行不存在!", SYSTEM_ERROR.getCode().toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return cellValue;
    }

    /**
     * 读取EXCEL指定sheet、row 数据
     *
     * @param wb excel文件
     * @param sheetName sheet名字
     * @param row 目标行
     * @return
     */
    public static Map<String, String> readExcelData(Workbook wb, String sheetName,int row) {
        Map<String, String> valueMap = new HashMap<>();
        Sheet sheet = wb.getSheet(sheetName);// 获取指定名称Sheet的内容
        Row rowKey = null;
        Row rowValue = null;
        Cell cellKey = null;
        Cell cellValue = null;
        rowKey = sheet.getRow(0);
        rowValue = sheet.getRow(row);
        int lastColNum = rowKey.getLastCellNum();
        FormulaEvaluator evaluator = wb.getCreationHelper().createFormulaEvaluator();
        for (int colNum = 0; colNum <= lastColNum; colNum++) {
            cellKey = rowKey.getCell(colNum);
            cellValue = rowValue.getCell(colNum);
            if (cellKey != null && StrUtil.isNotBlank(cellKey.toString())) {
                String attr = nameMap.get(cellKey.getStringCellValue());
                String value = "";
                if (cellValue == null) {
                    valueMap.put(attr, value);
                }else {
                    switch (cellValue.getCellType()) {
                        case STRING:
                            value = cellValue.getStringCellValue();
                            break;
                        case FORMULA: // 处理公式计算后的值，而非公式本身
                            cellValue.setCellFormula(null);
                            evaluator.evaluateFormulaCell(cellValue);
                            if (cellValue.getCellType().equals(NUMERIC)){
                                DecimalFormat df = new DecimalFormat("0");
                                value = df.format(cellValue.getNumericCellValue());
                            }else{
                                value = cellValue.getStringCellValue();
                            }

                            break;
                        default: // 对于其他类型，可以根据需要处理或忽略
                            value = "Cell type not supported";
                    }
                    valueMap.put(attr, value);
                }

            }
        }
        return valueMap;
    }

}
