package com.dayou.utils;

import org.apache.commons.io.FileUtils;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

public class ExcelConvertToHtmlUtil {

    /**
     * 将excel指定名字的sheet转换为html标记语言并返回为字符串
     * @param filePath     excel源文件文件的路径
     * @param isWithStyle  是否需要表格样式 包含 字体 颜色 边框 对齐方式
     * @param sheetName    要读取的sheet名字
     * @param type excel文件类型
     * @return String
     */
    public static String readExcelToHtmlCode(String filePath, boolean isWithStyle, String type, String sheetName){
        InputStream is = null;
        String htmlExcel = null;
        Map<String, String> stylemap = new HashMap<String, String>();
        try {
            // 不同文件类型处理不一样
            if ("csv".equalsIgnoreCase(type)) {
                htmlExcel = getCSVInfo(filePath);
            } else {
                File sourcefile = new File(filePath);
                is = Files.newInputStream(sourcefile.toPath());
                Workbook wb = WorkbookFactory.create(is);
                if (wb instanceof XSSFWorkbook) { // 03版excel处理方法
                    XSSFWorkbook xWb = (XSSFWorkbook) wb;
                    htmlExcel = getExcelInfo(xWb, isWithStyle, stylemap, sheetName);
                } else if (wb instanceof HSSFWorkbook) { // 07及10版以后的excel处理方法
                    HSSFWorkbook hWb = (HSSFWorkbook) wb;
                    htmlExcel = getExcelInfo(hWb, isWithStyle, stylemap, sheetName);
                }
            }
            htmlExcel = genHtmlCode(htmlExcel, stylemap, sheetName);
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        } finally {
            try {
                if (is != null)
                    is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return htmlExcel;
    }

    /**
     * 将excel指定名字的sheet并指定下标的表格转换为html标记语言并返回为字符串
     * @param lastRowNum   最后一行下标
     * @param lastColNum   最后一列下标
     * @param filePath     excel源文件文件的路径
     * @param isWithStyle  是否需要表格样式 包含 字体 颜色 边框 对齐方式
     * @param sheetName    要读取的sheet名字
     * @param type excel文件类型
     * @return String
     */
    public static String readExcelToHtmlCode(String filePath, boolean isWithStyle, String type, String sheetName, int lastRowNum, int lastColNum){
        InputStream is = null;
        String htmlExcel = null;
        Map<String, String> stylemap = new HashMap<String, String>();
        try {
            // 不同文件类型处理不一样
            if ("csv".equalsIgnoreCase(type)) {
                htmlExcel = getCSVInfo(filePath);
            } else {
                File sourcefile = new File(filePath);
                is = Files.newInputStream(sourcefile.toPath());
                Workbook wb = WorkbookFactory.create(is);
                if (wb instanceof XSSFWorkbook) { // 03版excel处理方法
                    XSSFWorkbook xWb = (XSSFWorkbook) wb;
                    htmlExcel = getExcelInfoByIndex(xWb, isWithStyle, stylemap, sheetName, lastRowNum, lastColNum);
                } else if (wb instanceof HSSFWorkbook) { // 07及10版以后的excel处理方法
                    HSSFWorkbook hWb = (HSSFWorkbook) wb;
                    htmlExcel = getExcelInfoByIndex(hWb, isWithStyle, stylemap, sheetName, lastRowNum, lastColNum);
                }
            }
            htmlExcel = genHtmlCode(htmlExcel, stylemap, sheetName);
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        } finally {
            try {
                if (is != null)
                    is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return htmlExcel;
    }

    /**
     * 将excel指定名字的sheet转换为html标记语言并保存为html文件
     * @param filePath     excel源文件文件的路径
     * @param htmlLocation 生成的html文件的路径
     * @param isWithStyle  是否需要表格样式 包含 字体 颜色 边框 对齐方式
     * @param sheetName    要读取的sheet名字
     * @param type excel文件类型
     * @return void
     */
    public static String readExcelToHtmlFile(String filePath, String htmlLocation, boolean isWithStyle, String type, String sheetName) throws Exception {
        InputStream is = null;
        String htmlExcel = null;
        Map<String, String> stylemap = new HashMap<String, String>();
        try {
            // 不同文件类型处理不一样
            if ("csv".equalsIgnoreCase(type)) {
                htmlExcel = getCSVInfo(filePath);
                writeCSVTypeFile(htmlExcel, htmlLocation, stylemap, sheetName);
            } else {
                File sourcefile = new File(filePath);
                is = Files.newInputStream(sourcefile.toPath());
                Workbook wb = WorkbookFactory.create(is);
                if (wb instanceof XSSFWorkbook) { // 03版excel处理方法
                    XSSFWorkbook xWb = (XSSFWorkbook) wb;
                    htmlExcel = getExcelInfo(xWb, isWithStyle, stylemap, sheetName);
                } else if (wb instanceof HSSFWorkbook) { // 07及10版以后的excel处理方法
                    HSSFWorkbook hWb = (HSSFWorkbook) wb;
                    htmlExcel = getExcelInfo(hWb, isWithStyle, stylemap, sheetName);
                }
                writeFile(htmlExcel, htmlLocation, stylemap, sheetName);
            }
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());
        } finally {
            try {
                if (is != null)
                    is.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return htmlLocation;
    }

    private static void getcscvvalue(BufferedReader reader, List col, String oldvalue, List list) {
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                String[] item = line.split(",", -1);
                boolean isbreak = false;
                for (String s : item) {
                    String value = s;
                    if (value.endsWith("\"")) {
                        value = oldvalue + value;
                        col.add(value);
                    } else if (item.length == 1) {
                        value = oldvalue + value;
                        getcscvvalue(reader, col, value, list);
                        isbreak = true;
                    } else if (value.startsWith("\"")) {
                        getcscvvalue(reader, col, value, list);
                        isbreak = true;
                    } else {
                        col.add(value);
                    }
                }
                if (!isbreak) {
                    list.add(col);
                    col = new ArrayList();
                }
            }
        } catch (IOException e) {
        }
    }

    private static String getCSVInfo(String filePath) {
        StringBuilder sb = new StringBuilder();
        DataInputStream in = null;
        try {
            in = new DataInputStream(Files.newInputStream(Paths.get(filePath)));
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            String line = null;
            List list = new ArrayList();
            while ((line = reader.readLine()) != null) {
                String[] item = line.split(",");
                List col = new ArrayList();
                for (String value : item) {
                    if (value.startsWith("\"")) {
                        getcscvvalue(reader, col, value, list);
                    } else {
                        col.add(value);
                    }
                }
                list.add(col);
            }
            sb.append("<table>");
            for (Object object : list) {
                List col = (List) object;
                if (col == null || col.isEmpty()) {
                    sb.append("<tr><td ></td></tr>");
                }
                sb.append("<tr>");
                for (Object o : col) {
                    String value = (String) o;
                    if (value == null || value.isEmpty()) {
                        sb.append("<td> </td>");
                        continue;
                    } else {
                        sb.append("<td>").append(value).append("</td>");
                    }
                }
                sb.append("</tr>");
            }
            sb.append("</table>");
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }

    /**
     * 读取excel文件，返回转换后的html字符串
     *
     * @param wb          工作簿
     * @param isWithStyle 是否带上样式
     * @param stylemap    样式
     * @return String
     */
    private static String getExcelInfo(Workbook wb, boolean isWithStyle, Map<String, String> stylemap, String sheetName) {
        StringBuffer sb = new StringBuffer();
        Sheet sheet = wb.getSheet(sheetName);// 获取指定名称Sheet的内容
        int lastRowNum = sheet.getLastRowNum();
        Map<String, String>[] map = getRowSpanColSpanMap(sheet);
        sb.append("<table id='table_").append(sheetName).append("' ");
        sb.append("class='block'");
        sb.append(">");
        Row row = null; // 兼容
        Cell cell = null; // 兼容

        for (int rowNum = sheet.getFirstRowNum(); rowNum <= lastRowNum; rowNum++) {
            row = sheet.getRow(rowNum);
            if (row == null) {
                sb.append("<tr><td ></td></tr>");
                continue;
            }
            sb.append("<tr>");
            int lastColNum = row.getLastCellNum();
            for (int colNum = 0; colNum <= lastColNum; colNum++) {
                cell = row.getCell(colNum);
                if (cell == null) { // 特殊情况 空白的单元格会返回null
                    sb.append("<td> </td>");
                    continue;
                }
                String stringValue = getCellValue(cell);
                if (map[0].containsKey(rowNum + "," + colNum)) {
                    String pointString = map[0].get(rowNum + "," + colNum);
                    map[0].remove(rowNum + "," + colNum);
                    int bottomeRow = Integer.parseInt(pointString.split(",")[0]);
                    int bottomeCol = Integer.parseInt(pointString.split(",")[1]);
                    int rowSpan = bottomeRow - rowNum + 1;
                    int colSpan = bottomeCol - colNum + 1;
                    sb.append("<td rowspan= '").append(rowSpan).append("' colspan= '").append(colSpan).append("' ");
                } else if (map[1].containsKey(rowNum + "," + colNum)) {
                    map[1].remove(rowNum + "," + colNum);
                    continue;
                } else {
                    sb.append("<td ");
                }

                // 判断是否需要样式
                if (isWithStyle) {
                    dealExcelStyle(wb, sheet, cell, sb, stylemap);// 处理单元格样式
                }

                sb.append("><nobr>");
                //如果单元格为空要判断该单元格是不是通过其他单元格计算得到的
                if (stringValue == null || stringValue.trim().isEmpty()) {
                    FormulaEvaluator evaluator = wb.getCreationHelper().createFormulaEvaluator();
                    if (evaluator.evaluate(cell) != null) {
                        //如果单元格的值是通过其他单元格计算来的，则通过单元格计算获取
                        String cellnumber = evaluator.evaluate(cell).getNumberValue() + "";
                        //如果单元格的值是小数，保留两位
                        if (cellnumber.contains(".")) {
                            String[] decimal = cellnumber.split("\\.");
                            if (decimal[1].length() > 2) {
                                int num1 = decimal[1].charAt(0) - '0';
                                int num2 = decimal[1].charAt(1) - '0';
                                int num3 = decimal[1].charAt(2) - '0';
                                if (num3 == 9) {
                                    num2 = 0;
                                } else if (num3 >= 5) {
                                    num2 = num2 + 1;
                                }
                                cellnumber = decimal[0] + "." + num1 + num2;
                            }
                        }
                        stringValue = cellnumber;
                    }
                    assert stringValue != null;
                    sb.append(stringValue.replace(String.valueOf((char) 160), " "));
                } else {
                    // 将ascii码为160的空格转换为html下的空格（ ）
                    sb.append(stringValue.replace(String.valueOf((char) 160), " "));
                }
                sb.append("</nobr></td>");
            }
            sb.append("</tr>");
        }
        sb.append("</table>");
        return sb.toString();
    }

    /**
     * 读取excel文件，返回转换后的html字符串
     * 指定下标
     * @param wb          工作簿
     * @param isWithStyle 是否带上样式
     * @param stylemap    样式
     * @return String
     */
    private static String getExcelInfoByIndex(Workbook wb, boolean isWithStyle, Map<String, String> stylemap, String sheetName, int lastRowNum, int lastColNum) {
        StringBuffer sb = new StringBuffer();
        Sheet sheet = wb.getSheet(sheetName);// 获取指定名称Sheet的内容
        Map<String, String>[] map = getRowSpanColSpanMap(sheet);
        sb.append("<table id='table_").append(sheetName).append("' ");
        sb.append("class='block'");
        sb.append(">");
        Row row = null; // 兼容
        Cell cell = null; // 兼容

        for (int rowNum = sheet.getFirstRowNum(); rowNum <= lastRowNum; rowNum++) {
            row = sheet.getRow(rowNum);
            if (row == null) {
                sb.append("<tr><td ></td></tr>");
                continue;
            }
            sb.append("<tr>");
            for (int colNum = 0; colNum <= lastColNum; colNum++) {
                cell = row.getCell(colNum);
                if (cell == null) { // 特殊情况 空白的单元格会返回null
                    sb.append("<td> </td>");
                    continue;
                }
                String stringValue = getCellValue(cell);
                if (map[0].containsKey(rowNum + "," + colNum)) {
                    String pointString = map[0].get(rowNum + "," + colNum);
                    map[0].remove(rowNum + "," + colNum);
                    int bottomeRow = Integer.parseInt(pointString.split(",")[0]);
                    int bottomeCol = Integer.parseInt(pointString.split(",")[1]);
                    int rowSpan = bottomeRow - rowNum + 1;
                    int colSpan = bottomeCol - colNum + 1;
                    sb.append("<td rowspan= '").append(rowSpan).append("' colspan= '").append(colSpan).append("' ");
                } else if (map[1].containsKey(rowNum + "," + colNum)) {
                    map[1].remove(rowNum + "," + colNum);
                    continue;
                } else {
                    sb.append("<td ");
                }

                // 判断是否需要样式
                if (isWithStyle) {
                    dealExcelStyle(wb, sheet, cell, sb, stylemap);// 处理单元格样式
                }

                sb.append("><nobr>");
                //如果单元格为空要判断该单元格是不是通过其他单元格计算得到的
                if (stringValue == null || stringValue.trim().isEmpty()) {
                    FormulaEvaluator evaluator = wb.getCreationHelper().createFormulaEvaluator();
                    if (evaluator.evaluate(cell) != null) {
                        //如果单元格的值是通过其他单元格计算来的，则通过单元格计算获取
                        String cellnumber = evaluator.evaluate(cell).getNumberValue() + "";
                        //如果单元格的值是小数，保留两位
                        if (cellnumber.contains(".")) {
                            String[] decimal = cellnumber.split("\\.");
                            if (decimal[1].length() > 2) {
                                int num1 = decimal[1].charAt(0) - '0';
                                int num2 = decimal[1].charAt(1) - '0';
                                int num3 = decimal[1].charAt(2) - '0';
                                if (num3 == 9) {
                                    num2 = 0;
                                } else if (num3 >= 5) {
                                    num2 = num2 + 1;
                                }
                                cellnumber = decimal[0] + "." + num1 + num2;
                            }
                        }
                        stringValue = cellnumber;
                    }
                    assert stringValue != null;
                    sb.append(stringValue.replace(String.valueOf((char) 160), " "));
                } else {
                    // 将ascii码为160的空格转换为html下的空格（ ）
                    sb.append(stringValue.replace(String.valueOf((char) 160), " "));
                }
                sb.append("</nobr></td>");
            }
            sb.append("</tr>");
        }
        sb.append("</table>");
        return sb.toString();
    }


    private static Map<String, String>[] getRowSpanColSpanMap(Sheet sheet) {
        Map<String, String> map0 = new HashMap<String, String>();
        Map<String, String> map1 = new HashMap<String, String>();
        int mergedNum = sheet.getNumMergedRegions();
        CellRangeAddress range = null;
        for (int i = 0; i < mergedNum; i++) {
            range = sheet.getMergedRegion(i);
            int topRow = range.getFirstRow();
            int topCol = range.getFirstColumn();
            int bottomRow = range.getLastRow();
            int bottomCol = range.getLastColumn();
            map0.put(topRow + "," + topCol, bottomRow + "," + bottomCol);
            int tempRow = topRow;
            while (tempRow <= bottomRow) {
                int tempCol = topCol;
                while (tempCol <= bottomCol) {
                    map1.put(tempRow + "," + tempCol, "");
                    tempCol++;
                }
                tempRow++;
            }
            map1.remove(topRow + "," + topCol);
        }
        return new Map[]{map0, map1};
    }

    /**
     * 获取表格单元格Cell内容
     *
     * @param cell 单元格
     * @return String
     */
    private static String getCellValue(Cell cell) {
        String result;
        switch (cell.getCellType()) {
            case NUMERIC:// 数字类型
                if (DateUtil.isCellDateFormatted(cell)) {// 处理日期格式、时间格式
                    SimpleDateFormat sdf = null;
                    if (cell.getCellStyle().getDataFormat() == HSSFDataFormat.getBuiltinFormat("h:mm")) {
                        sdf = new SimpleDateFormat("HH:mm");
                    } else {// 日期
                        sdf = new SimpleDateFormat("yyyy-MM-dd");
                    }
                    Date date = cell.getDateCellValue();
                    result = sdf.format(date);
                } else if (cell.getCellStyle().getDataFormat() == 58) {
                    // 处理自定义日期格式：m月d日(通过判断单元格的格式id解决，id的值是58)
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    double value = cell.getNumericCellValue();
                    Date date = org.apache.poi.ss.usermodel.DateUtil.getJavaDate(value);
                    result = sdf.format(date);
                } else {
                    double value = cell.getNumericCellValue();
                    CellStyle style = cell.getCellStyle();
                    DecimalFormat format = new DecimalFormat();
                    String temp = style.getDataFormatString();
                    // 单元格设置成常规
                    if (temp.equals("General")) {
                        format.applyPattern("#");
                    }
                    result = format.format(value);
                }
                break;
            case STRING:// String类型
                result = cell.getRichStringCellValue().toString();
                result = result.replace("\n", "<br>");
                break;
            default:
                result = "";
                break;
        }
        return result;
    }

    /**
     * 处理表格样式
     *
     * @param wb    表格
     * @param sheet sheet
     * @param sb    html
     */
    private static void dealExcelStyle(Workbook wb, Sheet sheet, Cell cell, StringBuffer sb, Map<String, String> stylemap) {
        CellStyle cellStyle = cell.getCellStyle();
        if (cellStyle != null) {
            HorizontalAlignment alignment = cellStyle.getAlignment();
            VerticalAlignment verticalAlignment = cellStyle.getVerticalAlignment();
            String _style = "vertical-align:" + convertVerticalAlignToHtml(verticalAlignment) + ";";
            if (wb instanceof XSSFWorkbook) {
                XSSFFont xf = ((XSSFCellStyle) cellStyle).getFont();
                if (xf.getBold()){
                    _style += "font-weight:400";
                }
                String align = convertAlignToHtml(alignment);
                int columnWidth = sheet.getColumnWidth(cell.getColumnIndex());
                _style += ";font-size: " + xf.getFontHeight() / 2 + "%;width:" + columnWidth + "px;text-align:" + align + ";";

                XSSFColor xc = xf.getXSSFColor();
                if (xc != null) {
                    _style += "color:#" + xc.getARGBHex().substring(2) + ";";
                }

                XSSFColor bgColor = (XSSFColor) cellStyle.getFillForegroundColorColor();
                if (bgColor != null) {
                    _style += "background-color:#" + bgColor.getARGBHex().substring(2) + ";"; // 背景颜色
                }
                _style += getBorderStyle(0, cellStyle.getBorderTop().getCode(), ((XSSFCellStyle) cellStyle).getTopBorderXSSFColor());
                _style += getBorderStyle(1, cellStyle.getBorderRight().getCode(), ((XSSFCellStyle) cellStyle).getRightBorderXSSFColor());
                _style += getBorderStyle(2, cellStyle.getBorderBottom().getCode(), ((XSSFCellStyle) cellStyle).getBottomBorderXSSFColor());
                _style += getBorderStyle(3, cellStyle.getBorderLeft().getCode(), ((XSSFCellStyle) cellStyle).getLeftBorderXSSFColor());
            } else if (wb instanceof HSSFWorkbook) {
                HSSFFont hf = ((HSSFCellStyle) cellStyle).getFont(wb);
                short boldWeight = hf.getFontHeight();
                short fontColor = hf.getColor();
                HSSFPalette palette = ((HSSFWorkbook) wb).getCustomPalette(); // 类HSSFPalette用于求的颜色的国际标准形式
                HSSFColor hc = palette.getColor(fontColor);
                String align = convertAlignToHtml(alignment);
                int columnWidth = sheet.getColumnWidth(cell.getColumnIndex());
                _style += "font-weight:" + boldWeight + ";font-size: " + hf.getFontHeight() / 2 + "%;text-align:" + align + ";width:" + columnWidth + "px;";
                String fontColorStr = convertToStardColor(hc);
                if (fontColorStr != null && !fontColorStr.trim().isEmpty()) {
                    _style += "color:" + fontColorStr + ";"; // 字体颜色
                }
                short bgColor = cellStyle.getFillForegroundColor();
                hc = palette.getColor(bgColor);
                String bgColorStr = convertToStardColor(hc);
                if (bgColorStr != null && !bgColorStr.trim().isEmpty()) {
                    _style += "background-color:" + bgColorStr + ";"; // 背景颜色
                }
                _style += getBorderStyle(palette, 0, cellStyle.getBorderTop().getCode(), cellStyle.getTopBorderColor());
                _style += getBorderStyle(palette, 1, cellStyle.getBorderRight().getCode(), cellStyle.getRightBorderColor());
                _style += getBorderStyle(palette, 3, cellStyle.getBorderLeft().getCode(), cellStyle.getLeftBorderColor());
                _style += getBorderStyle(palette, 2, cellStyle.getBorderBottom().getCode(), cellStyle.getBottomBorderColor());
            }
            String calssname = "";
            if (!stylemap.containsKey(_style)) {
                int count = stylemap.size();
                calssname = "td" + count;
                stylemap.put(_style, calssname);
            } else {
                calssname = stylemap.get(_style);
            }
            if (!"".equals(calssname)) {
                sb.append("class='").append(calssname).append("'");
            }
        }
    }

    /**
     * 单元格内容的水平对齐方式
     *
     * @param alignment 水平对其样式
     * @return String
     */
    private static String convertAlignToHtml(HorizontalAlignment alignment) {
        String align = "center";
        switch (alignment) {
            case LEFT:
                align = "left";
                break;
            case CENTER:
                align = "center";
                break;
            case RIGHT:
                align = "right";
                break;
            default:
                break;
        }
        return align;
    }

    /**
     * 单元格中内容的垂直排列方式
     *
     * @param verticalAlignment 垂直位置样式
     * @return String
     */
    private static String convertVerticalAlignToHtml(VerticalAlignment verticalAlignment) {

        String valign = "middle";
        switch (verticalAlignment) {
            case BOTTOM:
                valign = "bottom";
                break;
            case CENTER:
                valign = "middle";
                break;
            case TOP:
                valign = "top";
                break;
            default:
                break;
        }
        return valign;
    }

    private static String convertToStardColor(HSSFColor hc) {
        StringBuilder sb = new StringBuilder();
        if (hc != null) {
            if (HSSFColor.HSSFColorPredefined.AUTOMATIC.getIndex() == hc.getIndex()) {
                return null;
            }
            sb.append("#");
            for (int i = 0; i < hc.getTriplet().length; i++) {
                sb.append(fillWithZero(Integer.toHexString(hc.getTriplet()[i])));
            }
        }
        return sb.toString();
    }

    private static String fillWithZero(String str) {
        if (str != null && str.length() < 2) {
            return "0" + str;
        }
        return str;
    }

    static String[] bordesr = {"border-top:", "border-right:", "border-bottom:", "border-left:"};
    static String[] borderStyles = {"solid ", "solid ", "solid ", "solid ", "solid ", "solid ", "solid ", "solid ",
            "solid ", "solid", "solid", "solid", "solid", "solid"};

    private static String getBorderStyle(HSSFPalette palette, int b, short s, short t) {
        if (s == 0)
            return bordesr[b] + borderStyles[s] + "#000000 1px;";
        String borderColorStr = convertToStardColor(palette.getColor(t));
        borderColorStr = borderColorStr == null || borderColorStr.isEmpty() ? "#000000" : borderColorStr;
        return bordesr[b] + borderStyles[s] + borderColorStr + " 1px;";
    }

    private static String getBorderStyle(int b, short s, XSSFColor xc) {
        if (s == 0)
            return bordesr[b] + borderStyles[s] + "#000000 1px;";
        if (xc != null) {
            String borderColorStr = xc.getARGBHex();// t.getARGBHex();
            borderColorStr = borderColorStr == null || borderColorStr.isEmpty() ? "#000000"
                    : borderColorStr.substring(2);
            return bordesr[b] + borderStyles[s] + borderColorStr + " 1px;";
        }
        return "";
    }

    /*
     * @param content 生成的excel表格标签
     *
     * @param htmlPath 生成的html文件地址
     */
    private static void writeFile(String content, String htmlPath, Map<String, String> stylemap, String name) {
        File file2 = new File(htmlPath);
//        StringBuilder sb = new StringBuilder();
        try {
            file2.createNewFile();// 创建文件
            FileUtils.write(file2, genHtmlCode(content, stylemap, name), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 将html字符串拼接并返回
     * @param content table内容
     * @param stylemap 样式
     * @param name 表格名
     * @return String
     */
    private static String genHtmlCode(String content, Map<String, String> stylemap, String name){
        StringBuilder sb = new StringBuilder();
        sb.append("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><title>").append(name).append("</title><style type=\"text/css\">");
        sb.append("ul{list-style: none;max-width: calc(100%);padding: 0px;margin: 0px;overflow-x: scroll;white-space: nowrap;} ul li{padding: 3px 5px;display: inline-block;border-right: 1px solid #768893;} ul li.cur{color: #F59C25;} table{border-collapse:collapse;display:none;width:100%;} table.block{display: block;}");
        for (Map.Entry<String, String> entry : stylemap.entrySet()) {
            String mapKey = entry.getKey();
            String mapValue = entry.getValue();
            sb.append(" .").append(mapValue).append("{").append(mapKey).append("}");
        }
        sb.append("</style><script>");
        sb.append("function changetab(i){var block = document.getElementsByClassName(\"block\");block[0].className = block[0].className.replace(\"block\",\"\");var cur = document.getElementsByClassName(\"cur\");cur[0].className = cur[0].className.replace(\"cur\",\"\");var curli = document.getElementById(\"li_\"+i);curli.className += ' cur';var curtable = document.getElementById(\"table_\"+i);curtable.className=' block';}");
        sb.append("</script></head><body>");
        sb.append("<div>");
        sb.append(content);
        sb.append("</div>");
        sb.append("</body></html>");
        return sb.toString();
    }

    private static void writeCSVTypeFile(String content, String htmlPath, Map<String, String> stylemap, String name) {
        File file2 = new File(htmlPath);
        try {
            file2.createNewFile();// 创建文件
            FileUtils.write(file2, genCSVTypeHtmlCode(content, stylemap, name), "UTF-8");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 将CSV类型Excel的html字符串拼接并返回
     * @param content table内容
     * @param stylemap 样式
     * @param name 表格名
     * @return String
     */
    private static String genCSVTypeHtmlCode(String content, Map<String, String> stylemap, String name){
        return "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><title>" + name + "</title><style type=\"text/css\">" +
                "ul{list-style: none;max-width: calc(100%);padding: 0px;margin: 0px;overflow-x: scroll;white-space: nowrap;} ul li{padding: 3px 5px;display: inline-block;border-right: 1px solid #768893;} ul li.cur{color: #F59C25;} table{border-collapse:collapse;width:100%;} td{border: solid #000000 1px; min-width: 200px;}" +
                "</style></head><body>" +
                "<div>" +
                content +
                "</div>" +
                "</body></html>";
    }

}
