package com.dayou.utils;

import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;

import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * JSON工具类
 */
public class JsonUtil {


    /**
     * 从JSONObject文件读取json数组
     * @param filePath
     * @param key 值为json数组的key
     * @return
     * @throws IOException
     */
    public static JSONArray file2JsonArray(String filePath,String key) throws IOException {
        String jsonStr = new String(Files.readAllBytes(Paths.get(filePath)), StandardCharsets.UTF_8);
        JSONObject jsonObject = JSONUtil.parseObj(jsonStr);
        return new JSONArray(jsonObject.get(key));
    }

    /**
     * 从JSONArray文件读取json数组
     * @param filePath
     * @return
     * @throws IOException
     */
    public static JSONArray file2JsonArray(String filePath) throws IOException {
        return new JSONArray(
                new String(Files.readAllBytes(Paths.get(filePath)), StandardCharsets.UTF_8));

    }

    /**
     * 从JSONObject文件读取json对象
     * @param filePath
     * @return
     * @throws IOException
     */
    public static  JSONObject file2JsonObject(String filePath) throws IOException {
        return new JSONObject(new String(Files.readAllBytes(Paths.get(filePath)), StandardCharsets.UTF_8));
    }

    /**
     * 将JSON数组数据输出到文件
     * @param jsonArray
     * @param filePath
     * @throws IOException
     */
    public static void jsonArray2File(JSONArray jsonArray, String filePath) throws IOException {
        FileOutputStream fos = new FileOutputStream(filePath);
        fos.write(jsonArray.toString().getBytes());
        fos.close();
    }

    /**
     * 将JSONOBJECT数据输出到文件
     * @param jsonObject
     * @param filePath
     * @throws IOException
     */
    public static void jsonObject2File(JSONObject jsonObject, String filePath) throws IOException {
        FileOutputStream fos = new FileOutputStream(filePath);
        fos.write(jsonObject.toString().getBytes());
        fos.close();
    }

    /**
     * JSONArray 根据key聚合value
     * @param array
     * @param key
     * @return
     */
    public static List<Object> collectByKeyToList(JSONArray array, String key){
        return array.stream().map(obj -> ((JSONObject) obj).get(key))
                .filter(x->x!=null && !x.equals("/")).collect(Collectors.toList());

    }

    /**
     * JSONArray 根据key聚合value
     * @param array
     * @param key
     * @return
     */
    public static Set<Object> collectByKeyToSet(JSONArray array, String key){
        return array.stream().map(obj -> ((JSONObject) obj).get(key))
                .filter(x->x!=null && !x.equals("/")).collect(Collectors.toSet());

    }

    /**
     * groupBy
     * @param jsonArray
     * @param key
     * @return
     */
    public static Map<Object, List<Object>> groupBy(JSONArray jsonArray, String key){
        Map<Object, List<Object>> collect = jsonArray.stream().collect(Collectors.groupingBy(obj -> ((JSONObject) obj).get(key)));
        return collect;
    }
}
