package com.dayou.utils;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import com.dayou.dto.HousePledgeDTO;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 解析抵押权信息
 */
public class ParsingPledgeUtil {

    /**
     * 解析抵押权信息
     * @param pledgeInfo
     * @return
     */
    public static JSONObject parsingPledgeInfo(String pledgeInfo, JSONObject jsonObject){
        if (StrUtil.isNotBlank(pledgeInfo)){
            if (pledgeInfo.contains("未设立")){
                jsonObject.put("是否设置抵押权",false);
                return jsonObject;
            }
            if (pledgeInfo.contains("已设立")){
                jsonObject.put("是否设置抵押权",true);
                String pledgeType = extractPledgeType(pledgeInfo);
                if (StrUtil.isBlank(pledgeType)){
                    pledgeType = extractPledgeTypeX(pledgeInfo);
                }
                jsonObject.put("抵押权类型",pledgeType);
                String pledgeUser = extractMortgagee(pledgeInfo);
                jsonObject.put("抵押权人",pledgeUser);
                String pledgeValue = extractPledgeValueX(pledgeInfo);
                if (StrUtil.isBlank(pledgeValue)){
                    pledgeValue = extractPledgeValue(pledgeInfo);
                    if (StrUtil.isBlank(pledgeValue)){
                        pledgeValue = extractMortgageValue(pledgeInfo);
                    }
                }
                jsonObject.put("抵押权价值",pledgeValue);
                if (pledgeInfo.contains("向同一抵押权人申请续期贷款")){
                    jsonObject.put("向同一抵押权人申请续期贷款",true);
                }
                jsonObject.put("新抵押权人",extractNewPledgeUser(pledgeInfo));
                if (pledgeInfo.contains("现拟设立最高额抵押权")){
                    jsonObject.put("已设立抵押权现拟设立最高额抵押权同一抵押权人",true);
                }
                if (pledgeInfo.contains("债权可能连续发生")){
                    jsonObject.put("已设立最高额抵押权且抵押未到期",true);
                }
            }
        }
        return jsonObject;
    }

    private static String extractPledgeType(String input){
        Pattern pattern = Pattern.compile("已设立了(.*?)(，|$)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractPledgeTypeX(String input){
        Pattern pattern = Pattern.compile("已设立(.*?)(，|$)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractMortgagee(String input) {
        Pattern pattern = Pattern.compile("抵押权人为([^，]+)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractMortgageValue(String input) {
        Pattern pattern = Pattern.compile("抵押权利价值为([^，]+)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractPledgeValue(String input) {
        Pattern pattern = Pattern.compile("最高债权额([^，]+)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractPledgeValueX(String input) {
        Pattern pattern = Pattern.compile("最高债权额为([^，]+)");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

    private static String extractNewPledgeUser(String input) {
        Pattern pattern = Pattern.compile("向([^ ]+)申请");
        Matcher matcher = pattern.matcher(input);
        if (matcher.find()) {
            return matcher.group(1).trim();
        }
        return "";
    }

}
