package com.dayou.utils.table;

import cn.hutool.core.util.StrUtil;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 类说明： 富文本Html处理器
 *
 * @author: wucl
 * @since: 2023/8/31
 * created with IntelliJ IDEA.
 */
public class RichHtmlHandler {
    private String handledDocBodyBlock; //富文本所在doc中的区域
    private List<String> docBase64BlockResults = new ArrayList<String>(); //文档中图片64位编码的结果区
    private RichObject richObject = null; //富文本对象
    private Document doc = null; //默认文档为空

    public RichHtmlHandler(){
        super();
    }

    public RichHtmlHandler(RichObject richObj) throws IOException {
        this.richObject = richObj;
        doc = Jsoup.parse(richObject.getHtml());
    }


    /**
     * 获取样式的属性值
     * @param style
     * @param attributeKey
     * @return
     */
    private String getStyleAttrValue(String style, String attributeKey) {
        if (StrUtil.isEmpty(style)) {
            return "";
        }
        // 以";"分割
        String[] styleAttrValues = style.split(";");
        for (String item : styleAttrValues) {
            // 在以 ":"分割
            String[] keyValuePairs = item.split(":");
            if (attributeKey.equals(keyValuePairs[0])) {
                return keyValuePairs[1];
            }
        }
        return "";
    }

    /**
     * 给html添加必须的标签
     * @param html
     * @return
     */
    private String wrappHtml(String html){
        // 因为传递过来都是不完整,需要添加以下标签
        StringBuilder sb = new StringBuilder();
        sb.append("<html>");
        sb.append("<body>");
        sb.append(html);
        sb.append("</body>");
        sb.append("</html>");
        return sb.toString();
    }

    public String getHandledDocBodyBlock() {
        String raw = WordHtmlGeneratorHelper.string2Ascii(doc.getElementsByTag("body").html());
        return raw.replace("=3D", "=").replace("=", "=3D");
    }

    public void setHandledDocBodyBlock(String handledDocBodyBlock) {
        this.handledDocBodyBlock = handledDocBodyBlock;
    }

    public List<String> getDocBase64BlockResults() {
        return docBase64BlockResults;
    }

    public void setDocBase64BlockResults(List<String> docBase64BlockResults) {
        this.docBase64BlockResults = docBase64BlockResults;
    }


    public RichObject getRichObject() {
        return richObject;
    }

    public void setRichObject(RichObject richObject) {
        this.richObject = richObject;
    }
}
