package com.dayou.utils.table;

import cn.hutool.core.util.StrUtil;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

import static com.dayou.constants.HouseGuarantyDoc.TARGETS_DOCX;
import static com.dayou.constants.TableHTML.*;
import static com.dayou.enums.HouseTargetTableColumn.ACT_ADDRESS;

@Component
public class WordTableHelper {

    private final String FTL_TEMPLATE_DIR = "/ftl";

    private final String FTL_TEMPLATE = "wordTableTemplate.ftl";


    /**
     *
     * @param html excel表格解析后的html代码
     * @return 生成的word表格的存储路径
     */
    public String createTableWord(String html,String path,String tableName){
        try {
            if (StrUtil.isNotBlank(html)){
                RichObject richObject = new RichObject();
                richObject.setHtml(html);
                RichHtmlHandler richHtmlHandler = WordGeneratorWithFreemarker.createRichHtmlHandler(richObject);
                Map<String,String> data = new HashMap<>();
                data.put("table", richHtmlHandler.getHandledDocBodyBlock());
                String tablePath = path + tableName;
                WordGeneratorWithFreemarker.createDoc("/",FTL_TEMPLATE,data,tablePath);
                return tablePath;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        return null;
    }

    public String htmlTargetTableFormat(String srcHtml){
        Document document = Jsoup.parse(srcHtml);
        Elements table = document.getElementsByTag("table");
        table.attr("style",TABLE_STYLE);
        Elements td = document.select("table td");
        td.removeAttr("width");
        td.attr("style",TD_STYLE);
        return document.body().html();
    }

    /**
     * 删除实勘地址列
     * @param document
     */
    public void removeActAddressCol(Document document){
        Element table = document.select("table").first();
        Elements rows = table.select("tr");
        Elements cols = rows.get(0).select("td");
        int lastCol = cols.size()-1;
        String text = cols.get(lastCol).text();
        if (ACT_ADDRESS.getZhName().equals(text)){
            for (Element row : rows) {
                Elements cells = row.select("td");
                int removeIndex = row.select("td").size() - 1;
                cells.get(removeIndex).remove();
            }
        }

    }

    public String htmlCertificateTableFormat(String certificateTableHtml) {
        Document document = Jsoup.parse(certificateTableHtml);
        Elements table = document.getElementsByTag("table");
        table.attr("style",TABLE_STYLE);
        Elements rows = table.select("tr");
        for (int i = 0; i < rows.size(); i++){
            Elements tds = rows.get(i).select("td");
            for (int j = 0; j < tds.size(); j++){
                if (j==0 || j==1 || j==3 || j==4){
                    tds.get(j).attr("style",TD_STYLE_1);
                }
                else if (j==12){
                    tds.get(j).attr("style",TD_STYLE_3);
                }
                else {
                    tds.get(j).attr("style",TD_STYLE_2);
                }
            }
            if (i!=0){
                //合并权利性质列
                Element td8 = tds.get(8);
                Element td9 = tds.get(9);
                Element td10 = tds.get(10);
                td8.attr("colspan","3");
                String colText1 = td8.text() + td9.text() + td10.text();
                td8.text(colText1);
                td9.remove();
                td10.remove();
                //合并用途列
                Element td11 = tds.get(11);
                Element td12 = tds.get(12);
                Element td13 = tds.get(13);
                td11.attr("colspan","3");
                String colText2 = td11.text() + td12.text() + td13.text();
                td11.text(colText2);
                td12.remove();
                td13.remove();
                //合并面积列
                Element td14 = tds.get(14);
                Element td15 = tds.get(15);
                Element td16 = tds.get(16);
                Element td17 = tds.get(17);
                Element td18 = tds.get(18);
                td14.attr("colspan","5");
                String colText3 = td14.text() + td15.text() + td16.text()+ td17.text() + td18.text();
                td14.text(colText3);
                td15.remove();
                td16.remove();
                td17.remove();
                td18.remove();
            }
        }
        return document.body().html();
    }
}
