package com.dayou.mapper;

import com.dayou.dto.calculate.equipment.EqptQueryDTO;
import com.dayou.entity.AssetsCalculateEqptData;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.dayou.entity.AssetsCalculateEqptImpData;
import com.dayou.entity.AssetsCalculateEqptNonsData;
import com.dayou.vo.AssetsCalculateEqptImpDataVO;
import com.dayou.vo.AssetsCalculateEqptNonsDataVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 机器设备测算数据表 Mapper 接口
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-15
 */
public interface AssetsCalculateEqptDataMapper extends BaseMapper<AssetsCalculateEqptData> {

    /**
     * 获取是否有进口设备
     * @param calculateId 测算表id
     * @return Integer
     */
    Integer hasImportedEquipment(@Param("calculateId") Long calculateId);

    /**
     * 根据测算表id获取所有机器设备导入的数据
     * @param calculateId 测算表id
     * @param equipmentType 设备类型
     * @return List<AssetsCalculateEqptData>
     */
    List<AssetsCalculateEqptData> getAllByAssetsCalculateId(@Param("calculateId") Long calculateId, @Param("equipmentType") String equipmentType);

    /**
     * 分页查询导入的机器设备测算数据
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptData>
     */
    List<AssetsCalculateEqptData> getDataPage(@Param("dto") EqptQueryDTO dto);

    /**
     * 根据测算表id删除
     * @param calculateId 测算表id
     * @return Boolean
     */
    Boolean deleteDataByCalculateId(@Param("calculateId") Long calculateId);

    /**
     * 分页查询进口设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptImpData>
     */
    List<AssetsCalculateEqptImpDataVO> getImpDataPage(@Param("dto") EqptQueryDTO dto);

    /**
     * 查询进口设备信息集合
     * @param calculateId 测算表id
     * @return List<AssetsCalculateEqptImpData>
     */
    List<AssetsCalculateEqptImpDataVO> getImpDataList(@Param("calculateId") Long calculateId);

    /**
     * 根据id更新机器设备测算数据单价
     * @param id id
     * @param price 单价
     */
    void updateDataPrice(@Param("id") Long id, @Param("price") BigDecimal price);

    /**
     * 更新机器设备进口设备数据
     * @param assetsCalculateEqptImpData 进口设备信息
     * @return Boolean
     */
    Boolean updateImpData(@Param("data") AssetsCalculateEqptImpData assetsCalculateEqptImpData);

    /**
     * 新增机器设备进口设备数据
     * @param assetsCalculateEqptImpData 进口设备信息
     * @return Boolean
     */
    Boolean addImpData(@Param("data") AssetsCalculateEqptImpData assetsCalculateEqptImpData);

    /**
     * 根据机器设备信息id删除进口设备信息
     * @param eqptDataId 设备信息id
     * @return Boolean
     */
    Boolean deleteImpSDataByEqptId(@Param("eqptDataId") Long eqptDataId);

    /**
     * 分页查询非标设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    List<AssetsCalculateEqptNonsDataVO> getNonSDataPage(@Param("dto") EqptQueryDTO dto);

    /**
     * 查询非标设备信息集合
     * @param calculateId 测算表id
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    List<AssetsCalculateEqptNonsDataVO> getNonSDataList(@Param("calculateId") Long calculateId);

    /**
     * 更新机器设备非标设备数据
     * @param data 非标设备信息
     * @return Boolean
     */
    Boolean updateNonSData(@Param("data") AssetsCalculateEqptNonsData data);

    /**
     * 新增机器设备非标设备数据
     * @param data 非标设备信息
     * @return Boolean
     */
    Boolean addNonSData(@Param("data") AssetsCalculateEqptNonsData data);

    /**
     * 根据机器设备信息id删除非标设备信息
     * @param eqptDataId 设备信息id
     * @return Boolean
     */
    Boolean deleteNonSDataByEqptId(@Param("eqptDataId") Long eqptDataId);

}
