package com.dayou.mapper;

import com.dayou.common.DropDownBoxData;
import com.dayou.entity.AssetsCalculate;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.dayou.vo.AssetsCalculateVO;
import com.dayou.vo.CRReportReferencesCalculateVO;
import com.dayou.vo.EqptReportReferencesCalculateVO;
import com.dayou.vo.calculate.AssetsCalculateProgressVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 资产项目测算信息表 Mapper 接口
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-13
 */
public interface AssetsCalculateMapper extends BaseMapper<AssetsCalculate> {

    /**
     * 根据项目id获取未完成测算表的进度信息
     * @param projectId 项目id
     * @return AssetsCalculateStepBaseVO
     */
    AssetsCalculateProgressVO getCalculateProgress(@Param("projectId") Long projectId);

    /**
     * 通过id更新测算表进度
     * @param calculateId id
     * @param progress 进度
     * @return Boolean
     */
    Boolean updateCalculateProgress(@Param("calculateId") Long calculateId, @Param("progress") String progress);

    /**
     * 更新测算表基础测算信息
     * @param calculateId id
     * @param baseInfo 基础信息json字符串
     * @return Boolean
     */
    Boolean updateCalculateBaseInfo(@Param("calculateId") Long calculateId, @Param("baseInfo") String baseInfo);

    /**
     * 根据项目id查询所有测算表信息
     * @param projectId 项目id
     * @return List<AssetsCalculateVO>
     */
    List<AssetsCalculateVO> listAllByProjectId(@Param("projectId") Long projectId);

    /**
     * 获取新增DocumentProduction所需的测算信息
     * @param calculateId 测算表id
     * @return AssetsCalculateVO
     */
    AssetsCalculateVO getCalculateInfoForDoc(@Param("calculateId") Long calculateId);

    /**
     * 根据测算表id获取基础测算信息
     *
     * @param calculateId 测算表id
     * @return String
     */
    String getCalculateBaseInfo(@Param("calculateId") Long calculateId);

    /**
     * 根据项目id获取测算信息下拉框数据
     * @param projectId 项目id
     * @return List<DropDownBoxData>
     */
    List<DropDownBoxData> getDropDownBoxData(@Param("projectId") Long projectId);

    /**
     * 根据测算表id获取生成机器设备融资项目报告引用的测算表信息
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    EqptReportReferencesCalculateVO getEqptReferencesInfoById(@Param("calculateId") Long calculateId);

    /**
     * 根据测算表id获取生成债权分析报告引用的测算表信息
     * @param calculateId 测算表id
     * @return EqptReportReferencesCalculateVO
     */
    CRReportReferencesCalculateVO getCRReferencesInfoById(@Param("calculateId") Long calculateId);

}
