package com.dayou.mapper;

import com.dayou.bo.CRReportFillBO;
import com.dayou.bo.EqptReportFillBO;
import com.dayou.dto.report.ReportBaseInfoDTO;
import com.dayou.entity.AssetsReport;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.dayou.vo.AssetsReportVO;
import com.dayou.vo.report.AssetsReportProgressVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * <p>
 * 资产项目报告信息表 Mapper 接口
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-12-06
 */
public interface AssetsReportMapper extends BaseMapper<AssetsReport> {

    /**
     * 根据项目id查询所有报告信息
     * @param projectId 项目id
     * @return List<AssetsReport>
     */
    List<AssetsReportVO> listAllByProjectId(@Param("projectId") Long projectId);

    /**
     * 根据项目id获取未完成报告的进度信息
     * @param projectId 项目id
     * @return AssetsReportProgressVO
     */
    AssetsReportProgressVO getUnFinishedReportProgress(@Param("projectId") Long projectId);

    /**
     * 更新报告基础信息
     * @param reportBaseInfoDTO dto
     * @return Boolean
     */
    Boolean updateReportBaseInfo(@Param("reportBaseInfoDTO") ReportBaseInfoDTO reportBaseInfoDTO);

    /**
     * 通过id更新报告进度
     * @param reportId id
     * @param progress 进度
     * @return Boolean
     */
    Boolean updateReportProgress(@Param("reportId") Long reportId, @Param("progress") String progress);

    /**
     * 根据报告id获取报告基础信息
     * @param reportId 报告id
     * @return String
     */
    String getReportBaseInfo(@Param("reportId") Long reportId);

    /**
     * 根据报告id获取生成机器设备报告word文档所需信息
     * @param reportId 报告id
     * @return EqptReportFillBO
     */
    EqptReportFillBO getEqptReportFillBO(@Param("reportId") Long reportId);

    /**
     * 根据报告id获取生成债权分析报告word文档所需信息
     * @param reportId 报告id
     * @return CRReportFillBO
     */
    CRReportFillBO getCRReportFillBO(@Param("reportId") Long reportId);

    /**
     * 获取新增DocumentProduction所需的报告信息
     * @param reportId 报告id
     * @return AssetsReportVO
     */
    AssetsReportVO getReportInfoForDoc(@Param("reportId") Long reportId);

}
