package com.dayou.bo;

import com.alibaba.excel.annotation.ExcelProperty;
import com.dayou.annotation.WordTableColumn;
import com.dayou.converter.NullToEmptyStringConverter;
import com.dayou.dto.report.cr.CRReportBaseInfoBO;
import lombok.Data;

import java.math.BigDecimal;
import java.util.List;

@Data
public class CRReportFillBO {

    /**
     * 项目id
     */
    private Long projectId;

    /**
     * 测算表id
     */
    private Long calculateId;

    /**
     * 产品号
     */
    private String productionNo;

    /**
     * 中文日期
     */
    private String chineseReportDate;

    /**
     * 产权持有人拼接后的字符串(多个的情况)
     */
    private String ownerCompanyName;

    /**
     * 贷款本金
     */
    private BigDecimal loanPrincipal;

    /**
     * 贷款本金(千分位格式)
     */
    private String loanPrincipalStr;

    /**
     * 应收利息（含罚息）
     */
    private BigDecimal interest;

    /**
     * 应收利息（含罚息）(千分位格式)
     */
    private String interestStr;

    /**
     * 其他费用
     */
    private BigDecimal otherExpenses;

    /**
     * 其他费用(千分位格式)
     */
    private String otherExpensesStr;

    /**
     * 各项债权合计
     */
    private BigDecimal CRTotal;

    /**
     * 各项债权合计(千分位格式)
     */
    private String CRTotalStr;

    /**
     * 债权综合受偿比率
     */
    private String compositeRecoveryRatio;

    /**
     * 债权清算价值合计
     */
    private BigDecimal totalLiquidationValue;

    /**
     * 债权清算价值合计(千分位格式)
     */
    private String totalLiquidationValueStr;

    /**
     * 优先受偿价值合计
     */
    private BigDecimal totalPriorityClaimValue;

    /**
     * 优先受偿价值合计(千分位格式)
     */
    private String totalPriorityClaimValueStr;

    /**
     * 一般受偿价值合计
     */
    private BigDecimal totalGeneralClaimValue;

    /**
     * 一般受偿价值合计(千分位格式)
     */
    private String totalGeneralClaimValueStr;

    /**
     * 保证受偿价值合计
     */
    private BigDecimal totalGuaranteedClaimValue;

    /**
     * 保证受偿价值合计(千分位格式)
     */
    private String totalGuaranteedClaimValueStr;

    /**
     * 债权可回收金额合计
     */
    private BigDecimal totalRecoverableAmount;

    /**
     * 债权可回收金额合计(千分位格式)
     */
    private String totalRecoverableAmountStr;

    /**
     * 债权清算价值合计
     */
    private BigDecimal totalCreditorLiquidationValue;

    /**
     * 债权清算价值合计(千分位格式)
     */
    private String totalCreditorLiquidationValueStr;

    /**
     * 债权分析报告基础信息
     */
    private CRReportBaseInfoBO reportBaseInfo;

    /**
     * 本次所需处置时间
     */
    private String timeNeededForThisDisposal;

    /**
     * 本次清算率取值
     */
    private String liquidationRateForThis;

    /**
     * 债权汇总万元表信息
     */
    private List<Consolidated> consolidatedList;

    /**
     * 综合因素分析表数据
     */
    private List<IntegratedAnalysis> integratedAnalysisList;

    @Data
    public static class Consolidated {

        /**
         * 借款人
         */
        @ExcelProperty("借款人")
        private String debtorName;

        /**
         * 债权本金余额
         */
        @ExcelProperty("债权本金余额")
        private String debtPrincipalBalance;

        /**
         * 债权利息及费用
         */
        @ExcelProperty("债权利息及费用")
        private String debtInterestAndFees;

        /**
         * 实现债权费用
         */
        @ExcelProperty("实现债权费用")
        private String creditorRealizationExpenses;

        /**
         * 债权合计
         */
        @ExcelProperty("债权合计")
        private String CRTotal;

        /**
         * 优先受偿价值
         */
        @ExcelProperty("优先受偿价值")
        private String priorityClaimValue;

        /**
         * 一般受偿价值
         */
        @ExcelProperty("一般受偿价值")
        private String generalClaimValue;

        /**
         * 保证受偿价值
         */
        @ExcelProperty("保证受偿价值")
        private String guaranteedClaimValue;

        /**
         * 债权可回收金额
         */
        @ExcelProperty("债权可回收金额")
        private String recoverableAmount;

        /**
         * 债权清算折扣率
         */
        @ExcelProperty("债权清算折扣率")
        private String creditorLiquidationDiscountRate;

        /**
         * 债权清算价值
         */
        @ExcelProperty("债权清算价值")
        private String creditorLiquidationValue;

        /**
         * 本金受偿率
         */
        @ExcelProperty("本金受偿率")
        private String principalRecoveryRate;

        /**
         * 债权综合受偿率
         */
        @ExcelProperty("债权综合受偿率")
        private String comprehensiveRecoveryRate;

    }

    /**
     * 综合因素分析
     */
    @Data
    public static class IntegratedAnalysis {

        /**
         * 折扣
         */
        @ExcelProperty(value = "折扣", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "折扣", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String discount;

        /**
         * 权重
         */
        @ExcelProperty(value = "权重", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "权重", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String weight;

        /**
         * 因素
         */
        @ExcelProperty(value = "因素", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "因素", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String factor;

        /**
         * 序号
         */
        @ExcelProperty(value = "序号", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "序号", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String serialNumber;

        /**
         * 项目
         */
        @ExcelProperty(value = "项目", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "项目", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8, cellWidth = 60)
        private String project;

        /**
         * 因素描述
         */
        @ExcelProperty(value = "因素描述", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "因素描述", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8, cellWidth = 83)
        private String factorDescription;

        /**
         * 分值
         */
        @ExcelProperty(value = "分值", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "分值", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String score;

        /**
         * 取值
         */
        @ExcelProperty(value = "取值", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "取值", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String value;

        /**
         * 权重后
         */
        @ExcelProperty(value = "权重后", converter = NullToEmptyStringConverter.class)
        @WordTableColumn(name = "权重后", fontName = "华文细黑", fontSize = 7.5, titleBold = true, titleFontSize = 8)
        private String weighted;
    }


}
