package com.dayou.common;

/**
 * 测算表公式
 */
public class CalculateFormula {

    /**
     * 机器设备主模板公式,占位符{0,number,#}是行号
     * AT列 1~4 行是固定的
     */
    public static final EquipmentMainFormula[] EQUIPMENT_MAIN_FORMULA = EquipmentMainFormula.values();

    /**
     * 机器设备进口设备模板公式,占位符{0,number,#}是行号
     */
    public static final EquipmentImportedFormula[] EQUIPMENT_IMPORTED_FORMULA = EquipmentImportedFormula.values();

    /**
     * 机器设备非标设备模板公式,占位符{0,number,#}是行号
     */
    public static final EquipmentNonStandardFormula[] EQUIPMENT_NONSTANDARD_FORMULA = EquipmentNonStandardFormula.values();

    /**
     * 机器设备主模板公式
     */
    public enum EquipmentMainFormula {
        不含税单价("ROUND(M{0,number,#}/(1+N{0,number,#}),2)",15),
        原值("AU{0,number,#}",17),
        成新率("AC{0,number,#}",18),
        净值("ROUND(Q{0,number,#}*R{0,number,#},0)",19),
        原值增值率("ROUND((Q{0,number,#}-K{0,number,#})/K{0,number,#},4)", 20),
        净值增值率("ROUND((S{0,number,#}-L{0,number,#})/L{0,number,#},4)", 21),
        已使用年数("ROUND(($Y$2-F{0,number,#})/365,2)",23),
        年限成新率("IF((V{0,number,#}-W{0,number,#})/V{0,number,#}<15%,ROUND(X{0,number,#}/(W{0,number,#}+X{0,number,#}),2),ROUND((V{0,number,#}-W{0,number,#})/V{0,number,#},2))",25),
        综合成新率("ROUND(Y{0,number,#}*Z{0,number,#}+AA{0,number,#}*AB{0,number,#},2)",29),
        报价("AI{0,number,#}*I{0,number,#}",36),
        运杂费("ROUND(AJ{0,number,#}*AK{0,number,#},0)",38),
        基础费("ROUND(AJ{0,number,#}*AM{0,number,#},0)",40),
        安装费("AJ{0,number,#}*AO{0,number,#}",42),
        设备工程费("ROUND(AJ{0,number,#}+AL{0,number,#}+AN{0,number,#}+AP{0,number,#},0)",43),
        前期费("ROUND(AQ{0,number,#}*$AX$3,0)",44),
        资金成本("ROUND((AR{0,number,#}*((1+$AX$2)^($AX$1)-1)+AQ{0,number,#}*((1+$AX$2)^($AX$1/2)-1)),0)",45),
        可抵扣增值税("ROUND(AJ{0,number,#}/(1+13%)*13%+(AL{0,number,#}+AN{0,number,#}+AP{0,number,#})/(1+9%)*9%+AQ{0,number,#}*($AX$3-$AX$4)/1.06*0.06,0)",46),
        重置全价("ROUND(AQ{0,number,#}+AR{0,number,#}+AS{0,number,#}-AT{0,number,#},0)",47);

        /**
         * 公式
         */
        private String formula;

        /**
         * 列下标
         */
        private Integer column;

        EquipmentMainFormula(String formula, Integer column) {
            this.formula = formula;
            this.column = column;
        }

        public String getFormula() {
            return formula;
        }

        public Integer getColumn() {
            return column;
        }

        @Override
        public String toString() {
            return column + ":" + formula;
        }


    }

    /**
     * 机器设备进口设备模板公式
     */
    public enum EquipmentImportedFormula {
        购置价CIF到岸价("IF(M{0,number,#}=\"FOB\",(N{0,number,#}*(1+3%+0.4%)*Q{0,number,#}),N{0,number,#}*Q{0,number,#})",19),
        CIF到岸价折人民币("ROUND(T{0,number,#}*S{0,number,#},0)",21),
        关税("U{0,number,#}*V{0,number,#}",24),
        进口增值税("(U{0,number,#}+X{0,number,#})*Y{0,number,#}",26),
        设备价格("SUM(U{0,number,#},X{0,number,#},Z{0,number,#})",27),
        银行财务费("U{0,number,#}*AB{0,number,#}",29),
        外贸手续费("U{0,number,#}*AD{0,number,#}",31),
        费用小计("SUM(AC{0,number,#},AE{0,number,#})",32),
        设备原价("AA{0,number,#}+AF{0,number,#}",33);

        /**
         * 公式
         */
        private String formula;

        /**
         * 列下标
         */
        private Integer column;

        EquipmentImportedFormula(String formula, Integer column) {
            this.formula = formula;
            this.column = column;
        }

        public String getFormula() {
            return formula;
        }

        public Integer getColumn() {
            return column;
        }

        @Override
        public String toString() {
            return column + ":" + formula;
        }

    }

    /**
     * 机器设备非标设备模板公式
     */
    public enum EquipmentNonStandardFormula {
        主材费设备报价含税不含运费不含安装调试费("ROUND(O{0,number,#}/P{0,number,#}*Q{0,number,#}+(IF(R{0,number,#}=\"无\",0,S{0,number,#}/T{0,number,#}*U{0,number,#})),0)", 22),
        非标设备估值("ROUND((V{0,number,#}/W{0,number,#}+X{0,number,#})*(1+Y{0,number,#})*(1+Z{0,number,#})*(1+AA{0,number,#})/(1+AB{0,number,#}),0)",29);

        /**
         * 公式
         */
        private String formula;

        /**
         * 列下标
         */
        private Integer column;

        EquipmentNonStandardFormula(String formula, Integer column) {
            this.formula = formula;
            this.column = column;
        }

        public String getFormula() {
            return formula;
        }

        public Integer getColumn() {
            return column;
        }

        @Override
        public String toString() {
            return column + ":" + formula;
        }

    }

}
