package com.dayou.dto.report.cr;

import com.alibaba.excel.annotation.ExcelProperty;
import lombok.Data;

import java.util.List;

@Data
public class CRReportBaseInfoBO {

    /**
     * 债务人
     */
    private String debtor;

    /**
     * 评估目的
     */
    private String purpose;

    /**
     * 评估基准日
     */
    private String valuationBasisDate;

    /**
     * 评估报告日
     */
    private String reportDate;

    /**
     * 报告有效期
     */
    private String reportValidity;

    /**
     * 委托人概况
     */
    private ConsignorInfo consignorInfo;

    /**
     * 债务人信息集合
     */
    private List<DebtorInfo> debtorInfoList;

    /**
     * 保证人(自然人)集合
     */
    private List<PersonGuarantor> personGuarantorList;

    /**
     * 保证人(企业单位)集合
     */
    private List<EnterpriseGuarantor> enterpriseGuarantorList;

    /**
     * 资产情况
     */
    private List<Asset> assetList;

    /**
     * 诉讼情况
     */
    private List<Lawsuit> lawsuitList;

    /**
     * 债务人一般受偿能力分析
     */
    private String repaymentCapacityAnalysis;

    /**
     * 委托人概况
     */
    @Data
    public static class ConsignorInfo {

        /**
         * 公司名称
         */
        private String consignorCompanyName;

        /**
         * 营业场所
         */
        private String businessLocation;

        /**
         * 统一社会信用代码
         */
        private String consignorUnifiedSocialCreditCode;

        /**
         * 类型
         */
        private String consignorType;

        /**
         * 负责人
         */
        private String consignorLegalRepresentative;

        /**
         * 成立日期
         */
        private String consignorEstablishmentDate;

        /**
         * 经营范围
         */
        private String consignorBusinessScope;

    }

    /**
     * 债务人信息
     */
    @Data
    public static class DebtorInfo {

        /**
         * 序号
         */
        private Integer sort;

        /**
         * 公司名称
         */
        private String debtorName;

        /**
         * 统一社会信用代码
         */
        private String debtorUnifiedSocialCreditCode;

        /**
         * 住所
         */
        private String debtorResidence;

        /**
         * 注册资本
         */
        private String debtorRegisteredCapital;

        /**
         * 法定代表人
         */
        private String debtorLegalRepresentative;

        /**
         * 企业性质
         */
        private String debtorType;

        /**
         * 成立日期
         */
        private String debtorEstablishmentDate;

        /**
         * 营业期限
         */
        private String debtorBusinessOperationPeriod;

        /**
         * 经营范围
         */
        private String debtorBusinessScope;

    }

    /**
     * 保证人(自然人类型)
     */
    @Data
    public static class  PersonGuarantor {

        /**
         * 序号
         */
        private Integer sort;

        /**
         * 保证人
         */
        private String guarantorName;

        /**
         * 民族
         */
        private String guarantorEthnicGroup;

        /**
         * 出生年月
         */
        private String guarantorDateOfBirth;

        /**
         * 身份证号码
         */
        private String guarantorIdentificationNumber;

        /**
         * 住址
         */
        private String guarantorResidence;

        /**
         * 保证人一般受偿能力分析
         */
        private String repaymentCapacityAnalysis;

    }

    /**
     * 保证人(企业单位)
     */
    @Data
    public static class EnterpriseGuarantor {

        /**
         * 序号
         */
        private Integer sort;

        /**
         * 保证人
         */
        private String guarantorName;

        /**
         * 统一社会信用代码
         */
        private String guarantorUnifiedSocialCreditCode;

        /**
         * 类型
         */
        private String guarantorType;

        /**
         * 注册资本
         */
        private String guarantorRegisteredCapital;

        /**
         * 法定代表人
         */
        private String guarantorLegalRepresentative;

        /**
         * 成立日期
         */
        private String guarantorEstablishmentDate;

        /**
         * 营业期限
         */
        private String guarantorBusinessOperationPeriod;

        /**
         * 住所
         */
        private String guarantorResidence;

        /**
         * 经营范围
         */
        private String guarantorBusinessScope;

        /**
         * 保证人一般受偿能力分析
         */
        private String repaymentCapacityAnalysis;
    }

    /**
     * 资产情况
     */
    @Data
    public static class Asset {

        /**
         * 序号
         */
        private Integer sort;

        /**
         * 类型(债务人或保证人)
         */
        @ExcelProperty("债务人或保证人")
        private String type;

        /**
         * 名字
         */
        @ExcelProperty("企业")
        private String name;

        /**
         * 目前状态
         */
        private String status;

        /**
         * 房产
         */
        @ExcelProperty("房产")
        private String realEstate;

        /**
         * 土地
         */
        @ExcelProperty("土地")
        private String land;

        /**
         * 车辆
         */
        @ExcelProperty("车辆")
        private String car;

        /**
         * 对外投资
         */
        @ExcelProperty("对外投资")
        private String investment;

        /**
         * 其他资产
         */
        @ExcelProperty("其他资产")
        private String other;
    }

    @Data
    public static class Lawsuit {
        /**
         * 序号
         */
        @ExcelProperty("序号")
        private Integer sort;

        /**
         * 债务人或保证人
         */
        @ExcelProperty("债务人或保证人")
        private String name;

        /**
         * 诉讼信息
         */
        @ExcelProperty("诉讼信息")
        private String info;
    }

}
