package com.dayou.enums;


public enum BusinessEnum implements CodeMsgEnumInterface<String,String>{

    HOUSE_BUSINESS("HOUSE_BUSINESS","房地产", HouseSubBusiness.values()),
    HOUSE_GUARANTY_PROCESS("HOUSE_GUARANTY_PROCESS","房地产抵押节点",HouseGuarantyProcess.values()),
    HOUSE_GUARANTY_RULES("HOUSE_GUARANTY_RULES","房地产抵押文档规则",HouseGuarantyRules.values()),
    ;
    private String code;

    private String msg;

    private CodeMsgEnumInterface[] subBusiness;

    BusinessEnum(String code, String msg, CodeMsgEnumInterface[] subBusiness) {
        this.code = code;
        this.msg = msg;
        this.subBusiness = subBusiness;
    }

    @Override
    public String getCode() {
        return code;
    }

    @Override
    public String getMsg() {
        return msg;
    }

    public enum HouseSubBusiness implements CodeMsgEnumInterface<String,String>{

        GUARANTY("GUARANTY","抵押"),

        CONSULT("CONSULT","咨询"),

        ARMY("ARMY","军队");



        private String code;

        private String msg;

        private HouseSubBusiness(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        @Override
        public String getCode() {
            return code;
        }

        @Override
        public String getMsg() {
            return msg;
        }
    }


    public enum HouseGuarantyProcess implements CodeMsgEnumInterface<String,String>{

        CONSIGNOR_LETTER("CONSIGNOR_LETTER","《致委托人函》","consignor"),

        STATEMENT_CONDITIONS_EXPLAIN("STATEMENT_CONDITIONS_EXPLAIN","《假设和限制条件及使用报告说明》","conditions"),

        GUARANTY_RESULT_REPORT("GUARANTY_RESULT_REPORT","《房地产抵押估价结果报告》","guarantyResult"),
        ;

        private String code;

        private String msg;

        private String router;


        HouseGuarantyProcess(String code, String msg,String router){
            this.code = code;
            this.msg = msg;
            this.router = router;
        }

        @Override
        public String getCode() {
            return code;
        }

        @Override
        public String getMsg() {
            return msg;
        }

        public String getRouter() {
            return router;
        }
    }

    public enum HouseGuarantyChapter implements CodeMsgEnumInterface<String,String>{

        ORDINARY_ASSUMPTION("ORDINARY_ASSUMPTION","一般假设"),

        UNDEFINED_ASSUMPTION("UNDEFINED_ASSUMPTION","未定事项假设"),

        VIOLATE_ASSUMPTION("VIOLATE_ASSUMPTION","背离事实假设"),

        UNCONFORMITY_ASSUMPTION("UNCONFORMITY_ASSUMPTION","不相一致假设"),

        DEFICIENCY_ASSUMPTION("DEFICIENCY_ASSUMPTION","依据不足假设"),

        LIMIT_CONDITION("LIMIT_CONDITION","限制条件"),

        USE_REPORT_EXPLAIN("USE_REPORT_EXPLAIN","使用报告说明"),

        SPECIAL_HANDLE("SPECIAL_HANDLE","估价中的特殊处理事项")
        ;

        HouseGuarantyChapter(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        private String code;

        private String msg;

        @Override
        public String getCode() {
            return code;
        }

        @Override
        public String getMsg() {
            return msg;
        }
    }

    public enum HouseGuarantyRules implements CodeMsgEnumInterface<String,String>{

        IS_CHECK_ORIGIN_CERTIFICATE("IS_CHECK_ORIGIN_CERTIFICATE","是否核验权属资料原件"),
        IS_CHECK_ORIGIN_CERTIFICATE_X("IS_CHECK_ORIGIN_CERTIFICATE_X","是否核验权属资料原件"),
        IS_CHECK_ORIGIN_CERTIFICATE_XX("IS_CHECK_ORIGIN_CERTIFICATE_XX","是否核验权属资料原件"),
        IS_BUILDING_PART("IS_BUILDING_PART","估价对象为整个房地产中的一部分"),
        IS_HOTEL_PART("IS_HOTEL_PART","估价对象为整个酒店房地产中的一部分"),
        IS_REGISTER_BUILDING_YEAR("IS_REGISTER_BUILDING_YEAR","是否登记建成年代"),
        IS_USE_DIFFERENT("IS_USE_DIFFERENT","权属证书用途不一致"),
        IS_PROVIDE_LAND_BIG_CERTIFICATE("IS_PROVIDE_LAND_BIG_CERTIFICATE","未提供分户土地证，提供宗地大证"),
        IS_PROVIDE_LAND_BIG_CERTIFICATE_SHARE("IS_PROVIDE_LAND_BIG_CERTIFICATE_SHARE","未提供分户土地证，提供宗地大证需要采用成本法，分摊土地面积处理方式"),
        IS_DIVIDE_METHOD1("IS_DIVIDE_METHOD1","房屋分层面积处理方式一"),
        IS_DIVIDE_METHOD2("IS_DIVIDE_METHOD2","房屋分层面积处理方式二"),
        IS_ADDRESS_DIFF_CERTIFICATE("IS_ADDRESS_DIFF_CERTIFICATE","登记地址与实勘地址不一致（有证明）"),
        IS_ADDRESS_DIFF_NO_CERTIFICATE("IS_ADDRESS_DIFF_NO_CERTIFICATE","登记地址与实勘地址不一致（无证明）"),
        IS_DATE_DIFF("IS_DATE_DIFF","价值时点与查勘日期不一致"),
        IS_BIG_CERTIFICATE_ONLY_HOUSE("IS_BIG_CERTIFICATE_ONLY_HOUSE","未办理分户土地证和提供所在宗地大证仅限住宅商品房"),
        IS_NOT_ORIGIN_CERTIFICATE_AND_PROMISE_SAME("IS_NOT_ORIGIN_CERTIFICATE_AND_PROMISE_SAME","必须在“估价对象权利状况说明及承诺书”中说明未提供上述权属证书原件，并承诺提供的权属证书复印件与原件一致"),
        IS_NOT_ORIGIN_FILES_IN_CHENGDU("IS_NOT_ORIGIN_FILES_IN_CHENGDU","承诺提供的权属证书及与估价对象相关的法律文件复印件与原件一致（大成都范围内）"),
        IS_NOT_ORIGIN_FILES_OUT_CHENGDU("IS_NOT_ORIGIN_FILES_OUT_CHENGDU","承诺提供的权属证书及与估价对象相关的法律文件复印件与原件一致（大成都范围外）"),
        IS_RENT_OUT_NOT_CONTRACT_NO_AFFECT("IS_RENT_OUT_NOT_CONTRACT_NO_AFFECT","已出租未提供租赁合同且未考虑租约影响"),
        IS_NOT_CONTRACT_LAND_LOWER_HOUSE_SY("IS_NOT_CONTRACT_LAND_LOWER_HOUSE_SY","未提供土地出让合同且土地使用年限低于房屋年限且使用收益法评估"),
        IS_NOT_CONTRACT_LAND_LOWER_HOUSE_CB("IS_NOT_CONTRACT_LAND_LOWER_HOUSE_CB","未提供土地出让合同且土地使用年限低于房屋年限且使用成本法评估"),
        IS_NOT_PLEDGE("IS_NOT_PLEDGE","估价对象未设立抵押权"),
        IS_OLD_PLEDGE_TO_NEW_PLEDGE("IS_OLD_PLEDGE_TO_NEW_PLEDGE","适用于原设立抵押权未注销，向新的抵押权人申请贷款"),
        IS_OLD_PLEDGE_TO_OLD_PLEDGE("IS_OLD_PLEDGE_TO_OLD_PLEDGE","适用于原抵押权未注销，现拟设立最高额抵押权，同一抵押权人"),
        IS_TRANSFER_NOT_DEDUCT_SELL_MONEY("IS_TRANSFER_NOT_DEDUCT_SELL_MONEY","土地使用权未划拨但未扣除出让金"),
        IS_TRANSFER_DEDUCT_SELL_MONEY("IS_TRANSFER_DEDUCT_SELL_MONEY","土地使用权未划拨已扣除出让金"),
        IS_PROVIDE_TECH_REPORT("IS_PROVIDE_TECH_REPORT","是否提供技术报告"),
        IS_NS_BANK("IS_NS_BANK","是否是农商银行"),
        IS_NORMAL_BANK("IS_NORMAL_BANK","其他银行"),
        IS_NO_FIRST_MONEY("IS_NO_FIRST_MONEY","无法定优先受偿款"),
        IS_SAME_PLEDGE_USER_CONTINUE("IS_SAME_PLEDGE_USER_CONTINUE","适用于同一抵押权人续贷"),
        IS_OLD_PLEDGE_TO_NEW_PLEDGE_SPECIAL("IS_OLD_PLEDGE_TO_NEW_PLEDGE_SPECIAL","适用于原设立抵押权未注销，向新的抵押权人申请贷款"),
        IS_NORMAL_PLEDGE_TO_HIGH_PLEDGE("IS_NORMAL_PLEDGE_TO_HIGH_PLEDGE","适用于原已设立抵押权，现拟设立最高额抵押权，同一抵押权人"),
        IS_HIGH_PLEDGE_NOT_EXPIRE("IS_HIGH_PLEDGE_NOT_EXPIRE","适用于已设立最高额抵押权，抵押未到期"),
        IS_NOT_RENT_OUT("IS_NOT_RENT_OUT","估价对象未出租"),
        IS_RENT_OUT_NOT_AFFECT_HIGH("IS_RENT_OUT_NOT_AFFECT_HIGH","已出租，未考虑租约影响：高于市场租金"),
        IS_RENT_OUT_NOT_AFFECT_NORMAL("IS_RENT_OUT_NOT_AFFECT_NORMAL","已出租，未考虑租约影响：与市场租金相当"),
        IS_RENT_OUT_AFFECT_LOW("IS_RENT_OUT_AFFECT_LOW","已出租，考虑租约影响：低于市场租金"),
        IS_HAS_IMMOVABLE_CERTIFICATE("IS_HAS_IMMOVABLE_CERTIFICATE","是否提供不动产权证")




        ;

        HouseGuarantyRules(String code, String msg) {
            this.code = code;
            this.msg = msg;
        }

        private String code;

        private String msg;

        @Override
        public String getCode() {
            return code;
        }

        @Override
        public String getMsg() {
            return msg;
        }
    }
}
