package com.dayou.service;
import com.dayou.dto.calculate.equipment.EqptQueryDTO;
import com.dayou.entity.AssetsCalculateEqptData;
import com.baomidou.mybatisplus.extension.service.IService;
import com.dayou.entity.AssetsCalculateEqptImpData;
import com.dayou.entity.AssetsCalculateEqptNonsData;
import com.dayou.vo.AssetsCalculateEqptImpDataVO;
import com.dayou.vo.AssetsCalculateEqptNonsDataVO;

import java.util.List;

/**
 * <p>
 * 机器设备测算数据表 服务类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-15
 */
public interface AssetsCalculateEqptDataService extends IService<AssetsCalculateEqptData> {

    /**
     * 新增机器设备信息
     * @param assetsCalculateEqptData 设备信息
     * @return Boolean
     */
    Boolean addEqptData(AssetsCalculateEqptData assetsCalculateEqptData);

    /**
     * 获取是否有进口设备
     * @param calculateId 测算表id
     * @return Boolean
     */
    Boolean hasImportedEquipment(Long calculateId);

    /**
     * 根据测算表id获取所有机器设备导入的数据
     * @param calculateId 测算表id
     * @param equipmentType 设备类型
     * @return List<AssetsCalculateEqptData>
     */
    List<AssetsCalculateEqptData> getAllByAssetsCalculateId(Long calculateId, String equipmentType);

    /**
     * 分页查询导入的机器设备测算数据
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptData>
     */
    List<AssetsCalculateEqptData> getDataPage(EqptQueryDTO dto);

    /**
     * 根据测算表id删除
     * @param calculateId 测算表id
     * @return Boolean
     */
    Boolean deleteDataByCalculateId(Long calculateId);

    /**
     * 更新机器设备数据
     * @param assetsCalculateEqptData 机器设备信息
     * @return Boolean
     */
    Boolean updateData(AssetsCalculateEqptData assetsCalculateEqptData);

    /**
     * 批量更新
     * @param dataList 测算信息集合
     * @return Boolean
     */
    Boolean updateBatch(List<AssetsCalculateEqptData> dataList);

    /**
     * 分页查询进口设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptImpData>
     */
    List<AssetsCalculateEqptImpDataVO> getImpDataPage(EqptQueryDTO dto);

    /**
     * 查询进口设备信息集合
     * @param calculateId 测算表id
     * @return Page<AssetsCalculateEqptImpData>
     */
    List<AssetsCalculateEqptImpDataVO> getImpDataList(Long calculateId);

    /**
     * 更新机器设备进口设备数据
     * @param assetsCalculateEqptImpData 进口设备信息
     * @return Boolean
     */
    Boolean updateImpData(AssetsCalculateEqptImpData assetsCalculateEqptImpData);

    /**
     * 批量更新机器设备进口设备数据
     * @param dataList 进口设备信息
     * @return Boolean
     */
    Boolean updateBatchImpData(List<AssetsCalculateEqptImpData> dataList);

    /**
     * 分页查询非标设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    List<AssetsCalculateEqptNonsDataVO> getNonSDataPage(EqptQueryDTO dto);

    /**
     * 查询非标设备信息集合
     * @param calculateId 测算表id
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    List<AssetsCalculateEqptNonsDataVO> getNonSDataList(Long calculateId);

    /**
     * 更新机器设备非标设备数据
     * @param assetsCalculateEqptNonsData 非标设备信息
     * @return Boolean
     */
    Boolean updateNonSData(AssetsCalculateEqptNonsData assetsCalculateEqptNonsData);

    /**
     * 批量更新机器设备非标设备数据
     * @param dataList 非标设备信息
     * @return Boolean
     */
    Boolean updateBatchNonSData(List<AssetsCalculateEqptNonsData> dataList);

    /**
     * 根据机器设备信息id删除
     * @param eqptId 设备信息id
     * @return Boolean
     */
    Boolean deleteDqptDataById(Long eqptId);

}
