package com.dayou.service;
import com.dayou.common.DropDownBoxData;
import com.dayou.entity.AssetsCalculate;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dayou.vo.AssetsCalculateVO;
import com.dayou.vo.CRReportReferencesCalculateVO;
import com.dayou.vo.EqptReportReferencesCalculateVO;
import com.dayou.vo.calculate.AssetsCalculateProgressVO;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;

/**
 * <p>
 * 资产项目测算信息表 服务类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-13
 */
public interface AssetsCalculateService extends IService<AssetsCalculate> {

        /**
         * 根据项目id查询所有测算表信息
         * @param projectId 项目id
         * @return List<AssetsCalculateVO>
         */
        List<AssetsCalculateVO> listAllByProjectId(Long projectId);

        AssetsCalculate detail(Long id);

        /**
         * 新增或修改测算表信息
         * @param assetsCalculate 测算表信息
         * @return Boolean
         */
        Boolean add(AssetsCalculate assetsCalculate);

        Boolean update(AssetsCalculate assetsCalculate);

        Boolean delete(Long id);

        /**
         * 根据项目id获取未完成测算表的进度信息
         * @param projectId 项目id
         * @return AssetsCalculateStepBaseVO
         */
        AssetsCalculateProgressVO getUnFinishedCalculateProgress(Long projectId);

        /**
         * 导入保存机器设备基础信息
         * @param multipartFile Excel文件
         * @param calculateId 测算表id
         */
        void importEqptExcelData(MultipartFile multipartFile, Long calculateId) throws IOException;

        /**
         * 更新测算表基础测算信息
         * @param calculateId id
         * @param baseInfo 基础信息json字符串
         * @return Boolean
         */
        Boolean updateCalculateBaseInfo(Long calculateId, String baseInfo);

        /**
         * 生成机器设备测算表excel文件
         * @param calculateId 测算表id
         * @return Boolean
         */
        Boolean generateEquipmentCalculate(Long calculateId) throws IOException;

        /**
         * 根据测算表id获取基础测算信息
         * @param calculateId 测算表id
         * @return String
         */
        String getCalculateBaseInfo(Long calculateId);

        /**
         * 根据测算表id获取评估基准日
         * @param calculateId 测算表id
         * @return String
         */
        LocalDate getValuationBasisDate(Long calculateId);

        /**
         * 根据项目id获取测算信息下拉框数据
         * @param projectId 项目id
         * @return List<DropDownBoxData>
         */
        List<DropDownBoxData> getDropDownBoxData(Long projectId);

        /**
         * 根据测算表id获取生成机器设备融资项目报告引用的测算表信息
         * @param calculateId 测算表id
         * @return EqptReportReferencesCalculateVO
         */
        EqptReportReferencesCalculateVO getEqptReferencesInfoById(Long calculateId);

        /**
         * 根据测算表id获取生成债权分析报告引用的测算表信息
         * @param calculateId 测算表id
         * @return EqptReportReferencesCalculateVO
         */
        CRReportReferencesCalculateVO getCRReferencesInfoById(Long calculateId);

        /**
         * 获取新增DocumentProduction所需的测算信息
         * @param calculateId 测算表id
         * @return AssetsCalculateVO
         */
        AssetsCalculateVO getCalculateInfoForDoc(Long calculateId);

}
