package com.dayou.service;
import com.dayou.dto.report.ReportBaseInfoDTO;
import com.dayou.entity.AssetsReport;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.dayou.vo.AssetsReportVO;
import com.dayou.vo.report.AssetsReportProgressVO;
import com.fasterxml.jackson.core.JsonProcessingException;

import java.io.IOException;
import java.util.List;

/**
 * <p>
 * 资产项目报告信息表 服务类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-12-06
 */
public interface AssetsReportService extends IService<AssetsReport> {

        /**
         * 根据项目id查询所有报告信息
         * @param projectId 项目id
         * @return List<AssetsReport>
         */
        List<AssetsReportVO> listAllByProjectId(Long projectId);

        AssetsReport detail(Long id);

        Boolean add(AssetsReport assetsReport);

        Boolean update(AssetsReport assetsReport);

        Boolean delete(Long id);


        /**
         * 根据项目id获取未完成报告的进度信息
         * @param projectId 项目id
         * @return AssetsReportProgressVO
         */
        AssetsReportProgressVO getUnFinishedReportProgress(Long projectId);

        /**
         * 更新报告基础信息
         * @param reportBaseInfoDTO dto
         * @return Boolean
         */
        Boolean updateReportBaseInfo(ReportBaseInfoDTO reportBaseInfoDTO);

        /**
         * 根据报告id获取报告基础信息
         * @param reportId 报告id
         * @return String
         */
        String getReportBaseInfo(Long reportId);

        /**
         * 生成机器设备报告word文件
         * @param reportId 报告id
         * @return Boolean
         */
        Boolean generateEquipmentReport(Long reportId) throws Exception;

        /**
         * 生成债权分析报告word文件
         * @param reportId 报告id
         * @return Boolean
         */
        Boolean generateCRReport(Long reportId) throws Exception;

}
