package com.dayou.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.dayou.dto.calculate.equipment.EqptQueryDTO;
import com.dayou.entity.AssetsCalculateEqptData;
import com.dayou.entity.AssetsCalculateEqptImpData;
import com.dayou.entity.AssetsCalculateEqptNonsData;
import com.dayou.mapper.AssetsCalculateEqptDataMapper;
import com.dayou.service.AssetsCalculateEqptDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.vo.AssetsCalculateEqptImpDataVO;
import com.dayou.vo.AssetsCalculateEqptNonsDataVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 机器设备测算数据表 服务实现类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-15
 */
@Service
public class AssetsCalculateEqptDataServiceImpl extends ServiceImpl<AssetsCalculateEqptDataMapper, AssetsCalculateEqptData> implements AssetsCalculateEqptDataService {

    @Autowired
    private AssetsCalculateEqptDataMapper dataMapper;

    /**
     * 新增机器设备信息
     * @param assetsCalculateEqptData 设备信息
     * @return Boolean
     */
    @Override
    public Boolean addEqptData(AssetsCalculateEqptData assetsCalculateEqptData) {
        return this.save(assetsCalculateEqptData);
    }

    /**
     * 获取是否有进口设备
     * @param calculateId 测算表id
     * @return Boolean
     */
    @Override
    public Boolean hasImportedEquipment(Long calculateId) {
        return dataMapper.hasImportedEquipment(calculateId) > 0;
    }

    @Override
    public List<AssetsCalculateEqptData> getAllByAssetsCalculateId(Long calculateId, String equipmentType) {
        return dataMapper.getAllByAssetsCalculateId(calculateId, equipmentType);
    }

    /**
     * 分页查询导入的机器设备测算数据
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptData>
     */
    @Override
    public List<AssetsCalculateEqptData> getDataPage(EqptQueryDTO dto) {
        return dataMapper.getDataPage(dto);
    }

    /**
     * 根据测算表id删除
     * @param calculateId 测算表id
     * @return Boolean
     */
    @Override
    public Boolean deleteDataByCalculateId(Long calculateId) {
        return dataMapper.deleteDataByCalculateId(calculateId);
    }

    /**
     * 更新机器设备数据
     * @param assetsCalculateEqptData 机器设备信息
     * @return Boolean
     */
    @Override
    public Boolean updateData(AssetsCalculateEqptData assetsCalculateEqptData) {
        return this.updateById(assetsCalculateEqptData);
    }

    /**
     * 批量更新
     * @param dataList 测算信息集合
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateBatch(List<AssetsCalculateEqptData> dataList) {
        return this.updateBatchById(dataList);
    }

    /**
     * 分页查询进口设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptImpData>
     */
    @Override
    public List<AssetsCalculateEqptImpDataVO> getImpDataPage(EqptQueryDTO dto) {
        return dataMapper.getImpDataPage(dto);
    }

    /**
     * 查询进口设备信息集合
     * @param calculateId 测算表id
     * @return Page<AssetsCalculateEqptImpData>
     */
    @Override
    public List<AssetsCalculateEqptImpDataVO> getImpDataList(Long calculateId) {
        return dataMapper.getImpDataList(calculateId);
    }

    /**
     * 更新机器设备进口设备数据
     * @param assetsCalculateEqptImpData 进口设备信息
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateImpData(AssetsCalculateEqptImpData assetsCalculateEqptImpData) {
        // 判断是修改还是新增
        if (ObjectUtil.isNotNull(assetsCalculateEqptImpData.getId())) {
            dataMapper.updateDataPrice(assetsCalculateEqptImpData.getEqptDataId(), assetsCalculateEqptImpData.getOriginalPrice());
            return dataMapper.updateImpData(assetsCalculateEqptImpData);
        }else {
            dataMapper.updateDataPrice(assetsCalculateEqptImpData.getEqptDataId(), assetsCalculateEqptImpData.getOriginalPrice());
            return dataMapper.addImpData(assetsCalculateEqptImpData);
        }

    }

    /**
     * 批量更新机器设备进口设备数据
     * @param dataList 进口设备信息
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateBatchImpData(List<AssetsCalculateEqptImpData> dataList) {
        for (AssetsCalculateEqptImpData impData : dataList) {
            updateImpData(impData);
        }
        return Boolean.TRUE;
    }

    /**
     * 分页查询非标设备信息
     * @param dto 查询dto
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    @Override
    public List<AssetsCalculateEqptNonsDataVO> getNonSDataPage(EqptQueryDTO dto) {
        return dataMapper.getNonSDataPage(dto);
    }

    /**
     * 查询非标设备信息集合
     * @param calculateId 测算表id
     * @return Page<AssetsCalculateEqptNonsDataVO>
     */
    @Override
    public List<AssetsCalculateEqptNonsDataVO> getNonSDataList(Long calculateId) {
        return dataMapper.getNonSDataList(calculateId);
    }

    /**
     * 更新机器设备非标设备数据
     * @param assetsCalculateEqptNonsData 非标设备信息
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateNonSData(AssetsCalculateEqptNonsData assetsCalculateEqptNonsData) {
        // 判断是修改还是新增
        if (ObjectUtil.isNotNull(assetsCalculateEqptNonsData.getId())) {
            dataMapper.updateDataPrice(assetsCalculateEqptNonsData.getEqptDataId(), assetsCalculateEqptNonsData.getNonsEqptValue());
            return dataMapper.updateNonSData(assetsCalculateEqptNonsData);
        }else {
            dataMapper.updateDataPrice(assetsCalculateEqptNonsData.getEqptDataId(), assetsCalculateEqptNonsData.getNonsEqptValue());
            return dataMapper.addNonSData(assetsCalculateEqptNonsData);
        }
    }

    /**
     * 批量更新机器设备非标设备数据
     * @param dataList 非标设备信息
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateBatchNonSData(List<AssetsCalculateEqptNonsData> dataList) {
        for (AssetsCalculateEqptNonsData nonsData : dataList) {
            updateNonSData(nonsData);
        }
        return Boolean.TRUE;
    }

    /**
     * 根据机器设备信息id删除
     * @param eqptId 设备信息id
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean deleteDqptDataById(Long eqptId) {
        dataMapper.deleteImpSDataByEqptId(eqptId);
        dataMapper.deleteNonSDataByEqptId(eqptId);
        return this.removeById(eqptId);
    }
}
