package com.dayou.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjectUtil;
import com.dayou.common.BaseEntity;
import com.dayou.dto.AssetsProjectQueryDTO;
import com.dayou.entity.AssetsCalculate;
import com.dayou.entity.AssetsProject;
import com.dayou.entity.AssetsReport;
import com.dayou.mapper.AssetsProjectMapper;
import com.dayou.service.AssetsCalculateService;
import com.dayou.service.AssetsProjectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.AssetsReportService;
import com.dayou.vo.AssetsProjectVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import java.util.List;
import java.util.ArrayList;
import java.util.Objects;

import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 资产项目基本信息表 服务实现类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-11-12
 */
@Service
public class AssetsProjectServiceImpl extends ServiceImpl<AssetsProjectMapper, AssetsProject> implements AssetsProjectService {

    @Autowired
    private AssetsProjectMapper assetsProjectMapper;

    @Autowired
    private AssetsCalculateService assetsCalculateService;

    @Autowired
    private AssetsReportService assetsReportService;

    /**
     * 获取资产项目列表
     * @param page 分页
     * @param dto 资产项目查询dto
     * @return Page<AssetsProjectVO>
     */
    @Override
    public Page<AssetsProjectVO> getAssetsProject(Page page, AssetsProjectQueryDTO dto) {
        dto.setCreateUserId(StpUtil.getLoginIdAsLong());
        Page<AssetsProjectVO> assetsProjectVO = assetsProjectMapper.getAssetsProject(page, dto);
        return assetsProjectMapper.getAssetsProject(page, dto);
    }

    @Override
    public AssetsProject detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(AssetsProject assetsProject){
        assetsProject.setCreateUserId(StpUtil.getLoginIdAsLong());
        return  this.save(assetsProject);
    }

    @Override
    public Boolean update(AssetsProject assetsProject){
        AssetsProject oldInfo = this.getById(assetsProject.getId());
        // 判断是否有更改项目类型
        if (!Objects.equals(oldInfo.getProjectTypeId(), assetsProject.getProjectTypeId())) { // 更改了项目类型
            // 如果更改了项目类型,且项目下已经创建了测算表或报告就不允许更改项目类型
            int calculateCount = assetsCalculateService.count(new LambdaQueryWrapper<AssetsCalculate>().eq(AssetsCalculate::getProjectId, assetsProject.getId()).eq(BaseEntity::getDeleteStatus, 0));
            int reportCount = assetsReportService.count(new LambdaQueryWrapper<AssetsReport>().eq(AssetsReport::getProjectId, assetsProject.getId()).eq(BaseEntity::getDeleteStatus, 0));
            if (calculateCount > 0 || reportCount > 0) { // 创建了测算表或报告
                return false;
            }
            return this.updateById(assetsProject);
        } else {
            return this.updateById(assetsProject);
        }
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }
}
