package com.dayou.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.read.listener.PageReadListener;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.aspose.words.*;
import com.dayou.bo.CRReportFillBO;
import com.dayou.bo.EqptReportFillBO;
import com.dayou.config.FileNetConfig;
import com.dayou.dto.report.ReportBaseInfoDTO;
import com.dayou.dto.report.cr.CRReportBaseInfoBO;
import com.dayou.dto.report.cr.DebtorInfoStrBO;
import com.dayou.dto.report.cr.GuarantorInfoStrBO;
import com.dayou.dto.report.equipment.EqptReportBaseInfoDTO;
import com.dayou.entity.AssetsReport;
import com.dayou.entity.DocumentProduction;
import com.dayou.entity.TmplAssetReport;
import com.dayou.entity.TmplAssetReportSection;
import com.dayou.exception.BusinessException;
import com.dayou.mapper.AssetsReportMapper;
import com.dayou.mapper.TmplAssetReportMapper;
import com.dayou.mapper.TmplAssetReportSectionMapper;
import com.dayou.service.AssetsReportService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.DocumentProductionService;
import com.dayou.utils.*;
import com.dayou.vo.AssetsReportVO;
import com.dayou.vo.report.AssetsReportProgressVO;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;

import static com.dayou.enums.CRReportTmplCode.*;
import static com.dayou.enums.DocumentType.CR_REPORT;
import static com.dayou.enums.DocumentType.EQPT_REPORT;
import static com.dayou.enums.EqptReportTmplCode.*;
import static com.dayou.utils.AsposeWordUtil.findTableIndex;
import static com.dayou.utils.EasyExcelUtil.getCellValueByIndex;
import static com.dayou.utils.EasyExcelUtil.hasColumnIndexGetRowIndexByValue;

/**
 * <p>
 * 资产项目报告信息表 服务实现类
 * </p>
 *
 * @author mybatis-plus-generator
 * @since 2024-12-06
 */
@Service
public class AssetsReportServiceImpl extends ServiceImpl<AssetsReportMapper, AssetsReport> implements AssetsReportService {

    @Autowired
    private AssetsReportMapper assetsReportMapper;

    @Autowired
    private TmplAssetReportMapper tmplAssetReportMapper;

    @Autowired
    private TmplAssetReportSectionMapper reportSectionMapper;

    @Autowired
    private DocumentProductionService documentProductionService;

    @Autowired
    private FileNetConfig fileNetConfig;

    /**
     * 根据项目id查询所有报告信息
     *
     * @param projectId 项目id
     * @return List<AssetsReport>
     */
    @Override
    public List<AssetsReportVO> listAllByProjectId(Long projectId) {
        return assetsReportMapper.listAllByProjectId(projectId);
    }


    @Override
    public AssetsReport detail(Long id) {
        return this.getById(id);
    }

    @Override
    public Boolean add(AssetsReport assetsReport) {
        assetsReport.setCreateUserId(StpUtil.getLoginIdAsLong());
        if (ObjectUtil.isNull(assetsReport.getId())) {
            return this.save(assetsReport);
        } else {
            return this.updateById(assetsReport);
        }
    }

    @Override
    public Boolean update(AssetsReport assetsReport) {
        return this.updateById(assetsReport);
    }

    @Override
    public Boolean delete(Long id) {
        //逻辑删除
        return this.removeById(id);
    }

    /**
     * 根据项目id获取未完成报告的进度信息
     *
     * @param projectId 项目id
     * @return AssetsReportProgressVO
     */
    @Override
    public AssetsReportProgressVO getUnFinishedReportProgress(Long projectId) {
        return assetsReportMapper.getUnFinishedReportProgress(projectId);
    }

    /**
     * 更新报告基础信息
     *
     * @param reportBaseInfoDTO dto
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateReportBaseInfo(ReportBaseInfoDTO reportBaseInfoDTO) {
        // 如果baseInfo为null说明是第一次进行填写测算基本信息的操作,则更新报告进度
        if (ObjectUtil.isNull(getReportBaseInfo(reportBaseInfoDTO.getReportId()))) {
            assetsReportMapper.updateReportProgress(reportBaseInfoDTO.getReportId(), "GENERATE");
        }
        return assetsReportMapper.updateReportBaseInfo(reportBaseInfoDTO);
    }

    /**
     * 根据报告id获取报告基础信息
     *
     * @param reportId 报告id
     * @return String
     */
    @Override
    public String getReportBaseInfo(Long reportId) {
        return assetsReportMapper.getReportBaseInfo(reportId);
    }

    /**
     * 生成机器设备报告word文件
     *
     * @param reportId 报告id
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean generateEquipmentReport(Long reportId) throws Exception {

        // 获取与设置基础报告信息
        ObjectMapper objectMapper = new ObjectMapper();
        EqptReportFillBO eqptReportFillBO = assetsReportMapper.getEqptReportFillBO(reportId);
        // 设置增值率(评估净值 - 账面净值) / 账面净值,保留两位小数
        if (ObjectUtil.isNotNull(eqptReportFillBO.getEvaluateNetValue()) && ObjectUtil.isNotNull(eqptReportFillBO.getBookNetValue())) {
            eqptReportFillBO.setNetValueAddedRate(((eqptReportFillBO.getEvaluateNetValue().subtract(eqptReportFillBO.getBookNetValue())).divide(eqptReportFillBO.getBookNetValue(), 2, RoundingMode.HALF_UP)).doubleValue() * 100);
        }
        // 将账面、评估的原值、净值处理成千分位格式
        DecimalFormat decimalFormat = new DecimalFormat("#,##0.00");
        if (eqptReportFillBO.getBookOriginalValue() != null) {
            eqptReportFillBO.setBookOriginalValueStr(decimalFormat.format(eqptReportFillBO.getBookOriginalValue()));
        }
        if (eqptReportFillBO.getBookNetValue() != null) {
            eqptReportFillBO.setBookNetValueStr(decimalFormat.format(eqptReportFillBO.getBookNetValue()));
        }
        if (eqptReportFillBO.getEvaluateOriginalValue() != null) {
            eqptReportFillBO.setEvaluateOriginalValueStr(decimalFormat.format(eqptReportFillBO.getEvaluateOriginalValue().setScale(0, RoundingMode.HALF_UP)));
        }
        if (eqptReportFillBO.getEvaluateNetValue() != null) {
            eqptReportFillBO.setEvaluateNetValueStr(decimalFormat.format(eqptReportFillBO.getEvaluateNetValue().setScale(0, RoundingMode.HALF_UP)));
        }

        // 设置其他的信息
        eqptReportFillBO.setEqptReportBaseInfo(objectMapper.readValue(getReportBaseInfo(reportId), EqptReportBaseInfoDTO.class));
        // 设置委托人和产权持有人的名字
        eqptReportFillBO.setConsignorCompanyName(eqptReportFillBO.getEqptReportBaseInfo().getConsignorInfos().stream() // 将集合转换为流
                .map(EqptReportBaseInfoDTO.ConsignorInfo::getConsignorCompanyName) // 提取每个对象的consignorCompanyName属性
                .collect(Collectors.joining("、")));
        eqptReportFillBO.setOwnerCompanyName(eqptReportFillBO.getEqptReportBaseInfo().getPropertyOwnerInfos().stream() // 将集合转换为流
                .map(EqptReportBaseInfoDTO.PropertyOwnerInfo::getOwnerCompanyName) // 提取每个对象的ownerCompanyName属性
                .collect(Collectors.joining("、")));
        // 将评估基准日只截取年月,并设置为开始评估工作年月
        String[] parts = eqptReportFillBO.getEqptReportBaseInfo().getValuationBasisDate().split("日");
        String yearMonth = parts[0];
        eqptReportFillBO.setStartWorkMonth(yearMonth);
        // 将评估报告日转换为中文并设置到中文评估报告日中
        eqptReportFillBO.setChineseReportDate(DateToChinese.dateStrConvertChinese(eqptReportFillBO.getEqptReportBaseInfo().getReportDate()));

        // 获取框架模板(主模板)
        TmplAssetReport mainTmpl;
        // 获取封面模板
        TmplAssetReportSection coverTmpl = null;
        // 获取摘要模板
        TmplAssetReportSection digestTmpl = null;
        // 获取正文模板
        TmplAssetReportSection detailTmpl = null;
        // 获取委托人概况模板
        TmplAssetReportSection consignorTmpl = null;
        // 获取产权持有人概况模板
        TmplAssetReportSection ownerTmpl = null;
        // 获取目录模板
        TmplAssetReportSection catalogueTmpl = null;
        // 获取附件模板
        TmplAssetReportSection attachmentsTmpl = null;
        if (eqptReportFillBO.getEqptReportBaseInfo().getReportType().equals("评估报告")) {
            // 获取框架模板(主模板)
            mainTmpl = tmplAssetReportMapper.getTmplByCode(MAIN.getCode(), "机器设备");
            // 获取封面模板
            coverTmpl = reportSectionMapper.getTmplByCode(COVER.getCode(), mainTmpl.getId());
            // 获取摘要模板
            digestTmpl = reportSectionMapper.getTmplByCode(DIGEST.getCode(), mainTmpl.getId());
            // 获取正文模板
            detailTmpl = reportSectionMapper.getTmplByCode(DETAIL.getCode(), mainTmpl.getId());
            // 获取委托人概况模板
            consignorTmpl = reportSectionMapper.getTmplByCode(CONSIGNOR.getCode(), mainTmpl.getId());
            // 获取产权持有人概况模板
            ownerTmpl = reportSectionMapper.getTmplByCode(PROPERTY_OWNER.getCode(), mainTmpl.getId());
            // 获取目录模板
            catalogueTmpl = reportSectionMapper.getTmplByCode(CATALOGUE.getCode(), mainTmpl.getId());
            // 获取附件模板
            attachmentsTmpl = reportSectionMapper.getTmplByCode(ATTACHMENTS.getCode(), mainTmpl.getId());
        } else if (eqptReportFillBO.getEqptReportBaseInfo().getReportType().equals("咨询报告")) {
            // 获取框架模板(主模板)
            mainTmpl = tmplAssetReportMapper.getTmplByCode(CONSULT_MAIN.getCode(), "机器设备");
            // 获取封面模板
            coverTmpl = reportSectionMapper.getTmplByCode(CONSULT_COVER.getCode(), mainTmpl.getId());
            // 获取摘要模板
            digestTmpl = reportSectionMapper.getTmplByCode(CONSULT_DIGEST.getCode(), mainTmpl.getId());
            // 获取正文模板
            detailTmpl = reportSectionMapper.getTmplByCode(CONSULT_DETAIL.getCode(), mainTmpl.getId());
            // 获取委托人概况模板
            consignorTmpl = reportSectionMapper.getTmplByCode(CONSULT_CONSIGNOR.getCode(), mainTmpl.getId());
            // 获取产权持有人概况模板
            ownerTmpl = reportSectionMapper.getTmplByCode(CONSULT_PROPERTY_OWNER.getCode(), mainTmpl.getId());
            // 获取目录模板
            catalogueTmpl = reportSectionMapper.getTmplByCode(CONSULT_CATALOGUE.getCode(), mainTmpl.getId());
            // 获取附件模板
            attachmentsTmpl = reportSectionMapper.getTmplByCode(CONSULT_ATTACHMENTS.getCode(), mainTmpl.getId());
        } else {
            throw new BusinessException("无符合条件的模板，请咨询系统管理人员了解详情");
        }

        // 第一步:设置封面
        // 封面模板文件位置
        String coverTmplPath = fileNetConfig.getBaseDir() + coverTmpl.getSectionFileUrl() + coverTmpl.getSectionFileName();
        // 读取模板文件
        byte[] coverTmplByte = Files.readAllBytes(Paths.get(coverTmplPath));
        // 获取填充数据后的文件
        byte[] resultCoverByte = AsposeWordUtil.fillWordDataByMap(coverTmplByte, DataUtil.objToMap(eqptReportFillBO));
        // 将封面的字节流转成输入流以供后续使用
        InputStream coverIs = new ByteArrayInputStream(resultCoverByte);


        // 第二步:设置摘要内容
        // 封面模板文件位置
        String digestTmplPath = fileNetConfig.getBaseDir() + digestTmpl.getSectionFileUrl() + digestTmpl.getSectionFileName();
        // 读取模板文件
        byte[] digestTmplByte = Files.readAllBytes(Paths.get(digestTmplPath));
        // 获取填充数据后的文件
        byte[] resultdigestByte = AsposeWordUtil.fillWordDataByMap(digestTmplByte, DataUtil.objToMap(eqptReportFillBO));
        // 将摘要的字节流转成输入流以供后续使用
        InputStream digestIs = new ByteArrayInputStream(resultdigestByte);

        // 第三步:设置正文内容
        // 封面模板文件位置
        String detailTmplPath = fileNetConfig.getBaseDir() + detailTmpl.getSectionFileUrl() + detailTmpl.getSectionFileName();
        // 读取模板文件
        byte[] detailTmplByte = Files.readAllBytes(Paths.get(detailTmplPath));
        // 获取填充数据后的文件
        byte[] resultDetailByte = AsposeWordUtil.fillWordDataByMap(detailTmplByte, DataUtil.objToMap(eqptReportFillBO));
        // 将正文的字节流转成输入流以供后续使用
        InputStream detailIs = new ByteArrayInputStream(resultDetailByte);
        // 将正文读取出来
        Document detailDoc = new Document(detailIs);
        // 判断是否需要 质保期 注意事项段落,不需要就通过提前设置的书签移除掉
        DocumentBuilder detailBuilder = new DocumentBuilder(detailDoc);
        if (!eqptReportFillBO.getEqptReportBaseInfo().getAssetsInfo().getUnderWarranty()) {
            detailBuilder.moveToBookmark("underWarranty");
            detailBuilder.getCurrentParagraph().remove();
        }

        // 第四步:设置委托人概况到正文
        // 委托人概况模板文件位置
        String consignorTmplPath = fileNetConfig.getBaseDir() + consignorTmpl.getSectionFileUrl() + consignorTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下
        for (int i = eqptReportFillBO.getEqptReportBaseInfo().getConsignorInfos().size(); i > 0; i--) {
            EqptReportBaseInfoDTO.ConsignorInfo consignorInfo = eqptReportFillBO.getEqptReportBaseInfo().getConsignorInfos().get(i - 1);
            consignorInfo.setSort(i);
            consignorInfo.setSortUppercase(ArabicToChineseUtil.int2chineseNum(i));
            // 读取模板文件
            byte[] consignorTmplByte = Files.readAllBytes(Paths.get(consignorTmplPath));
            // 获取填充数据后的文件
            byte[] resultConsignorByte = AsposeWordUtil.fillWordDataByDomain(consignorTmplByte, consignorInfo);
            InputStream consignorIs = new ByteArrayInputStream(resultConsignorByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(consignorIs), "insertConsignor", false, false);
            consignorIs.close();
        }

        // 第五步:设置产权持有人概况到正文
        // 产权持有人概况模板文件位置
        String ownerTmplPath = fileNetConfig.getBaseDir() + ownerTmpl.getSectionFileUrl() + ownerTmpl.getSectionFileName();
        for (int i = eqptReportFillBO.getEqptReportBaseInfo().getPropertyOwnerInfos().size(); i > 0; i--) {
            EqptReportBaseInfoDTO.PropertyOwnerInfo propertyOwnerInfo = eqptReportFillBO.getEqptReportBaseInfo().getPropertyOwnerInfos().get(i - 1);
            propertyOwnerInfo.setSort(i);
            propertyOwnerInfo.setSortUppercase(ArabicToChineseUtil.int2chineseNum(i));
            // 读取模板文件
            byte[] propertyOwnerTmplByte = Files.readAllBytes(Paths.get(ownerTmplPath));
            // 获取填充数据后的文件
            byte[] propertyOwnerByte = AsposeWordUtil.fillWordDataByDomain(propertyOwnerTmplByte, propertyOwnerInfo);
            InputStream propertyOwnerIs = new ByteArrayInputStream(propertyOwnerByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(propertyOwnerIs), "insertPropertyOwner", false, false);
            propertyOwnerIs.close();
        }

        // 第六步:设置主体框架
        // 框架生成后文件位置
        String docUrl = fileNetConfig.getAssetOutputReportPath() + System.currentTimeMillis() + "_设备融资报告.docx";
        String mainPath = fileNetConfig.getBaseDir() + docUrl;
        // 框架模板文件位置
        String mainTmplPath = fileNetConfig.getBaseDir() + mainTmpl.getFileUrl() + mainTmpl.getFileName();

        // 读取模板文件
        byte[] mainTmplByte = Files.readAllBytes(Paths.get(mainTmplPath));
        // 获取填充数据后的文件
        byte[] resultMainByte = AsposeWordUtil.fillWordDataByMap(mainTmplByte, DataUtil.objToMap(eqptReportFillBO));
        InputStream mainIs = new ByteArrayInputStream(resultMainByte);
        // 将框架读取出来
        Document mainDoc = new Document(mainIs);
        // 将摘要插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, new Document(digestIs), "insertDigest", false, false);
        // 将正文插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, detailDoc, "insertDetail", false, false);

        // 目录模板文件位置
        String catalogueTmplPath = fileNetConfig.getBaseDir() + catalogueTmpl.getSectionFileUrl() + catalogueTmpl.getSectionFileName();
        // 将框架读取出来
        Document catalogueDoc = new Document(catalogueTmplPath);

        // 附件模板文件位置
        String attachmentsPath = fileNetConfig.getBaseDir() + attachmentsTmpl.getSectionFileUrl() + attachmentsTmpl.getSectionFileName();
        // 读取模板文件
        byte[] attachmentsTmplByte = Files.readAllBytes(Paths.get(attachmentsPath));
        // 获取填充数据后的文件
        byte[] resultAttachmentsByte = AsposeWordUtil.fillWordDataByMap(attachmentsTmplByte, DataUtil.objToMap(eqptReportFillBO));
        InputStream attachmentsIs = new ByteArrayInputStream(resultAttachmentsByte);
        // 将附件读取出来
        Document attachmentsDoc = new Document(attachmentsIs);
        // 将附件插入到目录
        AsposeWordUtil.insertDocumentAfterBookMark(catalogueDoc, attachmentsDoc, "insertAttachments", false, false);
        // 将目录插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, catalogueDoc, "insertCatalogue", false, false);

        // 第七步:删除预留书签插入位置的空白行
        DocumentBuilder mainBuilder = new DocumentBuilder(mainDoc);
        // 移除正文的书签所在段落
        mainBuilder.moveToBookmark("insertDetail");
        mainBuilder.getCurrentParagraph().remove();
        // 移除摘要的书签所在段落
        mainBuilder.moveToBookmark("insertDigest");
        mainBuilder.getCurrentParagraph().remove();
        // 移除目录的书签所在段落
        mainBuilder.moveToBookmark("insertCatalogue");
        mainBuilder.getCurrentParagraph().remove();
        // 移除附件的书签所在段落
        mainBuilder.moveToBookmark("insertAttachments");
        mainBuilder.getCurrentParagraph().remove();
        // 移除委托人概况的书签所在段落
        mainBuilder.moveToBookmark("insertConsignor");
        mainBuilder.getCurrentParagraph().remove();
        // 移除产权持有人概况的书签所在段落
        mainBuilder.moveToBookmark("insertPropertyOwner");
        mainBuilder.getCurrentParagraph().remove();

        // 第八步:生成并保存文档
        // 将封面和文档进行拼接
        Document coverDoc = new Document(coverIs);
        coverDoc.appendDocument(mainDoc, ImportFormatMode.KEEP_SOURCE_FORMATTING);
        // 更新目录页码
        AsposeWordUtil.updateCatalogueLink(coverDoc);

        // 开启word的修订模式(修改留痕)
        coverDoc.setTrackRevisions(true);

        coverDoc.save(mainPath);

        // 关闭各个段落的流
        coverIs.close();
        digestIs.close();
        detailIs.close();
        mainIs.close();
        attachmentsIs.close();

        // 更新报告进度
        assetsReportMapper.updateReportProgress(reportId, "FINISHED");

        // 保存文档信息到数据库
        // 设置文档信息
        AssetsReportVO assetsReportVO = assetsReportMapper.getReportInfoForDoc(reportId);
        DocumentProduction documentProduction = documentProductionService.getDocProdByBusinessId(assetsReportVO.getProjectId(), reportId);

        if (ObjectUtil.isNotNull(documentProduction)) { // 判断文档信息是否为空,不为空说明以前生成过文档,则更新文档版本号
            documentProduction.setDocVersion(documentProduction.getDocVersion() + 1);
        } else { // 为空重新new对象
            documentProduction = new DocumentProduction();
        }
        // 设置文档信息
        documentProduction.setBusinessType("ASSETS")
                .setBusinessId(assetsReportVO.getProjectId())
                .setDocType(EQPT_REPORT.getName())
                .setDocName(assetsReportVO.getReportName() + ".docx")
                .setConsignor(assetsReportVO.getPrincipal())
                .setDocUrl(docUrl)
                .setCreateUserId(StpUtil.getLoginIdAsLong())
                .setBusinessCate(assetsReportVO.getProjectTypeName())
                .setDocNo(ObjectUtil.isNotNull(eqptReportFillBO.getProductionNo()) ? eqptReportFillBO.getProductionNo() : "")
                .setBusinessSubId(reportId);
        return documentProductionService.saveDocumentProduction(documentProduction);
    }

    /**
     * 生成债权分析报告
     *
     * @param reportId 报告id
     * @return Boolean
     */
    @Override
    public Boolean generateCRReport(Long reportId) throws Exception {

        // 获取框架模板
        TmplAssetReport mainTmpl = tmplAssetReportMapper.getTmplByCode(ANALYSIS_MAIN.getCode(), "债权");
        // 获取封面模板
        TmplAssetReportSection coverTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_COVER.getCode(), mainTmpl.getId());
        // 获取摘要模板
        TmplAssetReportSection digestTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_DIGEST.getCode(), mainTmpl.getId());
        // 获取正文模板
        TmplAssetReportSection detailTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_DETAIL.getCode(), mainTmpl.getId());
        // 获取债务人信息模板
        TmplAssetReportSection debtorTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_DEBTOR.getCode(), mainTmpl.getId());
        // 获取自然人保证人信息模板
        TmplAssetReportSection personGuarantorTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_PERSON_GUARANTOR.getCode(), mainTmpl.getId());
        // 获取企业保证人信息模板
        TmplAssetReportSection enterpriseGuarantorTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_ENTERPRISE_GUARANTOR.getCode(), mainTmpl.getId());
        // 获取资产情况模板
        TmplAssetReportSection debtorAssetTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_DEBTOR_ASSET.getCode(), mainTmpl.getId());
        TmplAssetReportSection guarantorAssetTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_GUARANTOR_ASSET.getCode(), mainTmpl.getId());
        // 获取诉讼情况信息模板
        TmplAssetReportSection lawsuitTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_LAWSUIT.getCode(), mainTmpl.getId());
        // 获取诉讼情况信息模板
        TmplAssetReportSection guarantorAdditionalDebtRepaymentSourcesTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_GUARANTOR_ADDITIONAL_DEBT_REPAYMENT_SOURCES.getCode(), mainTmpl.getId());
        // 获取目录模板
        TmplAssetReportSection catalogueTmpl = reportSectionMapper.getTmplByCode(ANALYSIS_CATALOGUE.getCode(), mainTmpl.getId());

        // 获取与设置基础报告信息
        ObjectMapper objectMapper = new ObjectMapper();
        CRReportFillBO crReportFillBO = assetsReportMapper.getCRReportFillBO(reportId);
        CRReportBaseInfoBO crReportBaseInfoBO = objectMapper.readValue(getReportBaseInfo(reportId), CRReportBaseInfoBO.class);
        // 获取测算表
        DocumentProduction calculateDoc = documentProductionService.getDocProdByBusinessId(crReportFillBO.getProjectId(), crReportFillBO.getCalculateId());
        // 读取债务人信息
        List<DebtorInfoStrBO> debtorInfoStrList = new ArrayList<>();
        ExcelReader calculateReader = EasyExcel.read(fileNetConfig.getBaseDir() + calculateDoc.getDocUrl()).build();
        ReadSheet readDebtor = EasyExcel.readSheet("2.债务人分析明细表").head(DebtorInfoStrBO.class).registerReadListener(new PageReadListener<DebtorInfoStrBO>(debtorInfoStrList::addAll)).headRowNumber(4).build();
        calculateReader.read(readDebtor);
        // 去掉空的数据(excel表中因为有合计行等等,所以可能读出来空行)
        debtorInfoStrList = debtorInfoStrList.stream()
                .filter(obj -> obj.getInfoStr() != null)
                .collect(Collectors.toList());
        // 处理债务人信息
        List<CRReportBaseInfoBO.DebtorInfo> debtorInfoList = new ArrayList<>();
        // 目前来说只考虑一个债务人的情况，所以只拿第一个元素的受偿分析即可
        crReportBaseInfoBO.setRepaymentCapacityAnalysis(debtorInfoStrList.get(0).getRepaymentCapacityAnalysisStr());
        for (DebtorInfoStrBO infoStr : debtorInfoStrList) {
            CRReportBaseInfoBO.DebtorInfo debtorInfo = new CRReportBaseInfoBO.DebtorInfo();
            String debtorStr = infoStr.getInfoStr();
            debtorInfo.setDebtorName(debtorStr.substring(debtorStr.indexOf("公司名称：") + "公司名称：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorUnifiedSocialCreditCode(debtorStr.substring(debtorStr.indexOf("统一社会信用代码：") + "统一社会信用代码：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorResidence(debtorStr.substring(debtorStr.indexOf("住所：") + "住所：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorRegisteredCapital(debtorStr.substring(debtorStr.indexOf("注册资本：") + "注册资本：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorLegalRepresentative(debtorStr.substring(debtorStr.indexOf("法定代表人：") + "法定代表人：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorType(debtorStr.substring(debtorStr.indexOf("企业类型：") + "企业类型：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorEstablishmentDate(debtorStr.substring(debtorStr.indexOf("成立日期：") + "成立日期：".length(), debtorStr.indexOf("\n")));
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorBusinessOperationPeriod(debtorStr.indexOf("营业期限：") > 0 ? debtorStr.substring(debtorStr.indexOf("营业期限：") + "营业期限：".length(), debtorStr.indexOf("\n")) : null);
            debtorStr = debtorStr.replaceFirst("\n", "");
            debtorInfo.setDebtorBusinessScope(debtorStr.substring(debtorStr.indexOf("经营范围：") + "经营范围：".length()));
            // 设置债务人信息
            debtorInfoList.add(debtorInfo);
        }
        crReportBaseInfoBO.setDebtorInfoList(debtorInfoList);

        // 读取保证人信息
        List<GuarantorInfoStrBO> guarantorInfoStrBOList = new ArrayList<>();
        ReadSheet readGuarantor = EasyExcel.readSheet("3.保证人分析明细表").head(GuarantorInfoStrBO.class).registerReadListener(new PageReadListener<GuarantorInfoStrBO>(guarantorInfoStrBOList::addAll)).headRowNumber(4).build();
        calculateReader.read(readGuarantor);
        // 去掉空的数据(excel表中因为有合计行等等,所以可能读出来空行)
        guarantorInfoStrBOList = guarantorInfoStrBOList.stream()
                .filter(obj -> obj.getInfoStr() != null)
                .collect(Collectors.toList());
        // 处理保证人信息
        List<CRReportBaseInfoBO.PersonGuarantor> personGuarantorList = new ArrayList<>();
        List<CRReportBaseInfoBO.EnterpriseGuarantor> enterpriseGuarantorList = new ArrayList<>();
        for (GuarantorInfoStrBO infoStr : guarantorInfoStrBOList) {
            if (ObjectUtil.isNull(infoStr.getGuarantorType())) {
                throw new BusinessException("债权测算表_3.保证人分析明细表_保证人类型有为空的行,请检查后并重新上传后再试!");
            }
            if (infoStr.getGuarantorType().equals("企业")) {
                CRReportBaseInfoBO.EnterpriseGuarantor enterpriseGuarantor = new CRReportBaseInfoBO.EnterpriseGuarantor();
                enterpriseGuarantor.setRepaymentCapacityAnalysis(infoStr.getRepaymentCapacityAnalysisStr());
                String guarantorStr = infoStr.getInfoStr();
                enterpriseGuarantor.setGuarantorName(guarantorStr.substring(guarantorStr.indexOf("公司名称：") + "公司名称：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorUnifiedSocialCreditCode(guarantorStr.substring(guarantorStr.indexOf("统一社会信用代码：") + "统一社会信用代码：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorType(guarantorStr.substring(guarantorStr.indexOf("企业类型：") + "企业类型：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorRegisteredCapital(guarantorStr.substring(guarantorStr.indexOf("注册资本：") + "注册资本：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorLegalRepresentative(guarantorStr.substring(guarantorStr.indexOf("法定代表人：") + "法定代表人：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorEstablishmentDate(guarantorStr.substring(guarantorStr.indexOf("成立日期：") + "成立日期：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorBusinessOperationPeriod(guarantorStr.indexOf("营业期限：") > 0 ? guarantorStr.substring(guarantorStr.indexOf("营业期限：") + "营业期限：".length(), guarantorStr.indexOf("\n")) : null);
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorResidence(guarantorStr.substring(guarantorStr.indexOf("住所：") + "住所：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                enterpriseGuarantor.setGuarantorBusinessScope(guarantorStr.substring(guarantorStr.indexOf("经营范围：") + "经营范围：".length()));
                // 设置保证人信息
                enterpriseGuarantorList.add(enterpriseGuarantor);
            } else if (infoStr.getGuarantorType().equals("自然人")) {
                CRReportBaseInfoBO.PersonGuarantor personGuarantor = new CRReportBaseInfoBO.PersonGuarantor();
                personGuarantor.setRepaymentCapacityAnalysis(infoStr.getRepaymentCapacityAnalysisStr());
                String guarantorStr = infoStr.getInfoStr();
                personGuarantor.setGuarantorName(guarantorStr.substring(guarantorStr.indexOf("姓名：") + "姓名：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                personGuarantor.setGuarantorEthnicGroup(guarantorStr.substring(guarantorStr.indexOf("民族：") + "民族：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                personGuarantor.setGuarantorDateOfBirth(guarantorStr.substring(guarantorStr.indexOf("出生年月：") + "出生年月：".length(), guarantorStr.indexOf("\n")));
                guarantorStr = guarantorStr.replaceFirst("\n", "");
                personGuarantor.setGuarantorIdentificationNumber(guarantorStr.substring(guarantorStr.indexOf("身份证号码：") + "身份证号码：".length(), guarantorStr.indexOf("\n")));
                personGuarantor.setGuarantorResidence(guarantorStr.substring(guarantorStr.indexOf("住址：") + "住址：".length()));
                // 设置保证人信息
                personGuarantorList.add(personGuarantor);
            }
        }
        crReportBaseInfoBO.setPersonGuarantorList(personGuarantorList);
        crReportBaseInfoBO.setEnterpriseGuarantorList(enterpriseGuarantorList);

        // 读取资产情况
        List<CRReportBaseInfoBO.Asset> assetList = new ArrayList<>();
        ReadSheet readAsset = EasyExcel.readSheet("资产情况").head(CRReportBaseInfoBO.Asset.class).registerReadListener(new PageReadListener<CRReportBaseInfoBO.Asset>(assetList::addAll)).build();
        calculateReader.read(readAsset);
        // 从债务人和保证人分析表中获取当前状态并设置到资产信息中
        for (CRReportBaseInfoBO.Asset asset : assetList) {
            if (asset.getType().equals("债务人")) {
                Optional<DebtorInfoStrBO> optionalDebtorInfoStrBO = debtorInfoStrList.stream()
                        .filter(obj -> obj.getName().equals(asset.getName()))
                        .findFirst();
                optionalDebtorInfoStrBO.ifPresent(bo -> asset.setStatus(bo.getStatusStr()));
            } else if (asset.getType().equals("保证人")) {
                Optional<GuarantorInfoStrBO> optionalGuarantorInfoStrBO = guarantorInfoStrBOList.stream()
                        .filter(obj -> obj.getName().equals(asset.getName()))
                        .findFirst();
                optionalGuarantorInfoStrBO.ifPresent(bo -> asset.setStatus(bo.getStatusStr()));
            }
        }
        crReportBaseInfoBO.setAssetList(assetList);

        // 读取诉讼情况
        List<CRReportBaseInfoBO.Lawsuit> lawsuitList = new ArrayList<>();
        ReadSheet readLawsuit = EasyExcel.readSheet("诉讼情况").head(CRReportBaseInfoBO.Lawsuit.class).registerReadListener(new PageReadListener<CRReportBaseInfoBO.Lawsuit>(lawsuitList::addAll)).build();
        calculateReader.read(readLawsuit);
        crReportBaseInfoBO.setLawsuitList(lawsuitList);

        crReportFillBO.setReportBaseInfo(crReportBaseInfoBO);

        // 将评估报告日转换为中文并设置到中文评估报告日中
        crReportFillBO.setChineseReportDate(DateToChinese.dateStrConvertChinese(crReportFillBO.getReportBaseInfo().getReportDate()));

        // 获取汇总表合计行的下标(用来获取债权汇总表的合计信息)
        Integer rowindexAggregate = hasColumnIndexGetRowIndexByValue(1,"合计",fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表");
        if (ObjectUtil.isNotNull(rowindexAggregate)) {
            // 千分位格式处理
            DecimalFormat decimalFormat = new DecimalFormat("#,##0.00");
            // 设置贷款本金
            crReportFillBO.setLoanPrincipal(new BigDecimal(getCellValueByIndex(rowindexAggregate, 2, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表")));
            crReportFillBO.setLoanPrincipalStr(decimalFormat.format(crReportFillBO.getLoanPrincipal()));
            // 设置应收利息（含罚息）
            crReportFillBO.setInterest(new BigDecimal(getCellValueByIndex(rowindexAggregate, 3, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表")));
            crReportFillBO.setInterestStr(decimalFormat.format(crReportFillBO.getInterest()));
            // 设置其他费用
            crReportFillBO.setOtherExpenses(new BigDecimal(getCellValueByIndex(rowindexAggregate, 4, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表")));
            crReportFillBO.setOtherExpensesStr(decimalFormat.format(crReportFillBO.getOtherExpenses()));
            // 设置各项债权合计
            crReportFillBO.setCRTotal(new BigDecimal(getCellValueByIndex(rowindexAggregate, 5, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表")));
            crReportFillBO.setCRTotalStr(decimalFormat.format(crReportFillBO.getCRTotal()));
            // 设置债权清算价值
            crReportFillBO.setTotalLiquidationValue(new BigDecimal(getCellValueByIndex(rowindexAggregate, 11, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总表")));
            crReportFillBO.setTotalLiquidationValueStr(decimalFormat.format(crReportFillBO.getTotalLiquidationValue()));
        }

        // 获取汇总表合计行的下标(用来获取债权汇总表的合计信息)
        Integer rowindexAggregateTenThousand = hasColumnIndexGetRowIndexByValue(1,"合计",fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表");
        if (ObjectUtil.isNotNull(rowindexAggregateTenThousand)) {
            // 千分位格式处理
            DecimalFormat decimalFormat = new DecimalFormat("#,##0.00");
            // 设置优先受偿价值合计
            crReportFillBO.setTotalPriorityClaimValue(new BigDecimal(getCellValueByIndex(rowindexAggregateTenThousand, 6, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表")));
            crReportFillBO.setTotalPriorityClaimValueStr(decimalFormat.format(crReportFillBO.getTotalPriorityClaimValue()));
            // 设置一般受偿价值合计
            crReportFillBO.setTotalGeneralClaimValue(new BigDecimal(getCellValueByIndex(rowindexAggregateTenThousand, 7, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表")));
            crReportFillBO.setTotalGeneralClaimValueStr(decimalFormat.format(crReportFillBO.getTotalGeneralClaimValue()));
            // 设置保证受偿价值合计
            crReportFillBO.setTotalGuaranteedClaimValue(new BigDecimal(getCellValueByIndex(rowindexAggregateTenThousand, 8, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表")));
            crReportFillBO.setTotalGuaranteedClaimValueStr(decimalFormat.format(crReportFillBO.getTotalGuaranteedClaimValue()));
            // 设置债权可回收金额合计
            crReportFillBO.setTotalRecoverableAmount(new BigDecimal(getCellValueByIndex(rowindexAggregateTenThousand, 9, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表")));
            crReportFillBO.setTotalRecoverableAmountStr(decimalFormat.format(crReportFillBO.getTotalRecoverableAmount()));

            // 设置债权清算价值合计
            crReportFillBO.setTotalCreditorLiquidationValue(new BigDecimal(getCellValueByIndex(rowindexAggregateTenThousand, 11, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表")));
            crReportFillBO.setTotalCreditorLiquidationValueStr(decimalFormat.format(crReportFillBO.getTotalCreditorLiquidationValue()));
        }

        // 设置本次所需处置时间与清算率取值
        crReportFillBO.setTimeNeededForThisDisposal(getCellValueByIndex(66, 1, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "债权清算率"));
        crReportFillBO.setLiquidationRateForThis(String.valueOf(new Double(getCellValueByIndex(66, 2, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "债权清算率")) * 100));

        // 获取汇总万元表合计行的下标
        Integer allSheetRowIndex = hasColumnIndexGetRowIndexByValue(1,"合计",fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表");
        if (ObjectUtil.isNotNull(allSheetRowIndex)) {
            // 设置债权综合受偿比率
            double compositeRecoveryRatioDouble = Double.parseDouble(getCellValueByIndex(allSheetRowIndex, 13, fileNetConfig.getBaseDir() + calculateDoc.getDocUrl(), "汇总万元表"));
            crReportFillBO.setCompositeRecoveryRatio(String.valueOf(compositeRecoveryRatioDouble * 100));
        }

        // 读取汇总万元表
        List<CRReportFillBO.Consolidated> consolidatedList = new ArrayList<>();
        ReadSheet readConsolidated = EasyExcel.readSheet("汇总万元表").head(CRReportFillBO.Consolidated.class).registerReadListener(new PageReadListener<CRReportFillBO.Consolidated>(consolidatedList::addAll)).headRowNumber(4).build();
        calculateReader.read(readConsolidated);
        consolidatedList.remove(consolidatedList.size() - 1);
        crReportFillBO.setConsolidatedList(consolidatedList);

        // 读取综合因素分析表信息
        List<CRReportFillBO.IntegratedAnalysis> integratedAnalysisList =  EasyExcel.read(fileNetConfig.getBaseDir() + calculateDoc.getDocUrl()).head(CRReportFillBO.IntegratedAnalysis.class).sheet("债权清算率").doReadSync();
        for (int i = 0; i < 56; i++) {
            if (ObjectUtil.isNotEmpty(integratedAnalysisList.get(i).getWeight())) {
                integratedAnalysisList.get(i).setWeight( (int) (new Double(integratedAnalysisList.get(i).getWeight()) * 100) + "%");
            }
        }
        crReportFillBO.setIntegratedAnalysisList(integratedAnalysisList.subList(0, 56)); // 后边的信息是清算率信息,此处不需要

        // 第一步:设置封面
        // 封面模板文件位置
        String coverTmplPath = fileNetConfig.getBaseDir() + coverTmpl.getSectionFileUrl() + coverTmpl.getSectionFileName();
        // 读取模板文件
        byte[] coverTmplByte = Files.readAllBytes(Paths.get(coverTmplPath));
        // 获取填充数据后的文件
        byte[] resultCoverByte = AsposeWordUtil.fillWordDataByMap(coverTmplByte, DataUtil.objToMap(crReportFillBO));
        // 将封面的字节流转成输入流以供后续使用
        InputStream coverIs = new ByteArrayInputStream(resultCoverByte);

        // 封面模板文件位置
        String digestTmplPath = fileNetConfig.getBaseDir() + digestTmpl.getSectionFileUrl() + digestTmpl.getSectionFileName();
        // 读取模板文件
        byte[] digestTmplByte = Files.readAllBytes(Paths.get(digestTmplPath));
        // 获取填充数据后的文件
        byte[] resultdigestByte = AsposeWordUtil.fillWordDataByMap(digestTmplByte, DataUtil.objToMap(crReportFillBO));
        // 将摘要的字节流转成输入流以供后续使用
        InputStream digestIs = new ByteArrayInputStream(resultdigestByte);

        // 第三步:设置正文内容
        // 封面模板文件位置
        String detailTmplPath = fileNetConfig.getBaseDir() + detailTmpl.getSectionFileUrl() + detailTmpl.getSectionFileName();
        // 读取模板文件
        byte[] detailTmplByte = Files.readAllBytes(Paths.get(detailTmplPath));
        // 获取填充数据后的文件
        byte[] resultDetailByte = AsposeWordUtil.fillWordDataByMap(detailTmplByte, DataUtil.objToMap(crReportFillBO));
        // 将正文的字节流转成输入流以供后续使用
        InputStream detailIs = new ByteArrayInputStream(resultDetailByte);

        // 将正文读取出来
        Document detailDoc = new Document(detailIs);

        // 第四步:设置债务人信息到正文
        // 债务人信息模板文件位置
        String debtorTmplPath = fileNetConfig.getBaseDir() + debtorTmpl.getSectionFileUrl() + debtorTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下
        for (int i = crReportFillBO.getReportBaseInfo().getDebtorInfoList().size(); i > 0; i--) {
            CRReportBaseInfoBO.DebtorInfo debtorInfo = crReportFillBO.getReportBaseInfo().getDebtorInfoList().get(i - 1);
            debtorInfo.setSort(i);
            // 读取模板文件
            byte[] debtorTmplByte = Files.readAllBytes(Paths.get(debtorTmplPath));
            // 获取填充数据后的文件
            byte[] resultConsignorByte = AsposeWordUtil.fillWordDataByDomain(debtorTmplByte, debtorInfo);
            InputStream debtorIs = new ByteArrayInputStream(resultConsignorByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(debtorIs), "insertDebtor", false, false);
            debtorIs.close();
        }

        // 第五步:设置保证人信息到正文
        // 企业保证人信息模板文件位置
        String enterpriseGuarantorTmplPath = fileNetConfig.getBaseDir() + enterpriseGuarantorTmpl.getSectionFileUrl() + enterpriseGuarantorTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下
        int enterpriseCount = crReportFillBO.getReportBaseInfo().getEnterpriseGuarantorList().size();
        for (int i = enterpriseCount; i > 0; i--) {
            CRReportBaseInfoBO.EnterpriseGuarantor enterpriseGuarantor = crReportFillBO.getReportBaseInfo().getEnterpriseGuarantorList().get(i - 1);
            enterpriseGuarantor.setSort(i);
            // 读取模板文件
            byte[] enterpriseGuarantorTmplByte = Files.readAllBytes(Paths.get(enterpriseGuarantorTmplPath));
            // 获取填充数据后的文件
            byte[] resultEnterpriseGuarantorByte = AsposeWordUtil.fillWordDataByDomain(enterpriseGuarantorTmplByte, enterpriseGuarantor);
            InputStream enterpriseGuarantorIs = new ByteArrayInputStream(resultEnterpriseGuarantorByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(enterpriseGuarantorIs), "insertEnterpriseGuarantor", false, false);
            enterpriseGuarantorIs.close();
        }

        // 自然人保证人信息模板文件位置
        String personGuarantorTmplPath = fileNetConfig.getBaseDir() + personGuarantorTmpl.getSectionFileUrl() + personGuarantorTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下(企业和自然人用同一个序号排序)
        for (int i = crReportFillBO.getReportBaseInfo().getPersonGuarantorList().size() + enterpriseCount; i > enterpriseCount; i--) {
            CRReportBaseInfoBO.PersonGuarantor personGuarantor = crReportFillBO.getReportBaseInfo().getPersonGuarantorList().get(i - enterpriseCount - 1);
            personGuarantor.setSort(i);
            // 读取模板文件
            byte[] personGuarantorTmplByte = Files.readAllBytes(Paths.get(personGuarantorTmplPath));
            // 获取填充数据后的文件
            byte[] resultPersonGuarantorByte = AsposeWordUtil.fillWordDataByDomain(personGuarantorTmplByte, personGuarantor);
            InputStream personGuarantorIs = new ByteArrayInputStream(resultPersonGuarantorByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(personGuarantorIs), "insertPersonGuarantor", false, false);
            personGuarantorIs.close();
        }

        // 设置资产情况
        String debtorAssetTmplPath = fileNetConfig.getBaseDir() + debtorAssetTmpl.getSectionFileUrl() + debtorAssetTmpl.getSectionFileName();
        String guarantorAssetTmplPath = fileNetConfig.getBaseDir() + guarantorAssetTmpl.getSectionFileUrl() + guarantorAssetTmpl.getSectionFileName();
        int guarantorAssetSort = crReportFillBO.getReportBaseInfo().getPersonGuarantorList().size() + enterpriseCount;
        for (int i = crReportFillBO.getReportBaseInfo().getAssetList().size(); i > 0; i--) {
            CRReportBaseInfoBO.Asset asset = crReportFillBO.getReportBaseInfo().getAssetList().get(i - 1);
            if (asset.getType().equals("债务人")) {
                // 读取模板文件
                byte[] debtorAssetTmplByte = Files.readAllBytes(Paths.get(debtorAssetTmplPath));
                // 获取填充数据后的文件
                byte[] resultDebtorAssetByte = AsposeWordUtil.fillWordDataByDomain(debtorAssetTmplByte, asset);
                InputStream debtorAssetIs = new ByteArrayInputStream(resultDebtorAssetByte);
                detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(debtorAssetIs), "insertDebtorAsset", false, false);
                debtorAssetIs.close();
            } else if (asset.getType().equals("保证人")) {
                asset.setSort(guarantorAssetSort);
                // 读取模板文件
                byte[] guarantorAssetTmplByte = Files.readAllBytes(Paths.get(guarantorAssetTmplPath));
                // 获取填充数据后的文件
                byte[] resultGuarantorAssetByte = AsposeWordUtil.fillWordDataByDomain(guarantorAssetTmplByte, asset);
                InputStream guarantorAssetIs = new ByteArrayInputStream(resultGuarantorAssetByte);
                detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(guarantorAssetIs), "insertGuarantorAsset", false, false);
                guarantorAssetIs.close();
                guarantorAssetSort--;
            }
        }

        // 诉讼情况信息模板文件位置
        String lawsuitTmplPath = fileNetConfig.getBaseDir() + lawsuitTmpl.getSectionFileUrl() + lawsuitTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下(企业和自然人用同一个序号排序)
        for (int i = crReportFillBO.getReportBaseInfo().getLawsuitList().size(); i > 0; i--) {
            CRReportBaseInfoBO.Lawsuit lawsuit = crReportFillBO.getReportBaseInfo().getLawsuitList().get(i - 1);
            lawsuit.setSort(i);
            // 读取模板文件
            byte[] lawsuitTmplByte = Files.readAllBytes(Paths.get(lawsuitTmplPath));
            // 获取填充数据后的文件
            byte[] resultLawsuitByte = AsposeWordUtil.fillWordDataByDomain(lawsuitTmplByte, lawsuit);
            InputStream lawsuitIs = new ByteArrayInputStream(resultLawsuitByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(lawsuitIs), "insertLawsuit", false, false);
            lawsuitIs.close();
        }

        // 保证人补充偿债来源分析
        String guarantorAdditionalDebtRepaymentSourcesPath = fileNetConfig.getBaseDir() + guarantorAdditionalDebtRepaymentSourcesTmpl.getSectionFileUrl() + guarantorAdditionalDebtRepaymentSourcesTmpl.getSectionFileName();
        // 插入时会导致先插入的排在后插入的后面,所以把顺序反转一下(企业和自然人用同一个序号排序)
        for (int i = guarantorInfoStrBOList.size(); i > 0; i--) {
            GuarantorInfoStrBO guarantorInfo = guarantorInfoStrBOList.get(i - 1);
            guarantorInfo.setSort(i);
            // 读取模板文件
            byte[] guarantorAdditionalDebtRepaymentSourcesTmplByte = Files.readAllBytes(Paths.get(guarantorAdditionalDebtRepaymentSourcesPath));
            // 获取填充数据后的文件
            byte[] resultGuarantorAdditionalDebtRepaymentSourcesByte = AsposeWordUtil.fillWordDataByDomain(guarantorAdditionalDebtRepaymentSourcesTmplByte, guarantorInfo);
            InputStream resultGuarantorAdditionalDebtRepaymentSourcesIs = new ByteArrayInputStream(resultGuarantorAdditionalDebtRepaymentSourcesByte);
            detailDoc = AsposeWordUtil.insertDocumentAfterBookMark(detailDoc, new Document(resultGuarantorAdditionalDebtRepaymentSourcesIs), "insertGuarantorAdditionalDebtRepaymentSo", false, false);
            resultGuarantorAdditionalDebtRepaymentSourcesIs.close();
        }

        // 第六步:设置主体框架
        // 框架生成后文件位置
        String docUrl = fileNetConfig.getAssetOutputReportPath() + System.currentTimeMillis() + "_债权分析报告.docx";
        String mainPath = fileNetConfig.getBaseDir() + docUrl;
        // 框架模板文件位置
        String mainTmplPath = fileNetConfig.getBaseDir() + mainTmpl.getFileUrl() + mainTmpl.getFileName();

        // 读取模板文件
        byte[] mainTmplByte = Files.readAllBytes(Paths.get(mainTmplPath));
        // 获取填充数据后的文件
        byte[] resultMainByte = AsposeWordUtil.fillWordDataByMap(mainTmplByte, DataUtil.objToMap(crReportFillBO));
        InputStream mainIs = new ByteArrayInputStream(resultMainByte);
        // 将框架读取出来
        Document mainDoc = new Document(mainIs);
        // 将摘要插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, new Document(digestIs), "insertDigest", false, false);
        // 将正文插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, detailDoc, "insertDetail", false, false);

        // 目录模板文件位置
        String catalogueTmplPath = fileNetConfig.getBaseDir() + catalogueTmpl.getSectionFileUrl() + catalogueTmpl.getSectionFileName();
        // 将框架读取出来
        Document catalogueDoc = new Document(catalogueTmplPath);
        // 将目录插入到框架
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, catalogueDoc, "insertCatalogue", false, false);

        // 创建综合因素分析表
        Document tableDoc = new Document();
        DocumentBuilder tableBuilder = new DocumentBuilder(tableDoc);
        tableBuilder.startBookmark("table");
        tableBuilder.endBookmark("table");
        AsposeWordUtil.createTable(tableDoc, CRReportFillBO.IntegratedAnalysis.class, crReportFillBO.getIntegratedAnalysisList(), "table", false);
        //将第一列整列合并
        tableBuilder.moveToCell(0,1,0,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 2; i < 56; i++ ) {
            tableBuilder.moveToCell(0, i, 0, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第2行合并到31行
        tableBuilder.moveToCell(0,1,1,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 2; i < 31; i++ ) {
            tableBuilder.moveToCell(0, i, 1, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第33行合并到38行
        tableBuilder.moveToCell(0,32,1,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 33; i < 38; i++ ) {
            tableBuilder.moveToCell(0, i, 1, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第40行合并到47行
        tableBuilder.moveToCell(0,39,1,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 40; i < 47; i++ ) {
            tableBuilder.moveToCell(0, i, 1, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第49行合并到51行
        tableBuilder.moveToCell(0,48,1,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 49; i < 51; i++ ) {
            tableBuilder.moveToCell(0, i, 1, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第53行合并到55行
        tableBuilder.moveToCell(0,52,1,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 53; i < 55; i++ ) {
            tableBuilder.moveToCell(0, i, 1, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }

        //将第三列第2行合并到31行
        tableBuilder.moveToCell(0,1,2,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 2; i < 31; i++ ) {
            tableBuilder.moveToCell(0, i, 2, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第33行合并到38行
        tableBuilder.moveToCell(0,32,2,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 33; i < 38; i++ ) {
            tableBuilder.moveToCell(0, i, 2, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第40行合并到47行
        tableBuilder.moveToCell(0,39,2,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 40; i < 47; i++ ) {
            tableBuilder.moveToCell(0, i, 2, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第49行合并到51行
        tableBuilder.moveToCell(0,48,2,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 49; i < 51; i++ ) {
            tableBuilder.moveToCell(0, i, 2, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        //将第二列第53行合并到55行
        tableBuilder.moveToCell(0,52,2,0);
        tableBuilder.getCellFormat().setVerticalMerge(CellMerge.FIRST);//合并的第一个单元格 保留的值是第一个单元格的值
        for(int i = 53; i < 55; i++ ) {
            tableBuilder.moveToCell(0, i, 2, 0);
            tableBuilder.getCellFormat().setVerticalMerge(CellMerge.PREVIOUS);
        }
        AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, tableDoc, "insertIntegratedAnalysis", false, false);

        // 第七步:删除预留书签插入位置的空白行
        DocumentBuilder mainBuilder = new DocumentBuilder(mainDoc);
        mainBuilder.moveToBookmark("insertDetail");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertDigest");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertCatalogue");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertDebtor");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertEnterpriseGuarantor");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertPersonGuarantor");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertDebtorAsset");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertGuarantorAsset");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertLawsuit");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertIntegratedAnalysis");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("insertGuarantorAdditionalDebtRepaymentSo");
        mainBuilder.getCurrentParagraph().remove();
        mainBuilder.moveToBookmark("table");
        mainBuilder.getCurrentParagraph().remove();

        // 第八步:生成并保存文档
        // 将封面和文档进行拼接
        Document coverDoc = new Document(coverIs);
        coverDoc.appendDocument(mainDoc, ImportFormatMode.KEEP_SOURCE_FORMATTING);
        // 更新目录页码
        AsposeWordUtil.updateCatalogueLink(coverDoc);

        // 开启word的修订模式(修改留痕)
        coverDoc.setTrackRevisions(true);

        coverDoc.save(mainPath);

        // 关闭各个段落的流
        coverIs.close();
        digestIs.close();
        detailIs.close();
        mainIs.close();

        // 更新报告进度
        assetsReportMapper.updateReportProgress(reportId, "FINISHED");

        // 保存文档信息到数据库
        // 设置文档信息
        AssetsReportVO assetsReportVO = assetsReportMapper.getReportInfoForDoc(reportId);
        DocumentProduction documentProduction = documentProductionService.getDocProdByBusinessId(assetsReportVO.getProjectId(), reportId);

        if (ObjectUtil.isNotNull(documentProduction)) { // 判断文档信息是否为空,不为空说明以前生成过文档,则更新文档版本号
            documentProduction.setDocVersion(documentProduction.getDocVersion() + 1);
        } else { // 为空重新new对象
            documentProduction = new DocumentProduction();
        }
        // 设置文档信息
        documentProduction.setBusinessType("ASSETS")
                .setBusinessId(assetsReportVO.getProjectId())
                .setDocType(CR_REPORT.getName())
                .setDocName(assetsReportVO.getReportName() + ".docx")
                .setConsignor(assetsReportVO.getPrincipal())
                .setDocUrl(docUrl)
                .setCreateUserId(StpUtil.getLoginIdAsLong())
                .setBusinessCate(assetsReportVO.getProjectTypeName())
                .setDocNo(ObjectUtil.isNotNull(crReportFillBO.getProductionNo()) ? crReportFillBO.getProductionNo() : "")
                .setBusinessSubId(reportId);
        return documentProductionService.saveDocumentProduction(documentProduction);
    }
}
