package com.dayou.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.dto.DocumentProductionQueryDTO;
import com.dayou.entity.DocumentProduction;
import com.dayou.enums.BusinessEnum;
import com.dayou.enums.DocumentType;
import com.dayou.mapper.DocumentProductionMapper;
import com.dayou.service.DocumentProductionService;
import com.dayou.service.HouseExcelDataService;
import com.dayou.vo.HouseGuarantyBaseVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static com.dayou.enums.BusinessEnum.HouseSubBusiness.GUARANTY;

@Service
public class DocumentProductionServiceImpl extends ServiceImpl<DocumentProductionMapper, DocumentProduction>
        implements DocumentProductionService {

    @Autowired
    private DocumentProductionMapper documentProductionMapper;

    @Autowired
    private HouseExcelDataService houseExcelDataService;

    @Override
    public Page<DocumentProduction> getPage(DocumentProductionQueryDTO dto, Page page) {
        return  documentProductionMapper.getPage(page,dto);
    }

    /**
     * 保存文档信息
     * @param documentProduction 文档信息
     * @return Boolean
     */
    @Override
    public Boolean saveDocumentProduction(DocumentProduction documentProduction) {
        if (ObjectUtil.isNotNull(documentProduction.getId())) {
            return this.updateById(documentProduction);
        }else {
            return this.save(documentProduction);
        }
    }

    /**
     * 通过业务id获取文档信息
     * @param businessId 业务id
     * @param businessSubId 业务二级id
     * @return DocumentProduction
     */
    @Override
    public DocumentProduction getDocProdByBusinessId(Long businessId, Long businessSubId) {
        return documentProductionMapper.getDocProdByBusinessId(businessId, businessSubId);
    }

    @Override
    public Page<DocumentProduction> houseDocumentProduction(DocumentProductionQueryDTO dto, Page page) {
        return documentProductionMapper.houseDocumentProduction(page,dto);
    }

    @Override
    public Long saveDocumentProductionByExcelId(Long excelId) {
        HouseGuarantyBaseVO baseInfo = houseExcelDataService.getBaseInfo(excelId);
        DocumentProduction documentProduction = new DocumentProduction();
        BeanUtil.copyProperties(baseInfo,documentProduction);
        documentProduction.setBusinessId(excelId);
        documentProduction.setDocType(DocumentType.REPORT.getName());
        documentProduction.setBusinessCate(GUARANTY.getMsg());
        documentProduction.setCreateUserId(StpUtil.getLoginIdAsLong());
        documentProduction.setBusinessType(BusinessEnum.HOUSE_BUSINESS.getMsg());
        this.save(documentProduction);
        return documentProduction.getId();
    }
}
