package com.dayou.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.common.BaseEntity;
import com.dayou.doc.house.BaseInfoDO;
import com.dayou.doc.house.ConditionDO;
import com.dayou.doc.house.GuarantyResultDO;
import com.dayou.dto.LngLatDTO;
import com.dayou.entity.CertificateFixedAssets;
import com.dayou.entity.HouseExcelData;
import com.dayou.entity.HouseGuarantyArea;
import com.dayou.enums.BusinessEnum;
import com.dayou.mapper.HouseExcelDataMapper;
import com.dayou.mapper.HouseGuarantyAreaMapper;
import com.dayou.service.CertificateFixedAssetsService;
import com.dayou.service.HouseExcelDataService;
import com.dayou.service.HouseGuarantyAreaService;
import com.dayou.utils.EasyExcelUtil;
import com.dayou.vo.HouseGuarantyBaseVO;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import static org.apache.poi.ss.usermodel.CellType.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;


@Service
public class HouseExcelDataServiceImpl extends ServiceImpl<HouseExcelDataMapper, HouseExcelData> implements HouseExcelDataService {

    @Autowired
    private CertificateFixedAssetsService certificateFixedAssetsService;
    @Autowired
    private HouseExcelDataMapper houseExcelDataMapper;
    @Autowired
    private HouseGuarantyAreaService houseGuarantyAreaService;

    @Override
    public Long analysisOriginExcelData(MultipartFile excel,String path) {
        XSSFWorkbook workbook = null;
        try {
            workbook = new XSSFWorkbook(excel.getInputStream());
            Long excelId = doMainContent(workbook, path);
            doTargetAreaInfo(workbook,excelId);
            return excelId;
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public HouseGuarantyBaseVO getBaseInfo(Long excelId) {
        HouseExcelData base = getBase(excelId);
        return BeanUtil.copyProperties(base, HouseGuarantyBaseVO.class);
    }

    @Override
    public BaseInfoDO getBaseDO(Long excelId) {
        HouseExcelData base = getBase(excelId);
        return BeanUtil.copyProperties(base, BaseInfoDO.class);
    }

    @Override
    public ConditionDO getConditionDO(Long businessId) {
        HouseExcelData hed = houseExcelDataMapper.getConditionDO(businessId);
        ConditionDO condition = BeanUtil.copyProperties(hed, ConditionDO.class);
        //一般假设
        StringBuffer normalAssumptions = new StringBuffer();
        normalAssumptions.append(hed.getNormalAssumption1()).append("\n")
                .append(hed.getNormalAssumption2()).append("\n")
                .append(hed.getNormalAssumption3()).append("\n")
                .append(hed.getNormalAssumption4()).append("\n")
                .append(hed.getNormalAssumption5()).append("\n")
                .append(hed.getNormalAssumption6()).append("\n")
                .append(hed.getNormalAssumption7()).append("\n")
                .append(hed.getNormalAssumption8()).append("\n")
                .append(hed.getNormalAssumption9()).append("\n")
                .append(hed.getNormalAssumption10());
        condition.setYBJS(normalAssumptions.toString());
        //未定事项假设
        StringBuffer undefinedAssumptions = new StringBuffer();
        undefinedAssumptions.append(hed.getUndefinedAssumption1()).append("\n")
                .append(hed.getUndefinedAssumption2()).append("\n")
                .append(hed.getUndefinedAssumption3()).append("\n")
                .append(hed.getUndefinedAssumption4()).append("\n")
                .append(hed.getUndefinedAssumption5()).append("\n")
                .append(hed.getUndefinedAssumption6());
        condition.setWDSXJS(undefinedAssumptions.toString());
        //不相一致假设
        StringBuffer differentAssumptions = new StringBuffer();
        differentAssumptions.append(hed.getDifferentAssumption1()).append("\n")
                .append(hed.getDifferentAssumption2());
        condition.setBXYZJS(differentAssumptions.toString());
        //依据不足假设
        StringBuffer deficiencyAssumptions = new StringBuffer();
        deficiencyAssumptions.append(hed.getDeficiencyAssumption1()).append("\n")
                .append(hed.getDeficiencyAssumption2()).append("\n")
                .append(hed.getDeficiencyAssumption3()).append("\n")
                .append(hed.getDeficiencyAssumption4()).append("\n")
                .append(hed.getDeficiencyAssumption5());
        condition.setYJBZJS(deficiencyAssumptions.toString());
        //限制条件
        StringBuffer limitConditions = new StringBuffer();
        limitConditions.append(hed.getLimitCondition1()).append("\n")
                .append(hed.getLimitCondition2()).append("\n")
                .append(hed.getLimitCondition3()).append("\n")
                .append(hed.getLimitCondition4()).append("\n")
                .append(hed.getLimitCondition5()).append("\n")
                .append(hed.getLimitCondition6()).append("\n")
                .append(hed.getLimitCondition7()).append("\n")
                .append(hed.getLimitCondition8());
        condition.setXZTJ(limitConditions.toString());
        //使用本报告的一般说明
        StringBuffer normalExplain = new StringBuffer();
        normalExplain.append(hed.getNormalExplain1()).append("\n")
                .append(hed.getNormalExplain2()).append("\n")
                .append(hed.getNormalExplain3()).append("\n")
                .append(hed.getNormalExplain4()).append("\n")
                .append(hed.getNormalExplain5()).append("\n")
                .append(hed.getNormalExplain6()).append("\n")
                .append(hed.getNormalExplain7()).append("\n")
                .append(hed.getNormalExplain8()).append("\n")
                .append(hed.getNormalExplain9());
        condition.setSYBGSM(normalExplain.toString());
        //估价中的特殊处理事项
        StringBuffer expeHandle = new StringBuffer();
        expeHandle.append(hed.getExpeHandle1()).append("\n")
                .append(hed.getExpeHandle2()).append("\n")
                .append(hed.getExpeHandle3()).append("\n")
                .append(hed.getExpeHandle4());
        condition.setTSSXCL(expeHandle.toString());
        return condition;
    }

    @Override
    public GuarantyResultDO getResultReport(Long businessId) {
        HouseExcelData hed = houseExcelDataMapper.getResultReport(businessId);
        GuarantyResultDO guarantyResultDO = BeanUtil.copyProperties(hed, GuarantyResultDO.class);
        StringBuffer valueMeanings = new StringBuffer();
        valueMeanings.append(hed.getValueMeaning1()).append("\n")
                .append(hed.getValueMeaning2()).append("\n")
                .append(hed.getValueMeaning3()).append("\n")
                .append(hed.getValueMeaning4());
        guarantyResultDO.setValueMeaning(valueMeanings.toString());

        //获取估价对象区位状况描述
        List<HouseGuarantyArea> areas = houseGuarantyAreaService.list(new LambdaQueryWrapper<HouseGuarantyArea>().eq(HouseGuarantyArea::getExcelId, businessId)
                .select(HouseGuarantyArea::getDocContent).eq(BaseEntity::getDeleteStatus, false).orderByAsc(HouseGuarantyArea::getId));
        StringBuffer docContent = new StringBuffer();
        for (int i = 0; i < areas.size(); i++) {
            if (i!= areas.size()-1){
                docContent.append("\t").append(areas.get(i).getDocContent()).append("\n");
            }else {
                docContent.append("\t").append(areas.get(i).getDocContent());
            }

        }
        guarantyResultDO.setAreaInfo(docContent.toString());
        return guarantyResultDO;
    }


    private HouseExcelData getBase(Long excelId) {
        LambdaQueryWrapper<HouseExcelData> queryWrapper = new LambdaQueryWrapper<HouseExcelData>().eq(HouseExcelData::getId, excelId)
                .select(BaseEntity::getId,
                        HouseExcelData::getDocName,
                        HouseExcelData::getDocNo,
                        HouseExcelData::getBusinessType,
                        HouseExcelData::getConsignor,
                        HouseExcelData::getOwner,
                        HouseExcelData::getAppraiser1,
                        HouseExcelData::getAppraiser2,
                        HouseExcelData::getAppraNo1,
                        HouseExcelData::getAppraNo2,
                        HouseExcelData::getDocDate,
                        HouseExcelData::getValueTiming,
                        HouseExcelData::getWorkMethod,
                        HouseExcelData::getWorkPurpose,
                        HouseExcelData::getSummary1,
                        HouseExcelData::getSummary2,
                        HouseExcelData::getWorkPurpose,
                        HouseExcelData::getExcelUri);
        return this.getOne(queryWrapper);
    }

    /**
     * 解析正文内容
     * @param workbook
     * @param path
     * @return
     */
    private Long doMainContent(XSSFWorkbook workbook,String path ){
        Map<String, String> excelData = EasyExcelUtil.readExcelData(workbook, "正文内容", 1);
        HouseExcelData houseExcelData = BeanUtil.fillBeanWithMap(excelData, new HouseExcelData(), true);
        houseExcelData.setBusinessType(BusinessEnum.HouseSubBusiness.GUARANTY.getCode());
        houseExcelData.setUploadUserId(StpUtil.getLoginIdAsLong());
        houseExcelData.setExcelUri(path);
        this.save(houseExcelData);
        return houseExcelData.getId();
    }

    /**
     * 解析权属证件信息
     * @param workbook
     */
    private void doCertificates(XSSFWorkbook workbook,Long businessId){
        Sheet sheet = workbook.getSheet("权属信息");
        int lastRowNum = sheet.getLastRowNum();
        List<CertificateFixedAssets> certificates = new ArrayList<>();
        for (int row = 3; row <= lastRowNum; row++) {
            CertificateFixedAssets certificate = new CertificateFixedAssets();
            Row rowValue = sheet.getRow(row);
            //估价对象
            Cell cell0 = rowValue.getCell(0);
            certificate.setTid(cell0.getStringCellValue());
            //序号
            Cell cell1 = rowValue.getCell(1);
            if (NUMERIC.equals(cell1.getCellType())){
                certificate.setTno(String.valueOf(cell1.getNumericCellValue()));
            }else {
                certificate.setTno(cell1.getStringCellValue());
            }
            //不动产权证书号
            Cell cell2 = rowValue.getCell(2);
            certificate.setCertificateNo(cell2.getStringCellValue());
            //权利人
            Cell cell3 = rowValue.getCell(3);
            certificate.setOwnShipUser(cell3.getStringCellValue());
            //共有情况
            Cell cell4 = rowValue.getCell(4);
            certificate.setOwnShipInfo(cell4.getStringCellValue());
            //坐落
            Cell cell5 = rowValue.getCell(5);
            certificate.setLocation(cell5.getStringCellValue());
            //不动产单元号
            Cell cell6 = rowValue.getCell(6);
            certificate.setUnitNo(cell6.getStringCellValue());
            //权利类型
            Cell cell7 = rowValue.getCell(7);
            certificate.setOwnShipType(cell7.getStringCellValue());
            //权利性质
            Cell cell8 = rowValue.getCell(8);
            Cell cell9 = rowValue.getCell(9);
            Cell cell10 = rowValue.getCell(10);
            certificate.setOwnShipNature(cell8.getStringCellValue()+cell9.getStringCellValue()+cell10.getStringCellValue());
            //用途
            Cell cell11 = rowValue.getCell(11);
            Cell cell12 = rowValue.getCell(12);
            Cell cell13 = rowValue.getCell(13);
            certificate.setUseTo(cell11.getStringCellValue()+cell12.getStringCellValue()+cell13.getStringCellValue());
            //面积
            Cell cell14 = rowValue.getCell(14);
            Cell cell15 = rowValue.getCell(15);
            String acreage = "";
            if (NUMERIC.equals(cell15.getCellType())){
                acreage = String.valueOf(cell15.getNumericCellValue());
            }else {
                acreage = cell15.getStringCellValue();
            }
            Cell cell16 = rowValue.getCell(16);
            Cell cell17 = rowValue.getCell(17);
            String acreage1 = "";
            if (NUMERIC.equals(cell17.getCellType())){
                acreage1 = String.valueOf(cell17.getNumericCellValue());
            }else {
                acreage1 = cell17.getStringCellValue();
            }
            Cell cell18 = rowValue.getCell(18);
            certificate.setAcreageDesc(cell14.getStringCellValue()+acreage+cell16.getStringCellValue()
                +acreage1+cell18.getStringCellValue());
            //使用期限
            Cell cell19 = rowValue.getCell(19);
            certificate.setExpireDateDesc(cell19.getStringCellValue());
            //权利其他状况
            Cell cell20 = rowValue.getCell(20);
            certificate.setOwnShipOtherInfo(cell20.getStringCellValue());
            //附记
            Cell cell21 = rowValue.getCell(21);
            certificate.setAttachInfo(cell21.getStringCellValue());
            //分摊土地面积
            Cell cell22 = rowValue.getCell(22);
            if (NUMERIC.equals(cell22.getCellType())){
                certificate.setLandAcreage(String.valueOf(cell22.getNumericCellValue()));
            }else {
                certificate.setLandAcreage(cell22.getStringCellValue());
            }
            certificate.setBusinessType(BusinessEnum.HOUSE_BUSINESS.getMsg());
            certificate.setBusinessId(businessId);
            certificate.setCreateUserId(StpUtil.getLoginIdAsLong());
            certificates.add(certificate);
        }
        certificateFixedAssetsService.saveBatch(certificates);
    }

    private void doTargetAreaInfo(XSSFWorkbook workbook,Long excelId) throws InterruptedException {
        Sheet sheet = workbook.getSheet("估价对象描述");
        int lastRowNum = sheet.getLastRowNum();
        List<HouseGuarantyArea> houseGuarantyAreas = new ArrayList();
        for (int row = 1; row <= lastRowNum; row++) {
            HouseGuarantyArea houseGuarantyArea = new HouseGuarantyArea();
            Row rowValue = sheet.getRow(row);
            //估价对象
            Cell cell0 = rowValue.getCell(0);
            houseGuarantyArea.setTid(cell0.getStringCellValue());
            //坐落
            Cell cell1 = rowValue.getCell(1);
            houseGuarantyArea.setLocation(cell1.getStringCellValue());
            houseGuarantyArea.setExcelId(excelId);
            //获取经纬度 高德接口QPS太高会限制请求。估此处休眠半秒。
            Thread.sleep(500);
            LngLatDTO lngLatDTO = houseGuarantyAreaService.queryLngLat(houseGuarantyArea.getLocation());
            houseGuarantyArea.setLng(lngLatDTO.getLng());
            houseGuarantyArea.setLat(lngLatDTO.getLat());
            houseGuarantyAreas.add(houseGuarantyArea);
        }
        houseGuarantyAreaService.saveBatch(houseGuarantyAreas);
    }
}
