package com.dayou.service.impl;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.common.BaseEntity;
import com.dayou.dto.LngLatDTO;
import com.dayou.dto.RoadDTO;
import com.dayou.entity.HouseGuarantyArea;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.HouseGuarantyAreaMapper;
import com.dayou.service.HouseGuarantyAreaService;
import com.dayou.utils.HttpKit;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class HouseGuarantyAreaServiceImpl extends ServiceImpl<HouseGuarantyAreaMapper, HouseGuarantyArea> implements HouseGuarantyAreaService {

    @Value("${map.url}")
    private String mapGeoApi;

    @Value("${map.key}")
    private String mapKey;



    @Autowired
    private HouseGuarantyAreaMapper houseGuarantyAreaMapper;

    @Override
    public HouseGuarantyArea getByLngLat(String lngLat) {
        List<BigDecimal> lngLats = Arrays.stream(lngLat.split(","))
                .map(BigDecimal::new).collect(Collectors.toList());
        BigDecimal lng = lngLats.get(0);
        BigDecimal lat = lngLats.get(1);
        return houseGuarantyAreaMapper.getByLngLat(lng,lat);
    }



    @Override
    public List<HouseGuarantyArea> getAreasList(Long id) {
        List<HouseGuarantyArea> list = houseGuarantyAreaMapper.getAreasList(id);
        return list;
    }

    @Override
    public List<HouseGuarantyArea> getAreaInfo(Long excelId) {
        List<HouseGuarantyArea> ares = houseGuarantyAreaMapper.getAreaInfo(excelId);
        for (HouseGuarantyArea are :ares){
            are.setRoad(JSON.parseArray(are.getRoad1(), RoadDTO.class));
            are.setPublicTransport(JSON.parseArray(are.getPublicTransport1(),String.class));
            are.setPark(JSON.parseArray(are.getPark1(),String.class));
            are.setBusiness(JSON.parseArray(are.getBusiness1(),String.class));
            are.setCommunity(JSON.parseArray(are.getPublicTransport1(),String.class));
            are.setEducation(JSON.parseArray(are.getEducation1(),String.class));
            are.setHospital(JSON.parseArray(are.getHospital1(),String.class));
            are.setBank(JSON.parseArray(are.getBank1(),String.class));
            are.setHotel(JSON.parseArray(are.getHotel1(),String.class));
            are.setSpot(JSON.parseArray(are.getSpot1(),String.class));
        }
        return ares;
    }

    @Override
    public LngLatDTO queryLngLat(String address) {
        Map<String,String> params = new HashMap<>();
        params.put("address",address);
        params.put("key",mapKey);
        String ret = HttpKit.sendGet(mapGeoApi, params);
        JSONObject jsonObject = JSON.parseObject(ret);
        log.info(ret);
        if (jsonObject.get("status").equals("1")) {
            JSONArray geocodes = jsonObject.getJSONArray("geocodes");
            try {
                JSONObject geo = JSON.parseObject(geocodes.get(0).toString());
                String location = (String) geo.get("location");
                List<String> point = Arrays.asList(location.split(","));
                LngLatDTO lngLatDTO = new LngLatDTO();
                lngLatDTO.setLng(new BigDecimal(point.get(0)));
                lngLatDTO.setLat(new BigDecimal(point.get(1)));
                return lngLatDTO;
            } catch (Exception e) {
                ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"高德接口：获取经纬度数据失败: 地址有误。");
            }
        }else {
            if (jsonObject.get("info").equals("ENGINE_RESPONSE_DATA_ERROR")) {
                ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"高德接口："+address +"地址有误，无法解析出经纬度。");
            }else {
                ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"高德接口：获取经纬度数据失败:"+jsonObject.get("info"));
            }
        }
        return null;
    }

    @Override
    public Boolean updateArea(HouseGuarantyArea houseGuarantyArea) {
        String road = JSON.toJSONString(houseGuarantyArea.getRoad());
        String publicTransport = JSON.toJSONString(houseGuarantyArea.getPublicTransport());
        String park = JSON.toJSONString(houseGuarantyArea.getPark());
        String business = JSON.toJSONString(houseGuarantyArea.getBusiness());
        String community = JSON.toJSONString(houseGuarantyArea.getCommunity());
        String education = JSON.toJSONString(houseGuarantyArea.getEducation());
        String hospital = JSON.toJSONString(houseGuarantyArea.getHospital());
        String bank = JSON.toJSONString(houseGuarantyArea.getBank());
        String hotel = JSON.toJSONString(houseGuarantyArea.getHotel());
        String spot = JSON.toJSONString(houseGuarantyArea.getSpot());
        return this.update(new LambdaUpdateWrapper<HouseGuarantyArea>().eq(BaseEntity::getId,houseGuarantyArea.getId())
                .set(HouseGuarantyArea::getRoad,road).set(HouseGuarantyArea::getPublicTransport,publicTransport)
                .set(HouseGuarantyArea::getPark,park).set(HouseGuarantyArea::getBusiness,business).set(HouseGuarantyArea::getCommunity,community)
                .set(HouseGuarantyArea::getEducation,education).set(HouseGuarantyArea::getHospital,hospital)
                .set(HouseGuarantyArea::getBank,bank).set(HouseGuarantyArea::getHotel,hotel).set(HouseGuarantyArea::getSpot,spot)
                .set(HouseGuarantyArea::getDocContent,houseGuarantyArea.getDocContent()));
    }
}
