package com.dayou.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.aspose.words.Document;
import com.aspose.words.ImportFormatMode;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.common.BaseEntity;
import com.dayou.config.FileNetConfig;
import com.dayou.entity.DocumentProduction;
import com.dayou.entity.HouseGuarantyProcess;
import com.dayou.enums.BusinessEnum;
import com.dayou.mapper.HouseGuarantyProcessMapper;
import com.dayou.service.DocumentProductionService;
import com.dayou.service.HouseGuarantyProcessService;
import com.dayou.vo.ProcessVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.dayou.enums.BusinessEnum.HouseGuarantyProcess.*;

@Service
public class HouseGuarantyProcessServiceImpl extends ServiceImpl<HouseGuarantyProcessMapper, HouseGuarantyProcess> implements HouseGuarantyProcessService {

    @Autowired
    private HouseGuarantyProcessMapper houseGuarantyProcessMapper;

    @Autowired
    private FileNetConfig fileNetConfig;

    @Autowired
    private DocumentProductionService documentProductionService;


    @Override
    public HouseGuarantyProcess getHouseGuarantyProcess(Long id) {
        HouseGuarantyProcess process = houseGuarantyProcessMapper.getProcessByDocId(id);
        return process;
    }

    @Override
    public List<ProcessVO> getHouseGuarantyProcesses(Long id) {
        List<BusinessEnum.HouseGuarantyProcess> aProcesses = Arrays.stream(BusinessEnum.HouseGuarantyProcess.values()).collect(Collectors.toList());

        if (id != null){
            HouseGuarantyProcess parent = this.getOne(new LambdaQueryWrapper<HouseGuarantyProcess>().eq(BaseEntity::getId, id).select(HouseGuarantyProcess::getHome,HouseGuarantyProcess::getExcelId));
            Map<String, List<HouseGuarantyProcess>> collect = this.list(new LambdaQueryWrapper<HouseGuarantyProcess>().select(BaseEntity::getId, HouseGuarantyProcess::getHome,
                            BaseEntity::getUpdateTime, HouseGuarantyProcess::getProcessName,HouseGuarantyProcess::getDocUrl)
                    .eq(HouseGuarantyProcess::getHome, parent.getHome()).orderByDesc(BaseEntity::getCreateTime)).stream().collect(Collectors.groupingBy(HouseGuarantyProcess::getProcessName));

            List<ProcessVO> list = aProcesses.stream().map(x -> {
                ProcessVO processVO = new ProcessVO();
                processVO.setName(x.getMsg());
                processVO.setRouter(x.getRouter());
                List<HouseGuarantyProcess> houseGuarantyProcesses = collect.get(x.getMsg());
                if (CollectionUtil.isNotEmpty(houseGuarantyProcesses)){
                    HouseGuarantyProcess process = houseGuarantyProcesses.get(0);
                    if (process != null) {
                        processVO.setId(process.getId());
                        processVO.setUpdateDate(process.getUpdateTime());
                        processVO.setState(Boolean.TRUE);
                        processVO.setDocUrl(process.getDocUrl());
                    }
                }
                return processVO;
            }).collect(Collectors.toList());
            return list;
        }
        List<ProcessVO> list = aProcesses.stream().map(x -> {
            ProcessVO processVO = new ProcessVO();
            processVO.setName(x.getMsg());
            processVO.setRouter(x.getRouter());
            return processVO;
        }).collect(Collectors.toList());
        return list;
    }

    @Override
    public Boolean houseGuarantyMerge(Long id) throws Exception {
        HouseGuarantyProcess parent = this.getOne(new LambdaQueryWrapper<HouseGuarantyProcess>().eq(BaseEntity::getId, id).select(HouseGuarantyProcess::getHome,BaseEntity::getId,HouseGuarantyProcess::getExcelId));
        String baseDir = fileNetConfig.getBaseDir();
        String home = parent.getHome();
        List<HouseGuarantyProcess> docs = houseGuarantyProcessMapper.queryMaxCreateTimeDoc(home);
        HouseGuarantyProcess consignorLetter = docs.stream().filter(x -> CONSIGNOR_LETTER.getMsg().equals(x.getProcessName())).findFirst().get();
        HouseGuarantyProcess explain = docs.stream().filter(x -> STATEMENT_CONDITIONS_EXPLAIN.getMsg().equals(x.getProcessName())).findFirst().get();
        HouseGuarantyProcess resultReport = docs.stream().filter(x -> GUARANTY_RESULT_REPORT.getMsg().equals(x.getProcessName())).findFirst().get();
        Document cosignorDocument = new Document(baseDir+consignorLetter.getDocUrl());
        Document explainDocument = new Document(baseDir+explain.getDocUrl());
        Document resultDocument = new Document(baseDir+resultReport.getDocUrl());
        cosignorDocument.appendDocument(explainDocument, ImportFormatMode.KEEP_SOURCE_FORMATTING);
        cosignorDocument.appendDocument(resultDocument,ImportFormatMode.KEEP_SOURCE_FORMATTING);
        String subUrl = home +"FINAL_REPORT"+System.currentTimeMillis() + ".docx";
        String guarantyResultName = baseDir + subUrl;
        cosignorDocument.save(guarantyResultName);

        DocumentProduction dp = documentProductionService.getOne(new LambdaQueryWrapper<DocumentProduction>().select(DocumentProduction::getDocVersion).eq(DocumentProduction::getBusinessId, parent.getExcelId()));

        //更新document_production
        return documentProductionService.update(new LambdaUpdateWrapper<DocumentProduction>()
                .set(DocumentProduction::getDocUrl,subUrl)
                .set(BaseEntity::getUpdateTime,new Date())
                .set(DocumentProduction::getDocVersion,dp.getDocVersion()+1)
                .eq(DocumentProduction::getBusinessId,parent.getExcelId()));

    }

}
