package com.dayou.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import com.aspose.cells.SaveFormat;
import com.aspose.cells.Workbook;
import com.aspose.words.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dayou.common.BaseEntity;
import com.dayou.config.FileNetConfig;
import com.dayou.doc.house.BaseInfoDO;
import com.dayou.doc.house.ConditionDO;
import com.dayou.doc.house.GuarantyResultDO;
import com.dayou.dto.HouseGuarantyTableDTO;
import com.dayou.dto.HouseRightsDTO;
import com.dayou.entity.*;
import com.dayou.enums.DocumentType;
import com.dayou.enums.BusinessEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.service.*;
import com.dayou.table.HouseEntityTable;
import com.dayou.utils.*;
import com.dayou.utils.table.WordTableHelper;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;

import static com.dayou.constants.HouseGuarantyDoc.*;
import static com.dayou.enums.BusinessEnum.HouseGuarantyProcess.GUARANTY_RESULT_REPORT;
import static com.dayou.enums.BusinessEnum.HouseGuarantyProcess.STATEMENT_CONDITIONS_EXPLAIN;
import static com.dayou.enums.BusinessEnum.HouseSubBusiness.GUARANTY;

@Service
public class HouseGuarantyServiceImpl implements HouseGuarantyService {


    @Autowired
    private WordTableHelper wordTableHelper;

    @Autowired
    private HouseGuarantyProcessService houseGuarantyProcessService;

    @Autowired
    private FileNetConfig fileNetConfig;

    @Autowired
    private DocumentProductionService documentProductionService;

    @Autowired
    private CertificateFixedAssetsService certificateFixedAssetsService;

    @Autowired
    private CertificateHouseOwnService certificateHouseOwnService;

    @Autowired
    private CertificateLandUseService certificateLandUseService;

    @Autowired
    private HouseExcelDataService houseExcelDataService;

    @Override
    public Long createTableWord(HouseGuarantyTableDTO houseGuarantyTableDTO) {
        String targetTableHtml = houseGuarantyTableDTO.getTargetTableHtml();
        String certificateTableHtml = houseGuarantyTableDTO.getCertificateTableHtml();
        String homePath = houseGuarantyTableDTO.getHomePath();
        Long excelId = houseGuarantyTableDTO.getExcelId();
        String tHtml = wordTableHelper.htmlTargetTableFormat(targetTableHtml);
        String cHtml = wordTableHelper.htmlCertificateTableFormat(certificateTableHtml);
        HouseGuarantyProcess hgp = new HouseGuarantyProcess();
        hgp.setExcelId(excelId);
        hgp.setHome(homePath);
        hgp.setTargetsHtml(targetTableHtml);
        hgp.setCertificatesHtml(cHtml);
        hgp.setProcessName(BusinessEnum.HouseGuarantyProcess.CONSIGNOR_LETTER.getMsg());
        houseGuarantyProcessService.save(hgp);
        wordTableHelper.createTableWord(tHtml, fileNetConfig.getBaseDir() + homePath,TARGETS_DOCX);
        wordTableHelper.createTableWord(cHtml, fileNetConfig.getBaseDir() + homePath,CERT_DOCX);

        return hgp.getId();
    }



    @Override
    public Boolean genConsignorLetter(Long excelId,Long processId) throws Exception {
        //从 HouseExcelData中获取正文内容
        BaseInfoDO baseDO = houseExcelDataService.getBaseDO(excelId);
        String baseDir = fileNetConfig.getBaseDir();
        try {
            byte[] tmplWordByte = Files.readAllBytes(Paths.get(baseDir
                    + fileNetConfig.getHouseGuarantyTemplatePath() + CONSIGNOR_LETTER_TEMPLATE));
            byte[] resultWordByte = AsposeWordUtil.fillWordDataByDomain(tmplWordByte, baseDO);
            String docHome = baseDO.getExcelUri();
            String consignorLetterName = docHome +"CONSIGNOR_LETTER_TEMPLATE"+ System.currentTimeMillis() + ".docx";
            File resultFile = new File(baseDir + consignorLetterName);
            FileOutputStream fos = new FileOutputStream(resultFile);
            fos.write(resultWordByte);
            fos.close();
            insetTable2Word(baseDir + consignorLetterName,baseDir+docHome + TARGETS_DOCX,"targetTable");
            return houseGuarantyProcessService.update(new LambdaUpdateWrapper<HouseGuarantyProcess>().set(HouseGuarantyProcess::getDocUrl, consignorLetterName)
                .eq(BaseEntity::getId, processId));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

    }



    @Override
    public String genConditionLetter(Long id) {
        DocumentProduction dp = documentProductionService.getById(id);
        HouseGuarantyProcess process = houseGuarantyProcessService.getOne(new LambdaQueryWrapper<HouseGuarantyProcess>().eq(HouseGuarantyProcess::getExcelId,dp.getBusinessId()).select(BaseEntity::getId,HouseGuarantyProcess::getHome));
        String baseDir = fileNetConfig.getBaseDir();

        try {
            ConditionDO conditionDO = houseExcelDataService.getConditionDO(dp.getBusinessId());

            byte[] tmplWordByte = Files.readAllBytes(Paths.get(baseDir
                    + fileNetConfig.getHouseGuarantyTemplatePath() + CONDITION_LETTER_TEMPLATE));

            byte[] resultWordByte = AsposeWordUtil.fillWordDataByDomain(tmplWordByte, conditionDO);

            String conditionLetterName = process.getHome() +"CONDITION_LETTER_TEMPLATE"+System.currentTimeMillis() + ".docx";
            File resultFile = new File(baseDir + conditionLetterName);
            FileOutputStream fos = new FileOutputStream(resultFile);
            fos.write(resultWordByte);
            fos.close();
            //更新过程文档url
            HouseGuarantyProcess houseGuarantyProcess = new HouseGuarantyProcess();
            houseGuarantyProcess.setProcessName(STATEMENT_CONDITIONS_EXPLAIN.getMsg());
            houseGuarantyProcess.setDocUrl(conditionLetterName);
            houseGuarantyProcess.setHome(process.getHome());
            houseGuarantyProcess.setParentId(process.getId());
            houseGuarantyProcessService.save(houseGuarantyProcess);
            return conditionLetterName;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public Map<String, String> getBaseInfoFromFile(Long id) throws IOException {
        HouseGuarantyProcess process = houseGuarantyProcessService.getById(id);
        String baseDir = fileNetConfig.getBaseDir();
        String home = baseDir + process.getHome();
        //读取基本信息
        XSSFWorkbook workbook = new XSSFWorkbook(home + COLLECT1_XLSX);
        return EasyExcelUtil.getExcelCellValue(workbook, BASE_INFO_ZH);
    }

    @Override
    public Boolean genResultLetter(Long id) {
        DocumentProduction dp = documentProductionService.getById(id);
        HouseGuarantyProcess process = houseGuarantyProcessService.getOne(new LambdaQueryWrapper<HouseGuarantyProcess>().eq(HouseGuarantyProcess::getExcelId,dp.getBusinessId()).select(BaseEntity::getId,HouseGuarantyProcess::getHome));
        String baseDir = fileNetConfig.getBaseDir();

        GuarantyResultDO guarantyResultDO = houseExcelDataService.getResultReport(dp.getBusinessId());

        try {
            byte[] tmplWordByte = Files.readAllBytes(Paths.get(baseDir
                    + fileNetConfig.getHouseGuarantyTemplatePath() + GUARANTY_RESULT_TEMPLATE));

            byte[] resultWordByte = AsposeWordUtil.fillWordDataByDomain(tmplWordByte, guarantyResultDO);

            String guarantyResultName = process.getHome() +"GUARANTY_RESULT_TEMPLATE"+System.currentTimeMillis() + ".docx";
            File resultFile = new File(baseDir + guarantyResultName);
            FileOutputStream fos = new FileOutputStream(resultFile);
            fos.write(resultWordByte);
            fos.close();

            //生成文档中的表格
//            AsposeWordUtil.createTableWidthStyle(doc, HouseEntityTable.class, entityTables, "buildingInfo");
//            doc.save(baseDir + guarantyResultName);
            //插入房屋权属状况
            insetTable2Word(baseDir + guarantyResultName,baseDir+guarantyResultDO.getExcelUri() + CERT_DOCX,"targetCertificateTable");
            //插入估价对象一览表
            insetTable2Word(baseDir + guarantyResultName,baseDir+guarantyResultDO.getExcelUri() + TARGETS_DOCX,"targetTable");

            HouseGuarantyProcess houseGuarantyProcess = new HouseGuarantyProcess();
            houseGuarantyProcess.setProcessName(GUARANTY_RESULT_REPORT.getMsg());
            houseGuarantyProcess.setDocUrl(guarantyResultName);
            houseGuarantyProcess.setHome(process.getHome());
            houseGuarantyProcess.setParentId(process.getId());
            houseGuarantyProcessService.save(houseGuarantyProcess);
            return true;
        } catch (FileNotFoundException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 在指定文档书签后插入表格
     * @param mainWordPath
     * @param tableWordPath
     * @throws Exception
     */
    private void insetTable2Word(String mainWordPath,String tableWordPath,String bookmark)  throws Exception{
        // 主word文件
        Document mainDoc = new Document(mainWordPath);

        // 需要插入的word文件
        Document subDoc = new Document(tableWordPath);

        mainDoc = AsposeWordUtil.insertDocumentAfterBookMark(mainDoc, subDoc, bookmark, false);
        // 保存文件
        mainDoc.save(mainWordPath);
    }


    private DocumentProduction buildDocumentProduction(Long id,String docName,String docNo,String consignor){
        DocumentProduction dp = new DocumentProduction();
        dp.setBusinessType(BusinessEnum.HOUSE_BUSINESS.name());
        dp.setBusinessCate(GUARANTY.getMsg());
        dp.setBusinessId(id);
        dp.setDocName(docName);
        dp.setDocType(DocumentType.REPORT.getName());
        dp.setDocNo(docNo);
        dp.setConsignor(consignor);
        return dp;
    }

    private List<String> getCertificateTypes(Long businessId){
        List<String> certificateTypes = new ArrayList<>();
        int imm = certificateFixedAssetsService.count(new LambdaQueryWrapper<CertificateFixedAssets>().eq(CertificateFixedAssets::getBusinessType, GUARANTY).eq(CertificateFixedAssets::getBusinessId, businessId));
        if (imm>0){
            certificateTypes.add(IMMOVABLE_CER);
        }
        int house = certificateHouseOwnService.count(new LambdaQueryWrapper<CertificateHouseOwn>().eq(CertificateHouseOwn::getBusinessType, GUARANTY).eq(CertificateHouseOwn::getBusinessId, businessId));
        if (house>0){
            certificateTypes.add(HOUSE_OWN_CER);
        }
        int land = certificateLandUseService.count(new LambdaQueryWrapper<CertificateLandUse>().eq(CertificateLandUse::getBusinessType, GUARANTY).eq(CertificateLandUse::getBusinessId, businessId));
        if (land>0){
            certificateTypes.add(LAND_USE_CER);
        }
        return certificateTypes;
    }

    /**
     * 不动产证 0
     * 房新&土新 1
     * 房新&土老 2
     * 房老&土新 3
     * 房老&土老 4
     * 房新&土无 5
     * 房老&土无 6
     * @param businessId
     * @param tid
     * @param tno
     * @return
     */
    @Override
    public Integer getCertificateCombination(Long businessId,String tid,String tno){
        int imm = certificateFixedAssetsService.count(new LambdaQueryWrapper<CertificateFixedAssets>()
                .eq(CertificateFixedAssets::getBusinessType, GUARANTY)
                .eq(CertificateFixedAssets::getBusinessId, businessId)
                .eq(CertificateFixedAssets::getTid,tid)
                .eq(CertificateFixedAssets::getTno,tno));
        if (imm == 0) {
            CertificateHouseOwn certHouseOwn = certificateHouseOwnService.getOne(new LambdaQueryWrapper<CertificateHouseOwn>()
                    .eq(CertificateHouseOwn::getBusinessType, GUARANTY)
                    .eq(CertificateHouseOwn::getBusinessId, businessId)
                    .eq(CertificateHouseOwn::getTid, tid)
                    .eq(CertificateHouseOwn::getTno, tno)
                    .select(CertificateHouseOwn::getIsNew));

            CertificateLandUse certLandUse = certificateLandUseService.getOne(new LambdaQueryWrapper<CertificateLandUse>()
                    .eq(CertificateLandUse::getBusinessType, GUARANTY)
                    .eq(CertificateLandUse::getBusinessId, businessId)
                    .eq(CertificateLandUse::getTid, tid)
                    .eq(CertificateLandUse::getTno, tno)
                    .select(CertificateLandUse::getIsNew));


            if (certHouseOwn != null && certLandUse != null) {
                if (certHouseOwn.getIsNew() && certLandUse.getIsNew()) {
                    return 1;
                } else if (certHouseOwn.getIsNew()) {
                    return 2;
                } else if (certLandUse.getIsNew()) {
                    return 3;
                } else {
                    return 4;
                }
            } else if (certHouseOwn != null) {
                if (certHouseOwn.getIsNew()) {
                    return 5;
                } else {
                    return 6;
                }
            } else {
                ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR, "缺失权属证件信息");
            }
        }
        return 0;
    }

    @Override
    public HouseGuarantyProcess getCertificatesHtml(Long businessId) {
        return houseGuarantyProcessService.getOne(new LambdaQueryWrapper<HouseGuarantyProcess>()
                .eq(HouseGuarantyProcess::getExcelId, businessId).select(HouseGuarantyProcess::getCertificatesHtml,BaseEntity::getId));
    }
}